@part[chapter2,root "thesis.mss"]

@chapter[Issues of Design and Implementation]

This chapter presents issues and problems involved with the design and
implementation of the Mercury library system.  The first section,
"Design Issues", describes the transition from the purpose and
original conception of this system (described in the last chapter) to
an overall design of an ARGUS program realizing this purpose.  The second
section, "Implementation Issues", explains problems encountered in the
implementation of this design, and issues which were unexpected.  It
then presents the solutions to these problems used in the
implementation.

@section[Design Issues]

@subsection[Realization of Purpose]

In order to understand the transformation of the purpose described in
the last chapter into a
program realizing this purpose, it is useful to examine how the
individual facets of this purpose group together, and how they will
best be implemented in a program.  Since storage and recovery of
data, as well as information on the interrelations of the Mercury data
types, is a major part of the library, a good basic design is some
sort of database.  This will handle three of the five basic elements of
the library described in the last section.  A database allows for
access to information stored within, providing an easy way to view the
ports and their component variable types.  Additionally, by allowing
different libraries to access each other via a referencing system, the
easy access of other users' ports and/or a central library is provided.
Finally, the facility for a frontend to be written in any
Mercury-capable language is encompassed in a simple, yet
complete interface which will allow a frontend written in any
Mercury-capable language to access the library system.  This interface
should provide every command which would reasonably be required, with
individual frontends formatting the output, and only using the
commands relevant to their users.

The other two other design elements, translation between blue and
Mercury-capable languages and easy entry of modules, represent
the parsing and translating of Mercury data types.  This part of the
library should be able to scan a file written in blue, ARGUS, C, or
LISP, and present this information to the database in a form allowing
for rapid and simple storage, retrieval, and comparison.  Additionally,
when outputting information, there must be a mechanism for translating
to any of those languages.  This part of the library represents a
series of parsers and translators, able to take a file written in any
language and translate that to any other language, or to the format
used for storage.

These two elements, database and parsers,
represent the basic design for the Mercury library system.  Since
ARGUS is an effective language for working with databases, and the
Mercury system is being written in ARGUS, the library system is also
written in ARGUS.  The next sections describes how the two components
of the system, database and parsers, are transformed
into a collection of procedures, clusters, and guardians.

@subsection[ARGUS]

ARGUS is a distributed language in an advanced portion of its
development at the Laboratory for Computer Science.  It has various features
that make it a very useful and effective language, although only
those relevant to the library system's design and effectiveness are
discussed here (For more information, see @cite[argus] and
@cite[manual]).

ARGUS is based on the CLU language, having the same advantages.  It
supports the construction of well-structured programs through abstraction
mechanisms and by being object-oriented (so programs are naturally thought
of as operating on potentially long-lived objects) @cite[argus].
Two specific advantages of ARGUS over CLU are distribution and atomicity.

@i[Guardians] represent the distributed aspect of ARGUS.  They are
abstractions, similar to clusters in CLU, but they are also much more
advanced.  A guardian generally encapsulates a resource, in the
current case, a library system.  @i[Handlers] are provided by the
guardian in order to make these resources available to the outside
world.  Internally, guardians have data objects and processes.  The
data objects represent either the state of the guardian, or data local
to individual internal processes.  A process is initiated by each
handler call, in addition to an optional background process.

Atomicity allows for concurrent and reliable manipulation and
retrieval of data.  An atomic action has two properties:
indivisibility and recoverability.  Indivisibility requires that the
execution of one action never appears to overlap the execution of
any other actions.  Recoverability requires that the overall effect of
an action is either completion or nothing.  If anything goes wrong
within an action, either that action will run to completion, or the
original state before the action began will be restored.  In a database
setting such as the library system, these features allow multiple
people to simultaneously utilize, and even alter, an individual
library freely.  These features of ARGUS make it an effective
language for the library system.

@subsection[System Design]

Figure 2-1 shows the modular design of the library system.
Viewing the system from the top down, the @b[hglib] procedure comes
first.  This is simply the start-up procedure for the library system
(as required by ARGUS programs), but it also has an extremely simple
command interface to the library system, usable for testing and as an
interface until a more advanced frontend is designed.  The @b[merc]
guardian comes next.  This guardian is the frontend of the library
system.  It receives commands from the user, and either performs the
requested operation, or sends the command on to its proper place.
Operations involving libraries (opening, closing, merging, creating,
etc) are performed within this guardian, while operations involving
entry or translation of specs are sent on to the @b[language] cluster.
Commands sent to the language cluster are routed to the appropriate
individual language cluster, @b[blue] or @b[argus].  The @b[lisp] and
@b[c] clusters are currently unimplemented.  The @b[library] cluster
is the actual database, holding information on all modules in the
library.  Information is stored in objects called @b[blues] (objects of
the type defined in the cluster @b[blue], as opposed to the language
blue).  Those procedures and objects of some design importance
will be described in this section.
Appendix D contains the code for the entire library system.

@begin[figure]
@blankspace[4 inches]
@caption[Modular design of the library system]
@end[figure]

The @b[hglib] procedure creates a new @b[merc] guardian, and allows
for entry of commands in the basic format described in the next
chapter (see appendix B for a command summary).  Upon use of the quit
command, the program terminates, but the @b[merc] guardian remains in
existence.

The @b[merc] guardian has a variety of objects and commands enabling
it to run the entire library system.  Each active user of the library
system has a set of data objects for his or her library session.
There are six important data objects for each user:
the boolean @i[exists] shows whether there is a library
currently chosen for operation, the string @i[fname] contains the
filename of the current library, @i[lib] is the current @b[library],
@i[references] is an array of strings listing libraries references by
the current library, @i[data] is a string representing the value
of the data field (described in section 3.3, The Data Field), and
@i[password] is a unique string representing the user.
@i[Exists] prevents the use of
obsolete or uninitialized values for @i[fname], @i[lib], or
@i[references], as these values are only referenced if
@i[exists] = true.  The @b[merc] guardian also has three handlers:
@b[suicide], which destroys the guardian itself; @b[access], which
creates a password and variable space for a new user; and @b[act], which
accepts a string as input, performs the command represented
therein, and returns an array of strings representing the result of
that operation.  The @b[act] handler is the interface provided for
Mercury operation, accepting and performing commands.  The @b[access]
handler is also available through Mercury.

The @b[library] cluster is the database containing the current
library.  It contains all the modules defined in a library and
provides a set of procedures for altering and viewing those modules.
Due to the large number of modules which may be stored in a library,
and the different types of modules, this cluster contains six hash
tables.  Each hash table has a specific type of module, hashed either
alphabetically or by type.  The lists are called @i[groups, names,
types, subs, refs,] and @i[primitives].  @i[Groups] contains only
group specs, which are not elements of the blue language.  They
provide a method of organizing actual modules (generally ports) into
groups of modules, "setting aside" those which are interrelated in
some way.  The other five lists organize the actual type specs for
maximum efficiency in access, storage, and retrieval of data.

@i[Primitives] contains the primitive blue types in a table hashed by
name.  These types get their own list since they are extremely common,
and also since they exist in the same form in every library, as they
are pre-defined blue types.  @i[Refs] contains all types defined in
referenced libraries, hashed by name.  References remove the need for
redefining types contained in other libraries.  @i[Subs] contains
subtypes hashed by type, since their names are sequentially generated,
and not responsive to the name-hashing function.  Subtypes are those
modules that are used by other modules, but not defined as actual
modules themselves.  Since blue does not provide for nested
definitions and other languages do, such a mechanism is necessary when
translating from a nested definition in one language to blue.

The other two lists contain the same information, but organized in
different forms.  The actual user-defined modules are hashed by name
in @i[names], and by type in @i[types].  Since this information is
most important to the user, it is organized in this way to be most
accessible.  Additionally, since ARGUS uses pointers in arrays, very
little additional space is required for two lists, as opposed to one.
The procedures available to the @b[library] cluster allow additions or
deletions of modules from specific lists (except primitives, which
cannot be altered), or removal of a module from wherever it is.  There
are also iterators for those lists where they are useful and some
procedures to locate specific modules, or determine whether those
modules are in the library.

The remainder of the program provides mechanisms for parsing,
translation, and comparison of modules, as well as the internal
representation of modules.  The cluster @b[language] exists to allow
for easy addition of other languages to the library; the only change
required to add C, LISP, or libby (ARGUS library system) is a minor
modification to this cluster, as opposed to altering the merc
guardian.

Each language has a cluster with two operations available externally,
@b[parse] and @b[unparse].  The @b[parse] operation scans in the
relevant type information from a file (or parses the data field,
depending on the command).  These types are then converted into
modules and added to the library.  Any subtypes, including fields and
named types, are defined as separate modules.  Figure 2-2
gives an example of the representation of a nested named type.  The
@b[unparse] operation takes a module, and returns the representation
of that type in the given language.

@begin[figure]
@blankspace[3.5 inches]
@begin[quotation]
(defmtype inner (SEQUENCE (BYTE))@*

(defmtype outer inner)
@end[quotation]
@caption[Representation of nested types]
@end[figure]

The internal representation of data is defined by the cluster
@b[blue], and the equate @i[typ].  While the @b[blue] cluster does
provide @b[parse] and @b[unparse] procedures, it has an independent
set of procedures for comparing,  mutating and viewing modules.
A @b[blue] object, representing an individual module, also contains
a variety of information to define and individualize the type
represented.  Two identifiers are provided, one based on the type, the
other based on the time of creation.  A set of notes containing the
language the type was originally defined in, as well as fields for the
creator and any notes which can be entered later.  In the case of a
parameterized type, a list of parameter values is provided.  Finally,
there is a @i[typ] and a list of parents (pointers to all other types
which use the type in question).  The @i[typ] is a oneof which can
represent any type available to blue. Types with fields (such as
sequence, union, oneof, etc.) also contain information describing
those fields (a pointer to the field type, and a field name if
relevant).  There are two other types; group and undefined.  Groups,
as already described, allow for user-defined grouping of modules under
a specific name.  The
undefined type is used when adding information to the library, or when
the user specifically defines an unspecified type.  This design is
intended to create a tree structure where each node (type) possesses
information about itself, as well as pointers to those types it uses
and pointers to those types which use it. 

@section[Implementation Issues]

In implementing the library, there were many issues which had
to be considered.  This section discusses these issues: identifiers (ids),
circular definitions, type mutation, parameterized types, and parsing.

Each blue has a type id (tid)
representing its own type, and the tids of its subtypes, with
primitives having a single character as their tid@footnote[Based on a
byte-encoding scheme using self-encoding integers designed by Rob Austein].
When mutating a
blue, therefore, all of its ancestors must have their tids changed to
represent this.  And in the case of a circular definition, the tid will
be infinite, and mutation of circular types without any special method
of handling them will be a catastrophe.

To handle general mutation of blues, the previously mentioned pointers
to parents (types using the given type) exist.  Therefore, when any
blue is mutated, a procedure is
called which informs all of its ancestors to re-create their tids, by
first acting on parents, and then parents' parents, etc.  Then,
circular definitions were implemented as a special case in many ways;
mutation, tids, adding, and deleting.  Adding and deleting simply
required dummy definitions to be created while defining the circular
type.  Mutation and tids were handled differently.  In forming tids, if
the original type is encountered, rather than recursively inserting
its own tid, a special "circular token" is inserted in its place.  This
does not affect type equality in any way, but effectively keeps the
circular information.  When mutating blues, a list of those which have
been mutated is always kept.  If any blues are encountered a second
time, no command to re-id is given, and that blue's parents are
ignored.  Therefore, regardless of whether a definition is circular or
not, mutation will re-id all parents only once.

Abstract types also required some consideration with respect to the handling of
parameters.  When defining a parameterized type, the parameters are
represented as a field (similar to a field of a record).  And when
defining another type using the parameterized definition, the values
for these fields are set in the parameter list, part of each blue
definition.  For example, figure 2-3 demonstrates how
parameterized types look internally.

@begin[figure]
@blankspace[4 inches]
@begin[quotation]
(defabstype first parm1 parm2 (RECORD (field1 parm1) (field2 INT32)@*
@ @ @ @ @ @ @ @ @ @ @ @ @ @ @ @ @ @ @ @ @ @ @ (field3 parm2) (field4 parm1)))@*

(defabstype second first (STRING BOOL))
@end[quotation]
@caption[Internal representation of parameterized types]
@end[figure]

The ARGUS and blue parsers represent a large portion of the library
system's code.  Parsing is performed top-down.  A blue file contains
only type definitions, allowing relatively simple parsing.  ARGUS
files are more difficult, as they contain a variety of unimportant
code, as well as scoping.  Each procedure, guardian, and cluster is
handled individually, and most of the code is passed over and
ignored.  Relevant elements of the code, such as handler headers and
type definitions, are scanned and converted into @b[blue] objects.
