@Part(types, Root "manual.mss")

@Chapter(Types, Type Generators, and Type Specifications)

A @i(type) consists of a set of objects together with
a set of operations used to manipulate the objects.
@index(type)
Types can be classified according to whether their objects are mutable
or immutable, and atomic or non-atomic.
An @i(immutable) object (e.g., an integer) has a value that never varies,
while the value (state) of a @i(mutable) object can vary over time.
@index(object)@IndexSecondary(Primary="object", Secondary="mutable")
@IndexSecondary(Primary="object", Secondary="immutable")
@index(mutable)@index(immutable)
Objects of @i(atomic) types provide serializability and recovery for accessing
actions.
@IndexSecondary(Primary="object", Secondary="atomic")
@IndexSecondary(Primary="object", Secondary="non-atomic")
@i(Non-atomic) types may provide
synchronization by specifying that particular operations are executed
@i(indivisibly) on objects of the type.
@index(indivisible)@IndexSecondary(Primary="operation", Secondary="indivisibility")
An operation is indivisible if no other process may affect or observe
intermediate states of the operation's execution.
Indivisibility properties will be described for all
the built-in non-atomic types of Argus.

A @i(type generator) is a @i(parameterized) type definition,
@index(generator)@IndexSecondary(Primary="type", Secondary="generator")
@index(parameterized type)@IndexSecondary(Primary="type", Secondary="parameterized")
representing a (usually infinite) set of related types.
A particular type is obtained from a type generator by writing
the generator name along with specific values for the parameters;
for every distinct set of legal values,
a distinct type is obtained (see Section @ref(sect-instantiations)).
For example,
the @b[array] type generator has a single parameter that determines the element type;
@b[array][@b[int]], @b[array][@b[real]], and @b[array][@b[array][@b[int]]] are three distinct
types defined by the @b[array] type generator.
Types obtained from type generators are called @i(parameterized) types or
@i(instantiations) of the type generator;
others are called @i(simple) types.

In Argus code,
a type is specified by a syntactic construct called a @i(type@us<>spec).
@IndexEntry(Key="type_spec", Entry="type@us<>spec", Number)
The type specification for a simple type
is just the identifier (or reserved word) naming the type.
For parameterized types,
the type specification consists of the identifier (or reserved word) naming
the type generator, together with the actual parameter values.

To be used as arguments or results of handler and creator calls,
or as @b[image] objects (see Section @ref(sect-image)),
objects must be @i(transmissible).
@index(image)@index(transmissible)
@IndexSecondary(Primary="object", Secondary="transmissible")
Most of the built-in Argus types are transmissible,
that is, they have transmissible objects.
@IndexSecondary(Primary="type", Secondary="transmissible")
However, procedures and iterators are never transmissible.
For type generators,
transmissibility of a particular instantiation of the generator
may depend upon transmissibility of any type parameters.
A transmissible type provides the pseudo-operation @b[transmit]
@index(transmit)
and two internal operations @i(encode) and @i(decode).
@index(encode)@index(decode)
Generally, @i(encode) and @i(decode) are hidden from clients of the type.
They are called implicitly during message transmission
(see Section @ref(sect-transmit)) and in creating and decomposing @b[image] objects
(see Section @ref(sect-image)).
Transmissibility is discussed further in Section @ref(sect-transmit).

Argus provides all the built-in types of CLU@index(CLU) as well as some
new types and type generators.
This section gives an informal introduction to the built-in
types and type generators provided by Argus.
Many details are not discussed here, but
a complete definition of
each type and type generator is given in Appendix @ref(apdx-types).

@Section(Type Inclusion)
@Label(sect-tinclusion)@Label(sect-typeincl)@Label(sect-typecheck)
@Label(sect-type-incl)

The notion of @i(type inclusion) in Argus is different from that in CLU.
@index(type inclusion)@IndexSecondary(Primary="type", Secondary="inclusion")
The type @b[any] is a type like every other type,
@index(any)
and there is no implicit coercion to type @b[any],
so there is no need to make a special case for it in the type inclusion rule.
Type inclusion in Argus is the same as type equality
(see Section @ref(sect-instantiations)),
except for procedure, iterator, handler, and creator types.
A routine type @i(O) is included in another routine type @i(V),
when the number and types of arguments, and the number and types of normal
results, are equal, and for each exception in @i(O) there is a corresponding
exception in @i(V) of the same name with the same number and types of results.
Note that @i(V) may have more exceptions than does @i(O),
and that this rule is not recursive, that is, when comparing types
of arguments and results, type equality is used.
For example, if we have the following declarations in effect:
@begin[Display]
p : @b[proctype](@b[real], @b[real]) @b[returns](@b[real]) @b[signals](overflow, underflow)
q : @b[proctype](@b[real], @b[real]) @b[returns](@b[real])
@end[Display]
then the type of @i(q) is included in the type of @i(p) but not @i(vice versa).
Thus the assignment @i(p := q) is legal.

@Section(The Sequential Built-in Types and Type-generators)

In this section, we introduce
the sequential built-in types of Argus.  These types are generally the same as
types in CLU.
@IndexSecondary(Primary="CLU", Secondary="built-in types taken from")
@IndexSecondary(Primary="built-in", Secondary="type")
@IndexSecondary(Primary="type", Secondary="built-in")
This section concentrates on their new characteristics.

Recovery from aborted actions is trivial for immutable objects,
since the aborted actions cannot have modified these objects.
In particular the built-in scalar types
@b[null], @b[bool],
@b[int], @b[real], @b[char], and @b[string] are immutable, atomic, and transmissible.
The built-in mutable type generators inherited from CLU are not atomic.
@IndexSecondary(Primary="mutable", Secondary="versus atomic")


@SubSection(Null)

The type @b[null] has exactly one immutable object,
represented by the literal @b[nil],
which is atomic and transmissible.
@index(null)@index(nil)
See Section @ref(null-type-def) for details.

@SubSection(Bool)

The two immutable objects of type @b[bool], with literals @b[true] and @b[false],
represent logical truth values.
The binary operations @i(equal)@ (=), @i(and)@ (&), and @i(or)@ (|),
are provided, as well as unary @i(not)@ (~).
Objects of type @b[bool] are atomic and transmissible.
@index(bool)@index(true)@index(false)
See Section @ref(bool-type-def) for details.

@SubSection(Int)

The type @b[int] models (a range of) the mathematical integers.
The exact range is not part of the language definition@Foot{
However, implementations are encouraged to provide this and other
information about the limits of the built-in types in an equate module.
}.
@index(int)
Integers are immutable, atomic, transmissible,
and their literals are written as a sequence of one or more decimal digits.
(There are also octal and hexadecimal literals, see Appendix @ref(apdx-syntax).)
The binary operations
@i(add)@ (+), @i(sub)@ (@MinusSign()), @i(mul)@ (*), @i(div)@ (/),
@i(mod)@ (//), @i(power)@ (**), @i(max), and @i(min)
are provided, as well as unary @i(minus)@ (@MinusSign()) and @i(abs).
There are binary comparison operations
@i(lt)@ (<), @i(le)@ (<=), @i(equal)@ (=), @i(ge)@ (>=), and @i(gt)@ (>).
There are two operations, @i(from@us<>to) and @i(from@us<>to@us<>by),
for iterating over a range of integers.  See Section @ref(int-type-def) for
details.


@SubSection(Real)

The type @b[real] models (a subset of) the mathematical real numbers.
@index(real)@SeeAlso(Primary="floating point", Other="real")
The exact subset is not part of the language definition.
Reals are immutable, atomic, and transmissible,
although transmission of @b[real] objects between heterogeneous machine
architectures
may not be exact.
Real literals are written as a @i(mantissa) with an optional @i(exponent).
A mantissa is either a sequence of one or more decimal digits,
or two sequences (one of which may be empty) joined by a period.
The mantissa must contain at least one digit.
An exponent is 'E' or 'e',
optionally followed by '+' or '@MinusSign()',
followed by one or more decimal digits.
An exponent is required if the mantissa does not contain a period.
As is usual, @i(m)E@i(x) = @i(m)*10@+(@i(x)).
Examples of real literals are:
@begin[Display]
3.14        3.14E0        314e@MinusSign()2        .0314E+2        3.        .14
@end[Display]

As with integers,
the operations
@i(add)@ (+), @i(sub)@ (@MinusSign()), @i(mul)@ (*), @i(div)@ (/),
@i(mod)@ (//), @i(power)@ (**), @i(max), @i(min),
@i(minus)@ (@MinusSign()), @i(abs), @i(lt)@ (<), @i(le)@ (<=),
@i(equal)@ (=), @i(ge)@ (>=), and @i(gt)@ (>),
are provided.
It is important to note that there is no form of
@i(implicit) conversion between types.
The @i(i2r) operation converts an integer to a real,
@i(r2i) rounds a real to an integer,
and @i(trunc) truncates a real to an integer.
See Section @ref(real-type-def) for details.

@SubSection(Char)
@Label(sect-char)

The type @b[char] provides the alphabet for text manipulation.
Characters are immutable, atomic, transmissible, and form an ordered set.
Every implementation must provide at least 128, but no more than 512, characters;
the first 128 characters are the ASCII characters in their standard order.
@index(char)

Literals for the printing ASCII characters (octal 40 through octal 176),
other than single quote (') or backslash (\),
@IndexEntry(Key=" '", Entry="'", Number)
@IndexEntry(Key=" \", Entry="\", Number)
can be written as that character enclosed in single quotes.
Any character can be written by enclosing one of the
escape sequences listed in Table @ref(fig-character-escapes) in single quotes.
@IndexSecondary(Primary="char", Secondary="escapes")
@begin(Table)
@Caption(Character Escape Sequence Forms)
@tag(fig-character-escapes)
@bar()
@begin[Display]
@TabClear()
@u(escape sequence)        @^@u(character)
@BlankSpace(.5 lines)
\'@\'     @^(single quote)
\"@\"@\(double quote)
\\@\\@\(backslash)
\n@\NL@\(newline)
\t@\HT@\(horizontal tab)
\p@\FF@\(form feed, newpage)
\b@\BS@\(backspace)
\r@\CR@\(carriage return)
\v@\VT@\(vertical tab)
\***@\specified by octal value (exactly three octal digits)
\#**@\specified by hexadecimal value (exactly two hex digits)
\^*@\characters numbered 0-31 (with * a printing character)
\!*@\characters numbered 128-159 (with * a printing character)
\&*@\characters numbered 160-255 (with * a printing character)
@end[Display]
@bar()
@end(Table)
The escape sequences may be written using upper case letters,
but note that escape sequences of the form \&* are case sensitive.
A table of literals is given at the end of Appendix @ref(apdx-syntax).
Examples of character literals are:
@begin[Display]
\7'        'a'        '"'        '\"'        '\''        '\B'        '\177'
@end[Display]

There are two operations, @i(i2c) and @i(c2i),
for converting between integers and characters:
the smallest character corresponds to zero,
and the characters are numbered sequentially.
Binary comparison operations exist for characters
based on this numerical ordering:
@i(lt)@ (<), @i(le)@ (<=), @i(equal)@ (=), @i(ge)@ (>=), and @i(gt)@ (>).
For details, see Section @ref(char-type-def).

@SubSection(String)

The type @b[string] is used for representing text.
@index(string)@SeeAlso(Primary="string", Other="char escapes")
A string is an immutable, atomic, and transmissible
sequence of zero or more characters.
Strings are lexicographically ordered, based on the ordering for characters.
A string literal is written
as a sequence of zero or more characters or character escape sequences 
(see Table @ref(fig-character-escapes)),
enclosed in double quotes (").
@IndexEntry(Key=< ">, Entry=<">, Number)

The characters of a string are indexed sequentially starting from one.
The @i(fetch) operation is used to obtain a character by index.
The @i(substr) operation is used to obtain a substring.
The tail of a string can be gotten by using @i(rest).
Searching in strings is provided by the @i(indexc) and @i(indexs) operations.

Two strings can be concatenated together with @i(concat)@ (||),
and a single character can be appended to the end of a string with @i(append).
@i(C2s) converts a character to a single-character string.
The size of a string can be determined with @i(size).
@i(Chars) iterates over the characters of a string,
from the first to the last character.
There are also the usual lexicographic comparison operations:
@i(lt)@ (<), @i(le)@ (<=), @i(equal)@ (=), @i(ge)@ (>=), and @i(gt)@ (>).
For details, see Section @ref(string-type-def).

@SubSection(Any)
@Label(sect-any)

Objects of type @b[any] may contain objects of any type,
and thus provide an escape from compile-time type checking.
@index(any)
Unlike CLU@index(CLU),
which treats @b[any] differently from all other types,
@b[any] is a normal type in Argus.
To this end there is an explicit @i(create) operation generator,
and the @i(force) procedure is also an operation generator of type @b[any].

An object of type @b[any] can be thought of as containing an object and its type.
Since there are no operations provided by type @b[any] that change this state,
@b[any] objects can be considered to be immutable.
However, the state of the contained object may change
if that object is shared,
so from this point of view, the mutability and atomicity of an @b[any] object
depend on the mutability and atomicity of the contained object.
Objects of type @b[any] are not transmissible.

The @i(create) operation is parameterized by a type;
@i(create) takes a single argument of that type
and returns an @b[any] object containing the argument.
The @i(force) operation is also parameterized by a type;
@SeeAlso(Primary="force", Other="any")
it takes an @b[any] and extracts an object of that type,
signalling @i(wrong@us<>type) if the contained object's type
is not included in the parameter type.
The @i(is@us<>type) operation is parameterized by a type and checks whether
its argument contains an object whose type is included in the parameter type.
The detailed specification is found in Section @ref(any-type-def).

@SubSection(Sequence Types)

Sequences are immutable and they are atomic or transmissible when instantiated
with atomic or transmissible type parameters.
@index(sequence)
Although an individual sequence can have any length,
the length and members of a sequence are fixed when the sequence is created.
The elements of a sequence are indexed sequentially,
starting from one.
A sequence type specification has the form:
@begin[Display]
@b[sequence] [ type@us<>actual ]
@end[Display]
where a @i(type@us<>actual) is a @i(type@us<>spec), possibly augmented
with operation bindings (see Section @ref(sect-instantiations)).

The @i(new) operation returns an empty sequence.
A sequence constructor has the form:
@label(sect-sequence-constructor)
@IndexSecondary(Primary="sequence", Secondary="constructor")
@IndexSecondary(Primary="constructor", Secondary="sequence")
@begin[SyntaxDisplay]
type@us<>spec $ [ @BNFlbkt() expression , @BNFdots() @BNFrbkt() ]
@end[SyntaxDisplay]
and can be used to create a sequence with the given elements.

Although a sequence,
once created,
cannot be changed,
new sequences can be constructed from existing ones by means of the
@i(addh), @i(addl), @i(remh), and @i(reml) operations.
Other operations include @i(fetch), @i(replace), @i(top), @i(bottom),
@i(size), the @i(elements) and @i(indexes) iterators, and @i(subseq).
Invocations of the @i(fetch) operation can be written using a special
form:
@begin[Display]
q[i]			% fetch the element at index i of q  .
@end[Display]

Two sequences with equal elements are equal.
The @i(equal)@ (=) operation tests if two sequences have equal elements,
using the @i(equal) operation of the element type.
@i(Similar) tests if two sequences have similar elements,
using the @i(similar) operation of the element type.

All operations are indivisible except for
@i(fill@us<>copy), @i(equal), @i(similar), @i(copy), @i(encode), and @i(decode),
which are divisible at calls to the operations of the type parameter.

For the detailed specification, see Section @ref(sequence-type-def).

@SubSection(Array Types)

Arrays are one-dimensional, and mutable but not atomic.
@index(array)
They are transmissible only if their type parameter is transmissible.
The number of elements in an array can vary dynamically.
There is no notion of an "uninitialized" element.

The @i(state) of an array consists of an integer called the @i(low bound),
and a sequence of objects called the @i(elements).
The elements of an array are indexed sequentially,
starting from the low bound.
All of the elements must be of the same type;
this type is specified in the array type specification,
which has the form:
@begin[Display]
@b[array] [ type@us<>actual ]
@end[Display]

There are a number of ways to create a new array,
of which only two are mentioned here.
The @i(create) operation takes an argument specifying the low bound,
and creates a new array with that low bound and no elements.
Alternately, an array constructor can be used to create an array
with an arbitrary number of initial elements.
@label(sect-array-constructor)
@IndexSecondary(Primary="array", Secondary="constructor")
@IndexSecondary(Primary="constructor", Secondary="array")
For example,
@begin[Display]
@b[array][@b[int]] $ [5: 1, 2, 3, 4]
@end[Display]
creates an integer array with low bound 5, and four elements, while
@begin[Display]
@b[array][@b[bool]] $ [@b[true], @b[false]]
@end[Display]
creates a boolean array with low bound 1 (the default), and two elements.

An array type specification states nothing about the bounds of an array.
This is because arrays can grow and shrink dynamically, using the
@i(addh), @i(addl), @i(remh), and @i(reml) operations.
Other operations include @i(fetch),
@i(store), @i(top), @i(bottom), @i(high), @i(low),
the @i(elements) and @i(indexes) iterators, and @i(size).
Invocations of @i(fetch) and @i(store)
@IndexEntry(Key=" []", Entry="[]", Number)
can be written using special forms:
@begin[Display]
@TabClear()
a[i]		    @^% fetch the element at index i of a
a[i] := 3@\% store 3 at index i of a (by calling @i(store))
@end[Display]

Every newly created array has an identity that is distinct from all other arrays;
two arrays can have the same elements without being the same array object.
The identity of arrays can be distinguished with the @i(equal)@ (=) operation.
The @i(similar1) operation tests if two arrays have the same state,
using the @i(equal) operation of the element type.
@i(Similar) tests if two arrays have similar states,
using the @i(similar) operation of the element type.

All operations are indivisible, except @i(fill@us<>copy), @i(similar), @i(similar1), @i(copy),
@i(encode), and @i(decode), which are divisible at calls to operations of
the type parameter.

For the detailed specification, see Section @ref(array-type-def).

@SubSection(Structure Types)

A structure is an immutable collection of one or more named objects.
@index(struct)@SeeAlso(Primary="structure", Other="struct")
An instantiation is atomic or transmissible only if the type parameters
are all atomic or all transmissible.
The names are called @i(selectors),
and the objects are called @i(components).
Different components may have different types.
A structure type specification has the form:
@begin[SyntaxDisplay]
@b[struct] [ field@us<>spec , @BNFdots() ]
@end[SyntaxDisplay]
where
@begin[SyntaxDisplay]
@TabClear()
field@us<>spec @^@BNFequals() name , @BNFdots() : type@us<>actual
@end[SyntaxDisplay]
Selectors must be unique within a specification,
but the ordering and grouping of selectors is unimportant.

A structure is created using a structure constructor.
@label(sect-struct-constructor)
@IndexSecondary(Primary="struct", Secondary="constructor")
@IndexSecondary(Primary="constructor", Secondary="struct")
For example,
assuming that "info" has been equated to a structure type:
@begin[Display]
info = @b[struct][last, first, middle: @b[string],    age: @b[int]]
@end[Display]
the following is a legal structure constructor:
@begin[Display]
info $ {last: "Scheifler",    first: "Robert",    age: 32,    middle: "W."}
@end[Display]
An expression must be given for each selector,
but the order and grouping of selectors need not resemble
the corresponding type specification.

For each selector "sel",
there is an operation @i(get@us<>sel) to extract the named component,
and an operation @i(replace@us<>sel) to create a new structure with
the named component replaced with some other object.
Invocations of the @i(get) operations
@IndexEntry(Key=" .", Entry=".", Number)
can be written using a special form:
@begin[Display]
st.age			% get the 'age' component of st
@end[Display]

As with sequences,
two structures with equal components are in fact the same object.
The @i(equal)@ (=) operation tests if two structures have equal components,
using the @i(equal) operations of the component types.
@i(Similar) tests if two structures have similar components,
using the @i(similar) operations of the component types.

All operations are indivisible except for
@i(equal), @i(similar), @i(copy), @i(encode), and @i(decode),
which are divisible at calls to the operations of the type parameter.

For the detailed specification, see Section @ref(struct-type-def).

@SubSection(Record Types)

A record is a mutable collection of one or more named objects.
Records are never atomic, and are transmissible only if the parameter
types are all transmissible.
A record type specification has the form:
@begin[SyntaxDisplay]
@b[record] [ field@us<>spec , @BNFdots() ]
@end[SyntaxDisplay]
where (as for structures)
@begin[SyntaxDisplay]
@TabClear()
field@us<>spec @^@BNFequals() name , @BNFdots() : type@us<>actual
@end[SyntaxDisplay]
Selectors must be unique within a specification,
but the ordering and grouping of selectors is unimportant.

A record is created using a record constructor.
@label(sect-record-constructor)
@IndexSecondary(Primary="record", Secondary="constructor")
@IndexSecondary(Primary="constructor", Secondary="record")
For example:
@begin[Display]
professor $ {last: "Herlihy",    first: "Maurice",    age: 32,    middle: "P."}
@end[Display]

For each selector "sel",
there is an operation @i(get@us<>sel) to extract the named component,
and an operation @i(set@us<>sel) to replace the named component with
some other object.
@IndexEntry(Key=" .", Entry=".", Number)
Invocations of these operations can be written using a special form:
@begin[Display]
@TabClear()
r.middle	   @^% get the 'middle' component of r
r.age := 33@\% set the 'age' component of r to 33 (by calling @i(set@us<>age))
@end[Display]

As with arrays,
every newly created record has an identity that is distinct from all other records;
two records can have the same components without being the same record object.
The identity of records can be distinguished with the @i(equal)@ (=) operation.
The @i(similar1) operation tests if two records have equal components,
using the @i(equal) operations of the component types.
@i(Similar) tests if two records have similar components,
using the @i(similar) operations of the component types.

All operations are indivisible, except @i(similar), @i(similar1), @i(copy),
@i(encode), and @i(decode), which are divisible at calls to operations of
the type parameters.

For the detailed specification, see Section @ref(record-type-def).

@SubSection(Oneof Types)

A oneof type is a @i(tagged, discriminated union).
A oneof is an immutable labeled object,
to be thought of as "one of" a set of alternatives.
The label is called the @i(tag),
and the object is called the @i(value).
A oneof type specification has the form:
@begin[SyntaxDisplay]
@b[oneof] [ field@us<>spec , @BNFdots() ]
@end[SyntaxDisplay]
where (as for structures)
@begin[SyntaxDisplay]
@TabClear()
field@us<>spec @^@BNFequals() name , @BNFdots() : type@us<>actual
@end[SyntaxDisplay]
Tags must be unique within a specification,
but the ordering and grouping of tags is unimportant.
An instantiation is atomic or transmissible if and only if
all the type parameters are atomic or transmissible.

For each tag "t" of a oneof type,
there is a @i(make@us<>)t operation which takes an object
of the type associated with the tag,
and returns the object (as a oneof) labeled with tag "t".

To determine the tag and value of a oneof object,
one normally uses the @b[tagcase] statement (see Section @ref(sect-tagcase)).

The @i(equal)@ (=) operation tests if two oneofs have the same tag,
and if so, tests if the two value components are equal,
using the @i(equal) operation of the value type.
@i(Similar) tests if two oneofs have the same tag,
and if so, tests if the two value components are similar,
using the @i(similar) operation of the value type.

All operations are indivisible, except @i(equal),
@i(similar), @i(similar1), @i(copy),
@i(encode), and @i(decode), which are divisible at calls to operations of
the type parameters.

For the detailed specification, see Section @ref(oneof-type-def).

@SubSection(Variant Types)

A variant is a mutable oneof.  Variants are never atomic
and are transmissible if and only
if their type parameters are all transmissible.
A variant type specification has the form:
@begin[SyntaxDisplay]
@b[variant] [ field@us<>spec , @BNFdots() ]
@end[SyntaxDisplay]
where (as for oneofs)
@begin[SyntaxDisplay]
@TabClear()
field@us<>spec @^@BNFequals() name , @BNFdots() : type@us<>actual
@end[SyntaxDisplay]
The state of a variant is a pair consisting of a label called the @i(tag)
and an object called the @i(value).
For each tag "t" of a variant type,
there is a @i(make@us<>)t operation which takes an object of the type
associated with the tag,
and returns the object (as a variant) labeled with tag "t".
In addition,
there is a @i(change@us<>)t operation,
which takes an existing variant and an object of the type associated with "t",
and changes the state of the variant to be the pair consisting of the tag "t"
and the given object.
To determine the tag and value of a variant object,
one normally uses the @b[tagcase] statement (see Section @ref(sect-tagcase)).

Every newly created variant has an identity that is distinct from all
other variants;
two variants can have the same state without being the same variant object.
The identity of variants can be distinguished using the @i(equal)@ (=)
operation.
The @i(similar1) operation tests if two variants have the same tag,
and if so,
tests if the two value components are equal,
using the @i(equal) operation of the value type.
@i(Similar) tests if two variants have the same tag,
and if so,
tests if the two value components are similar,
using the @i(similar) operation of the value type.

All operations are indivisible, except @i(similar), @i(similar1), @i(copy),
@i(encode), and @i(decode), which are divisible at calls to operations of
the type parameters.

For the detailed specification, see Section @ref(variant-type-def).

@SubSection(Procedure and Iterator Types)

Procedures and iterators are created by the Argus system or by the @b[bind]
expression (see Section @ref(sect-bind)).
They are not transmissible.
As the identity of a procedure or iterator is immutable, they can be considered
to be atomic.
However, their atomicity can be violated
if a procedure or iterator has @b[own] data and thus a mutable state.
The immutability and atomicity of a procedure or iterator with @b(own) data
depends on that operation's specified semantics.

The type specification for a procedure or iterator contains
most of the information
stated in a procedure or iterator heading;
a procedure type specification has the form:
@begin[SyntaxDisplay]
@b[proctype] ( @BNFlbkt() type@us<>spec , @BNFdots() @BNFrbkt() ) @BNFlbkt() returns @BNFrbkt() @BNFlbkt() signals @BNFrbkt()
@end[SyntaxDisplay]
and an iterator type specification has the form:
@begin[SyntaxDisplay]
@b[itertype] ( @BNFlbkt() type@us<>spec , @BNFdots() @BNFrbkt() ) @BNFlbkt() yields @BNFrbkt() @BNFlbkt() signals @BNFrbkt()
@end[SyntaxDisplay]
where
@begin[SyntaxDisplay]
@TabClear()
@TabSet(0.9 inches)
returns @\@BNFequals() @b[returns] ( type@us<>spec , @BNFdots() )
yields @\@BNFequals() @b[yields] ( type@us<>spec , @BNFdots() )
signals @\@BNFequals() @b[signals] ( exception , @BNFdots() )
exception @\@BNFequals() name @BNFlbkt() ( type@us<>spec , @BNFdots() ) @BNFrbkt()
@end[SyntaxDisplay]
The first list of type specifications describes
the number, types, and order of arguments.
The @b[returns] or @b[yields] clause gives the number, types, and order of the objects
to be returned or yielded.
The @b[signals] clause lists the exceptions raised by the procedure or iterator;
for each exception name,
the number, types, and order of the objects to be returned is also given.
All names used in a @b[signals] clause must be unique.
The ordering of exceptions is not important.

Procedure and iterator types have an @i(equal)@ (=) operation.
Invocation is @i(not) an operation, but a primitive in Argus.
For the detailed specification of @b(proctype) and @b(itertype), see Section 
@ref(prociter-type-def).

@Section(Atomic@us<>Array, Atomic@us<>Record, and Atomic@us<>Variant)

Having described the types that Argus inherited from CLU, we now describe
the new types in Argus.
The mutable atomic type generators of Argus are @b[atomic@us<>array],
@b[atomic@us<>record], and @b[atomic@us<>variant].
@IndexEntry(Key="atomic_array", Entry="atomic@us<>array", Number)
@IndexEntry(Key="atomic_record", Entry="atomic@us<>record", Number)
@IndexEntry(Key="atomic_variant", Entry="atomic@us<>variant", Number)
@IndexSecondary(Primary="built-in", Secondary="atomic types")
@IndexSecondary(Primary="atomic", Secondary="built-in atomic types")
Types obtained from
these generators provide the same operations as the analogous types
obtained from @b[array], @b[record], and @b[variant], but
they differ in their synchronization and recovery properties.
Conversion operations are provided between each atomic type
generator and its non-atomic partner
(for example, @b[atomic@us<>array][t]$@i(aa2a) converts from an atomic array
to a (non-atomic) array).

An operation of an atomic type generator can be classified as a
@i(reader) or @i(writer)
@index(reader)@index(writer)@index(locking)
depending on whether it examines or modifies its
@i(principal) argument, that is, the argument
or result object of the operation's type.
@index(principal argument)
(For binary operations, such as @i(ar@us<>gets@us<>ar),
the operation is classified with respect to each argument.)
Intuitively, a @i(reader) only examines (reads) the state of its
principal argument,
while a @i(writer) modifies (writes) its principal argument.
Operations that create objects of an atomic type are classified as readers.
Reader/writer exclusion is achieved by locking:
readers acquire a read lock while writers acquire a write lock.
The locking rules are discussed in Section @ref(sect-lockrules).

If one or more of the type parameters is non-atomic, then the resulting
type is not atomic because modifications to component objects are not 
controlled.
However,
read/write locking still
occurs, as described above.  Thus, an atomic type generator instantiated with
a non-atomic parameter incurs the expense of atomic types without gaining any
benefit; such an instantiation is unlikely to be a correct solution to a 
problem.
Atomic type generators yield transmissible types only if the
type parameters are all transmissible.

Special operations are provided for each atomic type generator
to test and manipulate the locks associated with reader/writer
exclusion.  These operations are useful for implementing user-defined
atomic types (see Section @ref(sect-utypes)).  The @b[tagtest] and @b[tagwait]
statements (see Section @ref(sect-tagtest)) provide additional structured support
for atomic@us<>variants.
The operations 
@i(can@us<>read), @i(can@us<>write),
@i(Test@us<>and@us<>read), and @i(test@us<>and@us<>write) provide relatively 
unstructured access to lock information.  For complete definitions of these
operations, see Sections @ref(atomic_array-type-def),
@ref(atomic_record-type-def), and @ref(atomic_variant-type-def).

Assuming normal termination,
the following operations acquire read locks on their principal arguments
or the objects that they create.

@begin(Description)
@b[atomic@us<>array]:@\@~
@i{create, new, predict, fill, fill@us<>copy, size, low, high, empty,
top, bottom, fetch, similar, similar1, copy, copy1, elements, indexes,
test@us<>and@us<>read, a2aa, aa2a, encode, decode}

@b[atomic@us<>record]:@\@~
@i{create, get@us<> , similar, similar1, copy, copy1, test@us<>and@us<>read,
ar@us<>gets@us<>ar} (second argument), @i{r2ar, ar2r, encode, decode}

@b[atomic@us<>variant]:@\@~
@i{make@us<> , is@us<> , value@us<> , av@us<>gets@us<>av} (second argument),
@i{similar, similar1, copy, copy1, test@us<>and@us<>read, v2av, av2v, encode,
decode}
@end(Description)
The operations @i(similar) and @i(similar1)
acquire read locks on both arguments.
The operations @i(copy) and @i(copy1) acquire a read lock on the value
returned as well as their principal argument.
@i(Test@us<>and@us<>read) is a reader only if it returns @b[true];
otherwise it is neither a reader nor a writer.

Assuming normal termination,
the following operations acquire write locks on their principal arguments.

@begin(Description)
@b[atomic@us<>array]:@\@~
@i{set@us<>low, trim, store, addh, addl, remh, reml, test@us<>and@us<>write}

@b[atomic@us<>record]:@\@~
@i{set@us<> , ar@us<>gets@us<>ar} (first argument), @i{test@us<>and@us<>write}

@b[atomic@us<>variant]:@\@~
@i{change@us<> , av@us<>gets@us<>av} (first argument), @i{test@us<>and@us<>write}
@end(Description)

@i(Test@us<>and@us<>write) is a writer only if it returns @b[true];
otherwise it is neither a reader nor a writer.

The @i(equal, can@us<>read,) and @i(can@us<>write) operations
are neither readers nor writers.

When an operation of @b[atomic@us<>array] terminates with an exception,
its principal argument is never modified;
however, the @b[atomic@us<>array] operations listed above as writers
always obtain a write lock before the principal argument is examined,
hence there are cases in which they will obtain a write lock
and only read, but not modify their principal argument.
For example,
@b[atomic@us<>array][t]$@i(trim) is a writer when it signals @i(bounds).
On the other hand, when an @b[atomic@us<>array] operation
raises a signal because of an invalid argument, no locks are obtained.
For example,
when @b[atomic@us<>array][t]$@i(trim) signals @i(negative@us<>size),
it is neither a reader nor a writer since the array's state is
neither examined nor modified
(only the integer argument is examined).

For the detailed specification of atomic arrays, see Section @ref(atomic_array-type-def);
for atomic records, see Section @ref(atomic_record-type-def); and for
atomic variants, see Section @ref(atomic_variant-type-def).

@Section(Guardian Types)
@Label(sect-gtype)

Guardian types are user-defined types that
are implemented by guardian definitions
(see Section @ref(sect-guardian)).
A guardian definition has a header of the form:
@begin[SyntaxDisplay]
idn = @b[guardian]  @BNFlbkt() parms @BNFrbkt()  @b[is] idn , @BNFdots()  @BNFlbkt() @b[handles] idn , @BNFdots() @BNFrbkt() @BNFlbkt() where @BNFrbkt()
@end[SyntaxDisplay]
The creators are the operations named in the identifier list following @b[is];
a creator is a special kind of operation that can be called to create new
guardians that behave in accordance with the guardian definition.
Each guardian optionally provides @i(handlers) that can be
called to interact with it;
the names of these handlers are listed in the identifier
list following @b[handles].
(See Section @ref(sect-guardian) for more details.)

A guardian definition named @i(g) defines a guardian interface type @i(g).
@IndexSecondary(Primary="type", Secondary="guardian interface")
@IndexSecondary(Primary="guardian", Secondary="interface")
@IndexSecondary(Primary="guardian", Secondary="versus guardian interface")
An object of the guardian interface type provides an interface to a guardian
that behaves in accordance with the guardian definition.
An interface object is created whenever a new guardian is created, and
then the interface object can be used to access the guardian's handlers.
Interface objects are transmissible, and
after transmission they still give access to the same guardian.  In this
manual
a "guardian interface object" is often called simply a "guardian object".

The guardian type @i(g) for the guardian definition named @i(g)
has the following operations.
@index(guardian)@IndexSecondary(Primary="guardian", Secondary="type of")
@IndexSecondary(Primary="type", Secondary="of a guardian")
@begin(enumerate)
The creators listed in the @b[is] list of the guardian definition.

For each handler name @i(h) listed in the @b[handles] list,
an operation @i(get@us<>h) with type:
@b[proctype]@ (@i(g))@ @b[returns]@ (@i(ht)),
where @i(ht) is the type of @i(h).

@i(Equal) and @i(similar), both of type:
@b[proctype]@ (@i(g),@ @i(g))@ @b[returns]@ (@b[bool]),
which return @b[true] only if both arguments are the same guardian object.

@i(Copy), of type:  @b[proctype]@ (@i(g))@ @b[returns]@ (@i(g)),
which simply returns its argument.

@b[transmit].
@end(enumerate)
A creator may not be named @i(equal), @i(similar), @i(copy), @i(print),
or @i(get@us<>h) where @i(h) is the name of a handler.

Thus if @i(x) is a variable denoting a guardian interface object of
type @i(g), and @i(h) is a handler of @i(g), then
@i(g)$@i(get@us<>h(x)) will return this handler.
As usual with @i(get)@us<> operations, this call can be abbreviated to @i(x.h).
Note that the handlers themselves are not operations of the guardian interface
type;
thus @i(g)$@i(h) would be illegal.

A guardian interface type is somewhat like a structure type.
Its objects are constructed by the creators, and decomposed
by the @i(get)@us<> operations.
Guardian interface objects are immutable and atomic.

@Section(Handler and Creator Types)

Creators are operations of guardian types.
@index(creator)@index(handler)@index(handlertype)@index(creatortype)
@IndexSecondary(Primary="type", Secondary="of a creator")
@IndexSecondary(Primary="type", Secondary="of a handler")
Handler objects are created as a side-effect of guardian creation.
Unlike procedures and iterators, handlers and creators are transmissible.

The types of handlers and creators resemble the types of procedures:
@begin[SyntaxDisplay]
@b[handlertype] ( @BNFlbkt() type@us<>spec, ... @BNFrbkt() ) @BNFlbkt() returns @BNFrbkt() @BNFlbkt() signals @BNFrbkt()
@b[creatortype] ( @BNFlbkt() type@us<>spec, ... @BNFrbkt() ) @BNFlbkt() returns @BNFrbkt() @BNFlbkt() signals @BNFrbkt()
@end[SyntaxDisplay]

The argument, normal result, and exception result types
must all be transmissible.
The @i(signals) list for a @b[handlertype] or @b[creatortype] cannot include
either @i(failure) or @i(unavailable),
as these signals are implicit in the interface of all creators and handlers.

Handler and creator types provide @i(equal) and @i(similar)
operations which return @b[true] if and only if both arguments are the
same object,
and @i(copy) operations which simply return their argument.
For the detailed specification of @b(handlertype) and @b(creatortype),
see Section @ref(proccreator-type-def).


@Section(Image)
@Label(sect-image)

The @b[image] type provides an escape from compile-time type checking.
@index(image)@IndexSecondary(Primary="image", Secondary="versus any")
@index(any)@IndexSecondary(Primary="any", Secondary="versus image")
The main difference between @b[image] and @b[any] is that @b[image] objects are
transmissible.
An @b[image] object can be thought of as a portion of an undecoded message
or as the information needed to recreate an object of some type.
@b(Image) objects are immutable and atomic.

The @i(create) operation is parameterized by a transmissible type;
it takes a single argument of that type and encodes it
(using the @i(encode) operation of that type)
into an @b[image] object.
The @i(force) operation is also parameterized by a transmissible type;
it takes an @b[image] object and decodes it
(using the @i(decode) operation of that type) to an object of that type,
signalling @i(wrong@us<>type) if the encoded object's type is
not included in the parameter type.
The @i(is@us<>type) operation is parameterized by a type and
checks whether its argument is an encoded object of a type
included in the parameter type.
See Section @ref(image-type-def) for the detailed specification.

@Section(Mutex)
@Label(sect-mutex)

Mutex objects are mutable containers for information.
@index(mutex)@index(concurrency)
They are not atomic, but they provide synchronization and control of
writing to stable storage for their contained object.
Mutex itself does not provide operations for synchronizing
the use of mutex objects.
Instead, mutual exclusion is achieved using the @b[seize] statement
(see Section @ref(sect-seize)),
which allows a sequence of statements to be executed while a process
is in exclusive possession of the mutex object.
Mutex objects are transmissible if the contained object is transmissible.

The type generator @b[mutex] has a single parameter that
is the type of the contained object.
A mutex type specification has the form:
@begin[SyntaxDisplay]
@b[mutex] [type@us<>actual]
@end[SyntaxDisplay]
Mutex types provide operations to create and decompose
mutex objects, and to notify the system of
modifications to the mutex object or its contained object.

The @i(create) operation takes a single argument of the
parameter type and creates a new mutex object containing the argument object.
The @i(get@us<>value) operation obtains the contained
object from its mutex argument,
while @i(set@us<>value) modifies a mutex object
by replacing its contained object.
As with records, these operations can be called
using special forms, for example:
@begin[Display]
@TabClear()
m: @b[mutex][@b[int]] := @b[mutex][@b[int]]$create (0)@^
x: @b[int] := m.value@\% extract the contained object
m.value := 33@\% change the contained object
@end[Display]
@i(Set@us<>value) and @i(get@us<>value) are indivisible.

Mutexes can be distinguished with the @i(equal) (=) operation.
There are no operations that could cause or detect
sharing of the contained object by two mutexes.
Such sharing is dangerous, since
two processes would not be synchronized with each other
in their use of the contained object
if each possessed a different mutex.
In general, if an object is contained in a mutex object,
it should not be contained in any other object,
nor should it be referred to by a variable except
when in a @b[seize] statement that has possession of the containing mutex.

There are some mutex operations that seize the mutex object automatically.
@i(Copy) seizes its single argument object.
@i(Similar) seizes its two argument objects;
the first argument object is seized first and then the second.
In both cases possession is retained until the operations return.
Also, when a mutex object is encoded (for a message or when making an @b[image]),
the object is seized automatically.
See Section @ref(mutex-type-def) for the detailed specification of @b(mutex).

Mutexes are used primarily
to provide process synchronization and mutual exclusion on shared data,
especially 
to implement user-defined atomic types.
In such implementations, it is important
to control writing to stable storage.
The mutex operation @i(changed) provides the necessary control.
@i(Changed) informs the system that the calling action requires that
the argument object be copied to stable storage before the commit of the 
action's top-level parent (topaction).
Any mutex is @i(asynchronous):  its contained object is written to
stable storage independently of objects that
contain that mutex.
See Section @ref(sect-utypes) for further discussion of user-defined 
atomic objects.

@Section(Node)

Objects of type @b[node] stand for physical nodes.
@index(node)
The operation @i(here) takes no arguments and
returns the @b[node] object that denotes its caller's node.
@i(Equal), @i(similar), and @i(copy) operations are also provided.

The main use of @b[node] objects is in guardian creation
(see Section @ref(sect-guardian)), where
they are used to cause a newly created
guardian to reside at a particular node.
Objects of type @b[node] are immutable, atomic, and transmissible.
For the detailed specification, see Section @ref(node-type-def).

@Section(Other Type Specifications)

A type specification for a user-defined type has the form of a
@i(reference):
@begin[SyntaxDisplay]
@TabClear()
reference @BNFequals()@^ idn
@\@BNFor() idn [ actual@us<>parm , @BNFdots() ]
@\@BNFor() reference $ name
@end[SyntaxDisplay]
where each @i(actual@us<>parm) must be a compile-time computable constant
(see Section @ref(sect-constant)) or a @i(type@us<>actual)
(see Section @ref(sect-instantiations)).
@IndexSecondary(Primary="type", Secondary="user-defined")@index(reference)
A reference must denote a data abstraction to be used as a type specification;
this syntax is provided
for referring to a data abstraction that is named in an equate module
(see Section @ref(sect-equmod)).
@index(equate module)
For type generators,
actual parameters of the appropriate types and number must be supplied.
The order of parameters is always significant for user-defined types
(see Section @ref(sect-typegen)).

There are two special type specifications that are used
when implementing new abstractions: @b[rep], and @b[cvt].
These forms may only be used within a cluster;
they are discussed further in Section @ref(sect-cluster).

Within an implementation of an abstraction,
formal parameters declared with @b[type] can be used as type specifications.
@IndexSecondary(Primary="type", Secondary="parameter")
Finally, identifiers that have been equated to type specifications
can also be used as type specifications.
