@Part(scopes, Root "manual.mss")

@Chapter(Scopes, Declarations, and Equates)

This section describes how to introduce and use constants and variables,
and the scope of constant and variable names.
Scoping units are described first,
followed by a discussion of variables, and finally constants.

@Section(Scoping Units)
@Label(sect-scope)

Scoping units follow the nesting structure of statements.
@index(scope)@IndexSecondary(Primary="scope", Secondary="unit")
Generally,
a scoping unit is a body and an associated "heading".
The scoping units are as follows (see Appendix @ref(apdx-syntax) for details of
the syntax).
@begin(enumerate)
From the start of a @i(module) to its end.

From a @b[cluster], @b[proc], @b[iter], @b[equates], @b[guardian], @b[handler], or
@b[creator] to the matching @b[end].

From a @b[for], @b[do], @b[begin], @b[background], @b[recover], @b[enter], @b[coenter],
or @b[seize] to the matching @b[end].

From a @b[then] or @b[else] in an @b[if] statement to the end of the corresponding body.

From a @b[tag], @b[wtag], or @b[others] in a @b[tagcase], @b[tagwait], or @b[tagtest]
statement to the end of the corresponding body.

From a @b[when] or @b[others] in an @b[except] statement
to the end of the corresponding body.

From the start of a @i(type@us<>set) to its end.

From an @b[action] or @b[topaction] to the end of the corresponding body.
@end(enumerate)

The structure of scoping units is such that if one scoping unit
overlaps another scoping unit (textually), then one is fully
contained in the other.
The contained scope is called a @i(nested) scope, and the containing
scope is called a @i(surrounding) scope.

New constant and variable names may be introduced in a scoping unit.
Names for constants are introduced by equates, which are syntactically
restricted to appear grouped together at or near
the beginning of scoping units (except in type sets).
For example, equates may appear at the beginning of a body,
but not after any statements in the body.

In contrast, declarations, which introduce new variables,
are allowed wherever statements are allowed, and hence
may appear throughout a scoping unit.
Equates and declarations are discussed in more detail in the following two sections.

In the syntax there are two distinct nonterminals
for identifiers: @i(idn) and @i(name).
@index(idn)@IndexSecondary(Primary="idn", Secondary="versus name")
@index(name)@IndexSecondary(Primary="name", Secondary="versus idn")
Any identifier introduced by an equate or declaration is an @i(idn),
as is the name of the module being defined, and any operations it has.
An @i(idn) names a specific type or object.
The other kind of identifier is a @i(name).
A @i(name) is generally used to refer to a piece of something,
and is always used in context; for example, @i(names) are used
as record selectors.
The scope rules apply only to @i(idns).

The scope rules are simple:
@IndexSecondary(Primary="scope", Secondary="rules")
@begin(enumerate)
An @i(idn) may not be redefined in its scope.

Any @i(idn) that is used as an external reference in a module
may not be used for any other purpose in that module.
@end(enumerate)
Unlike other "block-structured" languages,
@index(block structure)
Argus prohibits the redefinition of an identifier in a nested scope.
An identifier used as an external reference names a module or constant;
the reference is resolved using the compilation environment.

@SubSection(Variables)

Objects are the fundamental "things" in the Argus universe;
variables are a mechanism for denoting (i.e., naming) objects.
@index(variable)
A variable has three properties:  its type, whether it is stable or not,
and the object that it currently denotes (if any).
A variable is said to be @i(uninitialized) if it does not denote any object.
@index(uninitialized variable)
@IndexSecondary(Primary="variable", Secondary="uninitialized")
Attempts to use uninitialized variables are programming errors
and (if not detected at compile-time) cause the guardian to crash.

There are only three things that can be done with variables:
@begin(enumerate)
New variables can be introduced.
Declarations perform this function, and are described below.

An object may be assigned to a variable.
After an assignment the variable denotes the object assigned.

A variable may be used as an expression.
The value of a variable
is the object that the variable denotes at the time the expression 
is evaluated.
@end(enumerate)

@SubSection(Declarations)
@Label(sect-declaration)

Declarations introduce new variables.
@index(declaration)
The scope of a variable is from its declaration to the
end of the smallest scoping unit containing its declaration;
hence, variables must be declared before they are used.

There are two sorts of declarations: those with initialization,
and those without.
@IndexSecondary(Primary="declaration", Secondary="simple")
Simple declarations (those without initialization) take the form
@begin[SyntaxDisplay]
@TabClear()
decl @^@BNFequals() idn , @BNFdots() : type@us<>spec
@end[SyntaxDisplay]
A simple declaration introduces a list of variables,
all having the type given by the @i(type@us<>spec).
This type determines the types of objects that can be assigned to the variable.
The variables introduced in a simple declaration initially denote no objects,
i.e., they are uninitialized.

A declaration with initialization combines
declarations and assignments into a single statement.
@IndexSecondary(Primary="declaration", Secondary="with initialization")
A declaration with initialization is entirely equivalent to
one or more simple declarations followed by an assignment statement.
The two forms of declaration with initialization are:
@begin[SyntaxDisplay]
idn : type@us<>spec := expression
@end[SyntaxDisplay]
and
@begin[SyntaxDisplay]
decl@-{1}, @BNFdots(), decl@-{n} := call @BNFlbkt() @@ primary @BNFrbkt()
@end[SyntaxDisplay]
These are equivalent to (respectively):
@begin[Display]
idn : type@us<>spec
idn := expression
@end[Display]
and
@begin[Display]
decl@-{1} @BNFdots() decl@-{n}	  %  declaring idn@-{1} @BNFdots() idn@-{m}
idn@-{1}, @BNFdots(), idn@-{m} := call @BNFlbkt() @@ primary @BNFrbkt()
@end[Display]
In the second form, the order of the idns in the
assignment statement is the same as in the original declaration
with initialization.
(The call must return @i(m) objects.)


@Section(Equates and Constants)
@Label(sect-constant)@Label(sect-equates)

An equate allows an identifier to be used as
an abbreviation for a constant, type set, or equate module name that
may have a lengthy textual representation.
@index(equate)
An equate also permits a mnemonic identifier to be used in place
of a frequently used constant, such as a numerical value.
We use the term constant in a very narrow sense here:
constants,
in addition to being immutable,
must be computable at compile-time.
Constants are either
types (built-in or user-defined),
or objects that are the results of evaluating constant expressions.
(Constant expressions are defined below.)

The syntax of equates is:
@begin[SyntaxDisplay]
@TabClear()

equate @^@BNFequals() idn = constant
@\@BNFor() idn = type@us<>set
@\@BNFor() idn = reference

constant @^@BNFequals() type@us<>spec
@\@BNFor() expression

type@us<>set @^@BNFequals() { idn | idn @b[has] oper@us<>decl , @BNFdots() @BNFlcurly() equate @BNFrcurly() }

reference @^@BNFequals() idn
@\@BNFor() idn [ actual@us<>parm , @BNFdots() ]
@\@BNFor() reference $ name
@end[SyntaxDisplay]
References can be used to name equate modules.

An equated identifier may not be used on the left-hand side of an assignment
statement.

The scope of an equated identifier is the smallest scoping unit
surrounding the equate defining it;
here we mean the entire scoping unit, not just the portion after the equate.
All the equates in a scoping unit must appear grouped near the beginning of the scoping unit.
The exact placement of equates depends on the containing syntactic construct;
usually equates appear at the beginnings of bodies.

Equates may be in any order within the a scoping unit.
Forward references among equates in the same scoping unit are
allowed, but cyclic dependencies are illegal.
For example,
@begin[Display]
x = y
y = z
z = 3
@end[Display]
is a legal sequence of equates, but
@begin[Display]
x = y
y = z
z = x
@end[Display]
is not.
Since equates introduce idns, the scoping restrictions on idns
apply (i.e., the idns may not be defined more than once).


@SubSection(Abbreviations for Types)

Identifiers may be equated to type specifications,
giving abbreviations for type names.


@SubSection(Constant Expressions)
@Label(sect-constant-exprs)

We define the subset of objects that equated identifiers may denote
by stating which expressions are constant expressions.
@index(constant)
(Expressions are discussed in detail in Section @ref(sect-expr).)
A @i(constant expression) is an expression that can be evaluated
at compile-time to produce an immutable object of a built-in type.
This includes:
@begin(enumerate)
Literals.

Identifiers equated to constants.

Formal parameters.

Procedure, iterator, and creator names.

Bind expressions (see Section @ref(sect-bind)),
where the routine bound and the explicit arguments are all constants.

Invocations of procedure operations of the built-in immutable types,
provided that all operands are constant expressions
that are not formal parameters.
@end(enumerate)
The built-in immutable types are:
@b[null], @b[int], @b[real], @b[bool], @b[char], @b[string],
sequence types, oneof types, structure types, procedure types,
iterator types, and creator types.

We explicitly forbid the use of formal parameters as operands to calls
in constant expressions,
since the values of formal parameters are not known at compile-time.
If the evaluation of a constant expression would signal an exception, the
constant defined by that expression is illegal.
