@Part(call, Root "manual.mss")

@Chapter(Assignment and Calls)
@Label(sect-assign)
@Label(sect-call)

The two fundamental activities of Argus programs are calls and
assignment of computed objects to variables.

Argus programs should use mutual exclusion or atomic data
to synchronize access to all shared variables,
@index(synchronization)
because Argus supports concurrency and thus
processes can interfere with each other during assignments.
@index(indivisibility)@index(concurrency)
@IndexSecondary(Primary="assignment", Secondary="and concurrency")
For example,
@begin[Display]
i := 1
j := 2
@end[Display]
is not equivalent to
@begin[Display]
i, j := 1, 2
@end[Display]
in the presence of concurrent assignments to the same variables,
because any interleaving of indivisible events is possible
in the presence of concurrency.

Argus is designed to allow complete compile-time type-checking.
@index(type checking)
The type of each variable is known by the compiler.
@index(type)
Furthermore, the type of objects that could result from
the evaluation of any expression is known at compile time.
Hence, every assignment can be checked at compile time to
ensure that the variable is only assigned objects of its declared type.
@IndexSecondary(Primary="assignment", Secondary="type checking for")
An assignment @i(v)@ :=@ @i(E) is legal only if
the type of @i(E) is included the type of @i(v).
The definition of type inclusion is given in Section @ref(sect-typeincl).

@Section(Assignment)

Assignment causes a variable to denote an object.
@index(assignment)
@IndexSecondary(Primary="assignment", Secondary="statement")
@IndexSecondary(Primary="statement", Secondary="assignment")
@IndexSecondary(Primary="assignment", Secondary="implicit")
Some assignments are implicitly performed as part of the execution
of various mechanisms of the language
(in exception handling, and the @b[tagcase], @b[tagtest],
and @b[tagwait] statements).
All assignments, whether implicit or explicit,
are subject to the type inclusion rule.

@SubSection(Simple Assignment)

The simplest form of assignment statement is:
@IndexEntry(Key=" :=", Entry=":=", Number)
@IndexSecondary(Primary="assignment", Secondary="simple")
@begin[SyntaxDisplay]
idn := expression
@end[SyntaxDisplay]
In this case the @i(expression) is evaluated, and then the
resulting object is assigned to the variable named by the @i(idn)
in an indivisible event.
Thus no other process may observe a "half-assigned" state of the variable,
but another process may observe various states
during the expression evaluation and between
the evaluation of the expression and the assignment.
The expression must return a single object
(whose type must be included in that of the variable).

@SubSection(Multiple Assignment)

There are two forms of assignment statement
@IndexSecondary(Primary="assignment", Secondary="multiple")
that assign to more than one variable at once:
@begin[SyntaxDisplay]
idn , @BNFdots() := expression , @BNFdots()
@end[SyntaxDisplay]
and
@begin[SyntaxDisplay]
idn , @BNFdots() := call @BNFlbkt() @@ primary @BNFrbkt()
@end[SyntaxDisplay]

The first form of multiple assignment is a generalization of simple assignment.
The first variable is assigned the first expression,
the second variable the second expression, and so on.
The expressions are all evaluated (from left to right)
before any assignments are performed.
The assignment of multiple objects to multiple
variables is an indivisible event,
but evaluation of the expressions
is divisible from the actual assignment.
The number of variables in the list must equal the number of expressions,
no variable may occur more than once,
and the type of each variable must include the type of the corresponding expression.

The second form of multiple assignment allows
one to retain the objects resulting from a call
returning two or more objects.
The first variable is assigned the first object,
the second variable the second object, and so on, but all the assignments
are carried out indivisibly.
The order of the objects is the same as in the @b[return] statement
executed in the called routine.
The number of variables must equal the number of objects returned,
no variable may occur more than once,
and the type of each variable must include the corresponding return type of
the called procedure.


@Section(Local Calls)
@Label(sect-proc-call)

In this section we discuss procedure calls;
@index(call)
@IndexSecondary(Primary="local", Secondary="call")
@IndexSecondary(Primary="call", Secondary="local")
iterator calls are discussed in Section @ref(sect-for-stmt).
However, argument passing is the same for both procedures and iterators.

Local calls take the form:
@begin[SyntaxDisplay]
primary ( @BNFlbkt() expression , @BNFdots() @BNFrbkt() )
@end[SyntaxDisplay]

The sequence of activities in performing a local call are as follows:
@begin(enumerate)
The @i(primary) is evaluated.

The @i(expressions) are evaluated, from left to right.

New variables are introduced corresponding to the formal arguments
@IndexSecondary(Primary="formal", Secondary="argument")
of the routine being called (i.e., a new environment is created
for the called routine to execute in).

The objects resulting from evaluating the @i(expressions)
(the actual arguments)
@index(actual argument)@IndexSecondary(Primary="argument", Secondary="actual")
@IndexSecondary(Primary="call", Secondary="by sharing")
are assigned to the corresponding new variables
(the formal arguments).
The first formal is assigned the first actual,
the second formal the second actual, and so on.
The type of each expression must be included in the type of the
corresponding formal argument.

Control is transferred to the routine at the start of its body.
@end(enumerate)
A call is considered legal in exactly
those situations where all the (implicit) assignments are legal.

A routine may assign an object to a
formal argument variable;
@index(assignment)
the effect is just as if that object were assigned to any other variable.
From the point of view of the called routine, the only difference
between its formal argument variables and its other local variables
is that the formals are initialized by its caller.

Procedures can terminate in two ways:
they can terminate @i(normally),
returning zero or more objects,
or they can terminate @i(exceptionally),
signalling an exceptional condition.
@IndexSecondary(Primary="termination", Secondary="of a routine")
When a procedure terminates normally, any result objects
become available to the caller,
and can be assigned to variables
or passed as arguments to other routines.
@index(exception)
When a procedure terminates exceptionally,
the flow of control will not go to the point of return
of the call, but rather will go to an exception handler
(see Section @ref(sect-excepth)).

@Section(Handler Calls)
@Label(sect-handlercall)@Label(sect-hcall)

As explained in Section @ref(sect-handler) and in Section @ref(sect-guardian),
a handler is an operation that belongs to some guardian.
A handler call causes an activation of the called handler to run
at the handler's guardian;
the activation is performed at the called handler's guardian by a new subaction
created solely for this purpose.
@index(call)@index(remote call)@IndexSecondary(Primary="call", Secondary="remote")
@index(guardian)@SeeAlso(Primary="RPC", Other"remote call")
@IndexSecondary(Primary="handler", Secondary="call")
Usually the handler's guardian is not the
same as the one in which the call occurs,
and the called handler's guardian is likely to reside at a different
node in the network than the calling guardian.
However,
it is legal to call a handler that belongs to
a guardian residing at the caller's node,
or even to call a handler belonging to the caller's guardian.

Although the form of a handler call looks like a procedure call:
@begin[SyntaxDisplay]
primary ( @BNFlbkt() expression, @BNFdots() @BNFrbkt() )
@end[SyntaxDisplay]
its meaning is very different.
Among other things, a handler is called remotely, with the
arguments and results being transmitted by value in messages,
@IndexSecondary(Primary="call", Secondary="by value")
and the call is run as a subaction of its calling action.
Below we present an overview of what happens
when executing a handler call and then a detailed description.

A handler call runs as a subaction of the calling action.
@index(subaction)
We will refer to this subaction as the @i(call action).
@index(call action)
@IndexSecondary(Primary="action", Secondary="call action")
@IndexSecondary(Primary="call", Secondary="action")
The first thing done by the call action is the transmission
of the arguments of the call.
Transmission is accomplished by encoding each argument object,
using the @i(encode) operation of its type.
@index(transmit)@index(encode)
The arguments are decoded at the called guardian by
a subaction of the call action called the @i(activation action).
@index(activation action)
@IndexSecondary(Primary="action", Secondary="activation action")
Each argument is decoded by using the @i(decode) operation of its type.
@index(decode)
The effect of transmission is that the arguments are passed
by value from the caller to the handler activation:
new objects come into existence at the handler's guardian
that are copies of the argument objects.
Object values are transmitted in such a way as to preserve
the internal sharing structure of each argument object is preserved@Foot{
This is only strictly true for the built-in types.
A user-defined type might not preserve internal sharing structure.
},
as well as any sharing structure between the argument objects in a single call.
See Section @ref(sect-transmit) for further discussion of transmission.

After the arguments have been transmitted,
the activation action performs the handler body.
When the handler body terminates, by executing a @b[return], @b[abort return],
@b[signal], or @b[abort signal] statement, the result objects are transmitted
to the caller by encoding them at the handler's guardian,
and committing or aborting the activation action (as it specified).
The call action then decodes the results at the caller's guardian.
@index(decode)
Once the results have been transmitted to the caller,
the call action commits and execution continues in the
caller as indicated by the caller's code.
@IndexSecondary(Primary="commit", Secondary="of a remote call action")
@IndexSecondary(Primary="abort", Secondary="of a remote call action")
(Note that the call action will commit even if the activation action aborts.)

The above discussion has ignored the possibility
of several problems that may arise in executing a handler call.
These problems either cause the call action or the activation action to abort
or result in the crash of the calling guardian.
A handler call attempted from outside a topaction or subaction
is a programming error, and so if this happens the calling guardian is crashed.
Other problems cause the call action or the activation action to be aborted,
and this is reflected back to the caller as an exception
raised by the Argus system.
Two such exceptions can be raised:  @i(failure)(@b[string])
and @i(unavailable)(@b[string]).
The @b[string] exception results summarize the problem that has occurred.

The meaning of a @i(failure) exception generated by the
@index(failure)
Argus system is that this particular call did not succeed,
and furthermore it is unlikely to succeed if repeated.
There are two reasons why @i(failure) is raised:
an error occurred in transmitting an argument or result,
or the handler's guardian no longer exists.

The Argus system raises the @i(unavailable) exception
@index(unavailable)
when it is unable to communicate with the handler's guardian.
Reasons why communication may fail include network
partitions and a crashes of the called guardian or its node.
The Argus system raises the @i(unavailable) exception only if communication
seems impossible at that time;
it may try many times to establish communication.
Therefore, when a call terminates with the @i(unavailable) exception,
there is little point in retrying the call immediately.
However, unlike a call terminated by the @i(failure) exception,
a call terminated by the @i(unavailable) exception
may complete successfully if retried later.
Note that the arguments and results
may be encoded several times as the system tries to
establish communication.

For example, suppose we have a handler call:
@begin[Display]
m.send@us<>mail(user, my@us<>message)
@end[Display]
where @i(m) is a mailer guardian, and the @i(send@us<>mail) handler
has the header
@begin[Display]
send@us<>mail = @b[handler] (u: user@us<>id, msg: message) @b[signals] (no@us<>such@us<>user)
@end[Display]
Then @i(user) and @i(my@us<>message) are encoded using the @i(encode) operations
of types user@us<>id and message, respectively,
and the encoded values are decoded at the called guardian
using the @i(decode) operations of these types.
If @i(user) is actually registered to receive mail,
then @i(send@us<>mail) will return normally;
otherwise it signals @i(no@us<>such@us<>user).
In either case no encoding or decoding of the reply is needed
since there is no result.

Possible exceptions from this call are @i(no@us<>such@us<>user), @i(failure),
and @i(unavailable).
So the call might be performed in an @b[except] statement:
@begin[Display]
@TabClear()
m.send@us<>mail(user, my@us<>message)
	@b[except] @^@b[when] no@us<>such@us<>user: ...
@\@b[when] unavailable (s: @b[string]): ...
@\@b[when] failure (s: @b[string]): ...
@\@b[end]
@end[Display]

@SubSection(Semantics of Handler Calls)

In this section we describe the semantics of a handler call in detail.
A handler call causes activity at
both the calling guardian and at the called guardian.
At the calling guardian, the sequence of activities in performing a handler
call is as follows:
@IndexSecondary(Primary="call", Secondary="semantics of remote call")
@IndexSecondary(Primary="remote call", Secondary="semantics of")
@begin(enumerate)
The @i(primary) is evaluated.

The argument @i(expressions) are evaluated from left to right.

A subaction, which we will refer to as the @i(call action),
@index(call action)
is created for the remote call.
All subsequent activity on behalf of the call will
be performed by the call action or one of its descendants.
For it to be possible to create the call action,
the caller must already be running as an action.
Remote calls by non-actions are programming errors and
cause the calling guardian to crash.

A call message is constructed.
@IndexSecondary(Primary="call", Secondary="message")
As part of constructing this message,
@i(encode) operations are performed on the argument objects.
@index(encode)
If any of the @i(encode) operations terminates with a @i(failure) exception,
@index(failure)
then the remote call will terminate with the same exception,
and the call action will be aborted.

The call message is sent to the guardian of the called handler,
and the call action waits for the completion of the call.

If the call message arrives at the node of the target guardian,
and the target guardian does not exist,
then the call action is aborted with the @i(failure) exception
having the @b[string] "guardian@ does@ not@ exist" as its exception result.
@index(failure)

If the system determines that it cannot communicate with the called guardian,
it aborts the call action.
The call action may be retried several times (beginning at step 3)
in attempts to communicate.
If repeated communication failures are encountered,
@IndexSecondary(Primary="failure", Secondary="of communications in a remote call")
the system aborts the call action and causes the call to terminate with the
@i(unavailable) exception.
@index(unavailable)
The system will cause this kind of termination
only when it is extremely unlikely
that retrying the call immediately will succeed.

Ordinarily, a call completes when a reply message
containing the results is received.
When the reply message arrives at the caller,
it is decoded using the @i(decode) operation for each result object.
@index(decode)
If any @i(decode) terminates with a @i(failure) exception,
the call action is aborted,
and the call terminates with the same exception.
Otherwise, the call action commits.

The call will terminate normally if the result message indicates
that the handler activation returned (instead of signalled);
otherwise it terminates with whatever exception was signalled.
@end(enumerate)

At the called guardian, the following activities take place.
@begin(enumerate)
A subaction of the call action is created
at the target guardian to run the call.
We will refer to this subaction as the @i(activation action).
@index(activation action)
@IndexSecondary(Primary="action", Secondary="activation action")
All activity at the target guardian occurs on behalf of the activation
action or one of its descendants.

The call message is decomposed into its constituent objects.
@IndexSecondary(Primary="call", Secondary="message")
As part of this process @i(decode) operations are performed on each argument.
@index(decode)
If any @i(decode) terminates with a @i(failure) exception,
@index(failure)
then the activation action is aborted,
and the call terminates with the same exception.

The called handler is called within the activation action.
This call is like a regular procedure call.
The objects obtained from decoding the message are the actual arguments,
and they are bound to the formals via implicit assignments.

If the handler terminates by executing an @b[abort return] or an
@b[abort signal] statement (see Section @ref(sect-signal)),
then all committed descendents of the activation action are aborted.
Then the reply message is constructed by encoding the result objects,
the activation action is aborted, and  the reply message is sent to the caller.
Otherwise, when the handler terminates,
the reply message is constructed by encoding the result objects,
the activation action commits, and  the reply message is sent to the caller.
If one of the calls of @i(encode) terminates with a @i(failure) exception,
@index(encode)@index(failure)
then the activation action is aborted,
and the call terminates with the same exception.
@end(enumerate)

When the Argus system terminates a call with the @i(unavailable) exception,
@index(unavailable)
it is possible that the activation action and/or some of its descendants
are actually running.
This could happen, for example, if the network partitions.
These running processes are called "orphans".
The Argus system makes sure that orphans will be aborted before they
@index(orphan)
can view inconsistent data (see Section @ref(sect-orphan)).

@Section(Creator Calls)
@Label(sect-creatorcall)@Label(sect-ccall)

Creators are called to cause new guardians to come into existence.
@index(creator)@index(creator call)
@index(call)@index(remote call)
@IndexSecondary(Primary="call", Secondary="remote")
@IndexSecondary(Primary="call", Secondary="creator")
As part of the call, the node at which the newly created guardian will
be located may be specified.
If the node is not specified,
@index(node)@IndexSecondary(Primary="node", Secondary="of guardian creation")
@index(guardian)@IndexSecondary(Primary="guardian", Secondary="creation")
then the new guardian is created at the same node as the caller of the creator.
The form of a creator call is:
@begin[SyntaxDisplay]
primary ( @BNFlbkt() expression, @BNFdots() @BNFrbkt() ) @BNFlbkt() @@ primary @BNFrbkt()
@end[SyntaxDisplay]

The @i(primary) following the at-sign (@@) must be of type @b[node].
@index(node)@IndexEntry(Key=" @@", Entry="@@", Number)

A creator call causes two activities to take place.
First, a new guardian is created at the indicated node.
Second, the creator is called as a handler at the newly created guardian.
This handler call has basically the same semantics as the
regular handler call described above.

The Argus system may also cause a creator call to abort
with the @i(failure) or @i(unavailable) exceptions.
The reasons for such terminations are the same as those for handler calls,
and the meanings are the same:
the @i(failure) exception means that the call should not be retried,
while the unavailable exception means
that the call should not be retried immediately.

@SubSection(Semantics of Creator Calls)

The activities carried out in executing a creator call are as follows.
@index(creator call)@index(call)
@IndexSecondary(Primary="creator call", Secondary="semantics of")
@IndexSecondary(Primary="call", Secondary="semantics of creator call")
@begin(enumerate)
The (first) @i(primary) is evaluated.

The argument @i(expressions) are evaluated from left to right.

The optional @i(primary) following the at-sign is evaluated to obtain a @b[node]
object.
If this @i(primary) is missing,
the node at which the call is taking place is used.

A subaction, which we will refer to as the @i(call action), is created.
@index(call action)
All subsequent activity takes place within this subaction.
As was the case for handler calls,
creators can be called only from within actions.
A creator call by a non-action is a programming error
and causes the calling guardian to crash.

A new guardian is created at the indicated node.
The creator obtained in step 1 will indicate the type of this guardian.
The selection of a particular load image for this
type will occur as discussed in Section @ref(sect-select).

As was the case for handler calls,
if the system cannot communicate with the indicated node,
the creator call will terminate with the @i(unavailable) exception.
If the system is unable to determine what implementation to load,
or if there is no implementation of the type
that can run on the indicated node,
or if the manager of the node refuses to allow the new guardian to be created,
the creator call will terminate with the @i(failure) exception.
In either case the call action will be aborted.

A remote call is now performed to the creator.
This call has the same semantics as described for handler calls
above in steps 4 through 9 of the activities at the calling node
and also steps 1 through 4 of activities at the called node.
However, if either the call action or the activation action aborts,
the newly created guardian will be destroyed.
@end(enumerate)


For example, suppose we execute the creator call
@begin[Display]
x: G := G$create(3) @@ n
@end[Display]
where @i(G) is a guardian type, @i(n) denotes an object of type @b[node], and
@i(create) has header
@begin[Display]
create = @b[creator] (n: @b[int]) @b[returns] (G) @b[signals] (not@us<>possible(@b[string]))
@end[Display]
The system will select an implementation of @i(G)
that is suitable for use at node @i(n),
and will then create a guardian at node @i(n) running that implementation.
Next @i(create) (3) is performed as a handler call at that new guardian.
If @i(create) returns,
then the assignment to @i(x) will occur,
causing @i(x) to refer to the new guardian that @i(create) returned;
now we can call the handlers provided by G.
The exceptions that can be signalled by this call are
@i(not@us<>possible), @i(failure), and @i(unavailable).
An example of a call that handles all these exceptions is:
@begin[Display]
@TabClear()
x: G := G$create (3) @@ n
	@b[except] @^@b[when] not@us<>possible (s: @b[string]): ...
@\@b[when] failure (s: @b[string]): ...
@\@b[when] unavailable (s: @b[string]): ...
@\@b[end]
@end[Display]

Creators are described in more detail in Section @ref(sect-guardian).
