@Part(exprs, Root "manual.mss")

@Chapter(Expressions)
@Label(sect-expr)

An expression evaluates to an object in the Argus universe.
This object is said to be the @i(result) or @i(value) of the expression.
@index(expression)@index(result)@index(value)
@IndexSecondary(Primary="expression", Secondary="forms of")
Expressions are used to name the object to which they evaluate.
@IndexSecondary(Primary="object", Secondary="as value of expression")
The simplest forms of expressions are literals, variables, parameters,
equated identifiers, equate module references, procedure, iterator, and creator
names, and @b[self].  These forms directly name their result object.
More complex expressions are built up out of nested procedure calls.
The result of such an expression is the value returned by the outermost call.

@Section(Literals)

@index(literal)

Integer, real, character, string, boolean and null literals are expressions.
The type of a literal expression is the type of the object named by the literal.
For example,
@b[true] is of type @b[bool],
"abc" is of type @b[string],
etc. (see the end of Appendix @ref(apdx-syntax) for details).


@Section(Variables)

@index(variable)

Variables are identifiers that denote objects of a given type.
The type of a variable is the type given in the declaration of that variable.
An attempt to use an uninitialized variable as an expression is a
programming error and causes the guardian to crash.


@Section(Parameters)

@index(parameter)

Parameters are identifiers that denote constants supplied
when a parameterized module is instantiated (see Section @ref(sect-parm)).
The type of a parameter is the type given in the declaration of that parameter.
Type parameters cannot be used as expressions.


@Section(Equated Identifiers)

@index(equated identifier)@IndexSecondary(Primary="identifier", Secondary="equated")
@index(constant)

Equated identifiers denote constants.
The type of an equated identifier is the type of the constant
which it denotes.  Identifiers equated to types, type@us<>sets, and equate modules
cannot be used as expressions.


@Section(Equate Module References)

@index(reference)@IndexSecondary(Primary="equate module", Secondary="reference")

Equate modules provide a named set of equates (see Section @ref(sect-equmod)).
To use a name defined in an equate module as an expression,
one writes:
@begin[SyntaxDisplay]
reference $ name
@end[SyntaxDisplay]
@IndexEntry(Key=" $", Entry="$", Number)
where
@begin[SyntaxDisplay]
@TabClear()
reference @^@BNFequals() idn
 @\@BNFor() idn [ actual@us<>parm , @BNFdots() ]
 @\@BNFor() reference $ name
@end[SyntaxDisplay]
The type of a @i(reference) is the type of the constant which it denotes.
Identifiers equated to types, type@us<>sets, and equate modules
cannot be used as expressions.


@Section(Self)
@Label(sect-self)

@index(self)

The expression @b[self] evaluates to the object
(of guardian type)
corresponding to the guardian instance
within which the expression is evaluated.
A @b[self] expression may only appear textually within the body of a guardian.
See Section @ref(sect-guardian) for further discussion.


@Section(Procedure, Iterator, and Creator Names)

@index(procedure)@index(iterator)@index(creator)

Procedures and iterators may be defined either as separate modules,
or within a cluster.
Creators may only be defined within a guardian module.
Those defined as separate modules are named by expressions of the form:
@begin[SyntaxDisplay]
idn @BNFlbkt() [ actual@us<>parm , @BNFdots() ] @BNFrbkt()
@end[SyntaxDisplay]
The actual parameters of a parameterized procedure or iterator can be either
constants or type actuals (see Section @ref(sect-instantiations)).

When a procedure, iterator, or creator is defined as an operation of a type,
that type is part of the name of the routine.
The form for naming an operation of a type is:
@begin[SyntaxDisplay]
type@us<>spec $ name @BNFlbkt() [ actual@us<>parm , @BNFdots() ] @BNFrbkt()
@end[SyntaxDisplay]
@IndexEntry(Key=" $", Entry="$", Number)

The type of this expression is just the type of the named routine.


@Section(Bind)
@Label(sect-bind)

@index(bind)@index(closure)
@IndexSecondary(Primary="procedure", Secondary="bound")
@IndexSecondary(Primary="iterator", Secondary="bound")
@IndexSecondary(Primary="procedure", Secondary="closure")

Closures may be created by the @b[bind] expression:
@begin[SyntaxDisplay]
@b[bind] entity ( @BNFlbkt() bind@us<>arg , @BNFdots() @BNFrbkt() )
@end[SyntaxDisplay]
where
@begin[SyntaxDisplay]
@TabClear()
bind@us<>arg @^@BNFequals() *
 @\@BNFor() expression
@end[SyntaxDisplay]
@begin[SyntaxDisplay]
entity  @^ @BNFequals() @^reference
@\   @BNFor()@\entity @big(.) name
@\   @BNFor()@\entity [ expression ]
@\   @BNFor()@\@b(bind) entity ( @BNFlbkt() bind@us()arg , @BNFdots() @BNFrbkt() )
@\   @BNFor()@\type@us()spec $ name @BNFlbkt() [ actual@us<>parm , @BNFdots() ] @BNFrbkt()
@\   @BNFor()@\type@us()spec $ { field , @BNFdots() }
@\   @BNFor()@\type@us()spec $ [ @BNFlbkt() expression : @BNFrbkt() @BNFlbkt() expression , @BNFdots() @BNFrbkt() ]
@\   @BNFor()@\@b(up) ( expression )
@\   @BNFor()@\@b(down) ( expression )
@end[SyntaxDisplay]
An @i(entity) is a simple kind of expression that is used to prevent
syntactic ambiguity.
@index(entity)

The number of @i(bind@us<>args) must match the entity's number of formals.
A @i(bind@us<>arg) that is an asterisk (*) indicates an argument position in which
no binding is made.
If a @i(bind@us<>arg) is an expression, the type of the expression must be included
in the type of the corresponding formal.
The type of the @b[bind] expression as a whole is a routine type obtained from
the type of the @i(entity) by deleting argument positions that are bound.

The evaluation of a @b[bind] expression proceeds by first evaluating the
entity and then evaluating, from left to right, any @i(bind@us<>args)
that are expressions.
The @i(entity) may evaluate to a procedure, iterator, handler,
or creator object.
Suppose that the @i(entity) is a procedure or iterator object.
(Creator and handler bindings are discussed below.)
Then the result is formed by binding the argument objects
to the corresponding formals of the entity to form a closure;
note that the procedure or iterator is not called when the @b[bind] expression
is evaluated.
When the closure is called,
the object denoted by the @i(entity) is passed
all the bound objects and any actual arguments supplied in the call,
all in the corresponding argument positions.

For example, suppose we have:
@begin[Display]
p = @b[proc](x: T, y: @b[int], w: S) @b[returns](R) @b[signals](too@us<>big)
@end[Display]
Then
@begin[Display]
q := @b[bind] p(*, 3 + 4, *)
@end[Display]
produces a procedure whose type is
@b[proctype](@i(T), @i(S)) @b[returns](@i(R)) @b[signals](@i(too@us<>big))
and assigns it to @i(q).
A call of @i(q)(@i(a, b)) is then equivalent to
the call @i(p)(@i(a, 7, b)).

Bound routines will be stored in stable storage if they
@IndexSecondary(Primary="stable", Secondary="storage and closures")
are accessible from a stable variable (see Section @ref(sect-guardian-state)).
In this case the entity and
the @i(bind@us<>args) should denote atomic objects.

There is only one instance of a routine's @b[own] data
for each parameterization;
thus all the bindings of a routine share its @b[own] data, if any
(see Section @ref(sect-owns)).
@index(own data)
Each binding is generally a new object; thus the relevant
@i(equal) operation may treat syntactically identical bindings as distinct.
@IndexSecondary(Primary="bind", Secondary="and routine equality")
@IndexSecondary(Primary="procedure", Secondary="equality of bound procedures")
@IndexSecondary(Primary="iterator", Secondary="equality of bound iterators")
@IndexSecondary(Primary="handler", Secondary="equality of bound handlers")
@IndexSecondary(Primary="creator", Secondary="equality of bound creators")


The semantics of binding a creator or handler are similar to binding a
procedure or iterator;
the differences arise from argument transmission.
Encoding of bound argument objects happens
when the @b[bind] expression is evaluated and sharing is only preserved among
objects bound at the same time (see Section @ref(sect-transmit)).
@IndexSecondary(Primary="handler", Secondary="bound")
@IndexSecondary(Primary="creator", Secondary="bound")
In more detail,
the evaluation of a @b[bind] expression proceeds by first evaluating the
@i(entity) and then evaluating, from left to right,
any @i(bind@us<>args) that are expressions.
Then the argument objects are encoded, from left to right,
preserving sharing among
@index(encode)@IndexSecondary(Primary="encode", Secondary="with bind")
these objects.
The result is formed by binding the encoded argument objects
to the corresponding formals of the entity to form a closure.
Note that the entity is not called when the @b[bind] expression
is evaluated.

When the closure is called, first any other arguments are evaluated
and encoded (not sharing with the bound objects) and then the call to
the entity is initiated.
Decoding of the arguments at the called guardian is done
in reverse of the order of encoding;
@index(decode)@IndexSecondary(Primary="encode", Secondary="with bind")
that is, other arguments are decoded before bound arguments and
the most recently bound arguments are decoded first.
Sharing is preserved on decoding only among groups of bound arguments
and among the other arguments, not between groups.
Thereafter the call proceeds as normally.

For example, if we execute
@begin[Display]
@TabClear()
h1 := @b[bind] h(x, y, *)
h1(z)
@end[Display]
then sharing of objects between @i(x) and @i(y) will be preserved by transmission,
but sharing will not be preserved between @i(x) and @i(z) or @i(y) and @i(z).

Closures can be used in equates, provided all the expressions are
constants (see Section @ref(sect-constant-exprs)).
However, a handler cannot appear in an equate, since it is not a
constant.
@IndexSecondary(Primary="bind", Secondary="and equates")


@Section(Procedure Calls)

Procedure calls have the form:
@index(call)
@IndexSecondary(Primary="local", Secondary="call")
@IndexSecondary(Primary="call", Secondary="procedure")
@IndexSecondary(Primary="call", Secondary="expression")
@begin[SyntaxDisplay]
primary ( @BNFlbkt() expression , @BNFdots() @BNFrbkt() )
@end[SyntaxDisplay]
The @i(primary) is evaluated to obtain a procedure object,
and then the expressions are evaluated left to right to obtain the argument objects.
The procedure is called with these arguments,
and the object returned is the result of the entire expression.
For more discussion see Section @ref(sect-call).

Any procedure call @i[p(E@-{1}, ... E@-{n})] must satisfy two constraints
to be used as an expression:@ 
the type of @i(p) must be of the form:
@begin[SyntaxDisplay]
@b[proctype] (T@-{1}, ..., T@-{n}) @b[returns] (R) @b[signals] (...)
@end[SyntaxDisplay]
and the type of each expression @i(E)@-{i} must be included in
the corresponding type @i(T)@-{i}.
The type of the entire call expression is given by @i(R).


@Section(Handler Calls)
@Label(sect-handlers)

Handler calls have the form:
@index(remote call)@index(call)
@IndexSecondary(Primary="call", Secondary="remote")
@IndexSecondary(Primary="call", Secondary="handler")
@begin[SyntaxDisplay]
primary ( @BNFlbkt() expression, @BNFdots() @BNFrbkt() )
@end[SyntaxDisplay]
The @i(primary) is evaluated to obtain a handler object,
and then the expressions are evaluated left to right
to obtain the argument objects.
The handler is then called with these arguments as
discussed in Section @ref(sect-hcall).
The following expressions are examples of handler calls:
@begin[Display]
h(x)
info@us<>guard.who@us<>is@us<>user("john", "doe")
dow@us<>jones.info("XYZ Corporation")
@end[Display]

Any handler call @i[h(E@-{1}, ... E@-{n})]
must satisfy the following constraints when used as an expression.
The type of @i(h) must be of the form:
@begin[Display]
@b[handlertype] (T@-{1}, ... T@-{n}) @b[returns] (R) @b[signals] (...)
@end[Display]
and the type of each expression @i(E)@-{i} must be included in
the corresponding type @i(T)@-{i}.
The type of the entire call expression is given by @i(R).

As explained in Section @ref(sect-handlercall),
the execution of a handler call starts by creating a subaction.
Therefore an attempt to call a handler from a process that is not
running an action is a programming error
and will cause the calling guardian to crash.
This crash occurs after all of the component expressions have been evaluated.


@Section(Creator Calls)

Creator calls have the form:
@index(remote call)@index(call)
@IndexSecondary(Primary="call", Secondary="remote")
@IndexSecondary(Primary="call", Secondary="creator")
@IndexSecondary(Primary="creator call", Secondary="as expression")
@begin[SyntaxDisplay]
primary ( @BNFlbkt() expression, @BNFdots()  @BNFrbkt() ) @BNFlbkt() @@ primary @BNFrbkt()
@end[SyntaxDisplay]
The first @i(primary) is evaluated to obtain a creator object,
the argument expressions are evaluated left to right to obtain the
argument objects,
and then the @i(primary) following the at-sign (@@), if present,
is evaluated to obtain a @b[node] object.
If the @i(primary) following the at-sign is omitted, then @b[node]$@i(here)()
is used.
@IndexEntry(Key=" @@", Entry="@@", Number)
The guardian is then created at that node, and the creator called,
as discussed in Section @ref(sect-ccall).
The following are examples of creator calls:
@begin[Display]
mailer$create() @@ n
spooler[devtype]$create()
@end[Display]

A creator call @i[c(E@-{1},...,E@-{n})@@n] must satisfy the following
constraints when used as an expression.
The type of @i(c) must be of the form:
@begin[Display]
@b[creatortype] (T@-{1},...,T@-{n}) @b[returns] (R) @b[signals] (...)
@end[Display]
where each @i(T)@-{i} includes the type of the corresponding expression @i(E)@-{i}.
@i(N) must be of type @b[node].
The type of the entire call expression is given by @i(R).

As with handler calls,
an attempt to call a creator from a process that is not running an action
will cause the calling guardian to crash
after all component expressions have been evaluated.


@Section(Selection Operations)

Selection operations provide access to the individual elements or
components of a collection.
@index(fetch)@index(get)
Simple notations are provided for calling
the @i(fetch) operations of array-like types,
and the @i(get) operations of record-like types.
In addition,
these "syntactic sugarings" for selection operations may be used for
user-defined types with the appropriate properties.


@SubSection(Element Selection)


An element selection expression has the form:
@index(fetch)@IndexSecondary(Primary="selection", Secondary="of element")
@begin[SyntaxDisplay]
primary [ expression ]
@end[SyntaxDisplay]
This form is just syntactic sugar for a call of a @i(fetch) operation,
and is computationally equivalent to:
@begin[Display]
T$fetch(primary, expression)
@end[Display]
where @i(T) is the type of the @i(primary).
@i(T) must provide a procedure operation named @i(fetch),
which takes two arguments whose types include
the types of @i(primary) and @i(expression),
and which returns a single result.


@SubSection(Component Selection)


The component selection expression has the form:
@index(get)@IndexSecondary(Primary="selection", Secondary="of component")
@begin[SyntaxDisplay]
primary @big(.) name
@end[SyntaxDisplay]
This form is just syntactic sugar for a call of a @i(get@us<>name)
operation, and is computationally equivalent to:
@begin[Display]
T$get@us<>@i(name)(primary)
@end[Display]
where T is the type of @i(primary).
T must provide a procedure operation named @i(get@us<>name),
that takes one argument and returns a single result.
Of course, the type of the procedure's argument must
include the type of the @i(primary).


@Section(Constructors)


Constructors are expressions that enable users to create and initialize
@index(constructor)
sequences, arrays, atomic arrays,
structures, records, and atomic records.
There are no constructors for user-defined types.
@IndexSecondary(Primary="constructor", Secondary="none for user-defined types")
@IndexSecondary(Primary="type", Secondary="user-defined")

@SubSection(Sequence Constructors)

A sequence constructor has the form:
@IndexSecondary(Primary="constructor", Secondary="sequence")
@index(sequence)
@begin[SyntaxDisplay]
type@us<>spec $ [ @BNFlbkt() expression , @BNFdots() @BNFrbkt() ]
@end[SyntaxDisplay]
The @i(type@us<>spec) must name a sequence type: @b[sequence][@i(T)].
This is the type of the constructed sequence.
The expressions are evaluated to obtain the elements of the sequence.
They correspond (left to right) to the indexes 1, 2, 3, etc.
For a sequence of type @b[sequence][@i(T)],
the type of each element expression in the constructor must be included in @i(T).

A sequence constructor is computationally equivalent to
a sequence @i(new) operation,
followed by a number of sequence @i(addh) operations.


@SubSection(Array and Atomic Array Constructors)
@Label(sect-array-constr)

An array or atomic array constructor has the form:
@IndexSecondary(Primary="constructor", Secondary="array")
@index(array)@IndexEntry(Key="atomic_array", Entry="atomic@us<>array", Number)
@begin[SyntaxDisplay]
type@us<>spec $ [ @BNFlbkt() expression : @BNFrbkt() @BNFlbkt() expression , @BNFdots() @BNFrbkt() ]
@end[SyntaxDisplay]
The @i(type@us<>spec) must name an array or atomic array type: @b[array][@i(T)]
or @b[atomic@us<>array][@i(T)].
This is the type of the constructed array.
The optional expression preceding the colon (:) must evaluate to an integer,
and becomes the low bound of the constructed array or atomic array.
If this expression is omitted,
the low bound is 1.
The optional list of expressions is evaluated to obtain
the elements of the array.
These expressions correspond (left to right) to the indexes
@i(low@us<>bound), @i(low@us<>bound)+1, @i(low@us<>bound)+2, etc.
For an array or atomic array of type @b[array][@i(T)]
or @b[atomic@us<>array][@i(T)],
the type of each element expression in the constructor must be included in @i(T).
A constructor of the form @b[array][@i(T)]$[] has a low bound of 1 and no
elements.

An array constructor is computationally equivalent to
a @i(create) operation,
followed by a number of @i(addh) operations.


@SubSection(Structure, Record, and Atomic Record Constructors)
@Label(sect-rec-constr)

A structure, record, or atomic record constructor has the form:
@IndexSecondary(Primary="constructor", Secondary="structure")
@IndexSecondary(Primary="constructor", Secondary="record")
@index(struct)
@index(record)@IndexEntry(Key="atomic_record", Entry="atomic@us<>record", Number)
@begin[SyntaxDisplay]
type@us<>spec $ { field , @BNFdots() }
@end[SyntaxDisplay]
where
@begin[SyntaxDisplay]
@TabClear()
field @^@BNFequals() name , @BNFdots() : expression
@end[SyntaxDisplay]
Whenever a field has more than one name, it is equivalent to a sequence of
fields, one for each name.
Thus, if @i(R) = @b[record][ a:@  @b[int], b:@  @b[int], c:@  @b[int] ],
then the following two constructors are equivalent:
@begin[Display]
R${a, b: p(),   c: 9}
R${a: p(),   b: p(),   c: 9}
@end[Display]

In the following we discuss only record constructors;
structure and atomic record constructors are similar.
In a record constructor, the type specification
must name a record type: @b[record][@i(S@-{1}:T@-{1},@ ...,@ S@-{n}:T@-{n})].
This is
the type of the constructed record.  The component names
in the field list must be exactly the names @i(S@-{1},@ ...,@ S@-{n}),
although these names may appear in any order.
The expressions are evaluated left to right, and there is one evaluation per
component name even if several component names are grouped with the
same expression.
The type of the expression for component @i(S@-{i}) must be included in @i(T@-{i}).
The results of these evaluations form the components of a
newly constructed record.
This record is the value of the entire constructor expression.


@Section(Prefix and Infix Operators)


Argus allows prefix and infix notation to be used as a shorthand for
the operations listed in Table @ref(tab-operator-shorthands).
@IndexSecondary(Primary="operator", Secondary="prefix")
@IndexSecondary(Primary="operator", Secondary="infix")
@IndexSecondary(Primary="operator", Secondary="binary")
@IndexSecondary(Primary="operator", Secondary="unary")
@IndexEntry(Key=" **", Entry="**", Number)
@IndexEntry(Key=" //", Entry="//", Number)
@IndexEntry(Key=" +", Entry="+, @MinusSign(), etc.", Number)
@IndexEntry(Key=" ||", Entry="||", Number)
@IndexEntry(Key=" &", Entry="&", Number)
@IndexEntry(Key=" |", Entry="|", Number)
@IndexEntry(Key=" <", Entry="<, >, etc.", Number)
@IndexEntry(Key=" ~", Entry="~", Number)
@IndexEntry(Key=" =", Entry="=", Number)
The table shows the shorthand form and
the computationally equivalent expanded form for each operation.
For each operation,
the type @i(T) is the type of the first operand.
@begin(Table)
@Caption(Prefix and Infix Operators:  shorthands and expansions)
@Tag(tab-operator-shorthands)
@bar()
@begin[Display]
@TabClear()
@BlankSpace(.5 lines)
@TabDivide(2)
@u(Shorthand form @\Expansion)
@BlankSpace(.5 lines)
expr@-{1} ** expr@-{2} @\T$power(expr@-{1}, expr@-{2})
expr@-{1} // expr@-{2} @\T$mod(expr@-{1}, expr@-{2})
expr@-{1} / expr@-{2} @\T$div(expr@-{1}, expr@-{2})
expr@-{1} * expr@-{2} @\T$mul(expr@-{1}, expr@-{2})
expr@-{1} || expr@-{2} @\T$concat(expr@-{1}, expr@-{2})
expr@-{1} + expr@-{2} @\T$add(expr@-{1}, expr@-{2})
expr@-{1} @MinusSign() expr@-{2} @\T$sub(expr@-{1}, expr@-{2})
expr@-{1} < expr@-{2} @\T$lt(expr@-{1}, expr@-{2})
expr@-{1} <= expr@-{2} @\T$le(expr@-{1}, expr@-{2})
expr@-{1} = expr@-{2} @\T$equal(expr@-{1}, expr@-{2})
expr@-{1} >= expr@-{2} @\T$ge(expr@-{1}, expr@-{2})
expr@-{1} > expr@-{2} @\T$gt(expr@-{1}, expr@-{2})
expr@-{1} ~< expr@-{2} @\~ (expr@-{1} < expr@-{2})
expr@-{1} ~<= expr@-{2} @\~ (expr@-{1} <= expr@-{2})
expr@-{1} ~= expr@-{2} @\~ (expr@-{1} = expr@-{2})
expr@-{1} ~>= expr@-{2} @\~ (expr@-{1} >= expr@-{2})
expr@-{1} ~> expr@-{2} @\~ (expr@-{1} > expr@-{2})
expr@-{1} & expr@-{2} @\T$and(expr@-{1}, expr@-{2})
expr@-{1} @^| expr@-{2} @\T$or(expr@-{1}, expr@-{2})
@\@MinusSign()@ expr @\T$minus(expr)
@\~@ expr @\T$not(expr)
@end[Display]
@bar()
@end(Table)

Operator notation is used most heavily for the built-in types,
but may be used for user-defined types as well.
When these operations are provided for user-defined types,
they should be free of side-effects,
and they should mean roughly the same thing as they do for the built-in types.
For example,
the comparison operations should only be used for types that have
a natural partial or total order.
Usually,
the comparison operations
(@i(lt), @i(le), @i(equal), @i(ge), @i(gt))
will be of type
@begin[Display]
@b[proctype] (T, T) @b[returns] (@b[bool])
@end[Display]
the other binary operations (e.g., @i(add), @i(sub)) will be of type
@begin[Display]
@b[proctype] (T, T) @b[returns] (T) @b[signals] (...)
@end[Display]
and the unary operations will be of type
@begin[Display]
@b[proctype] (T) @b[returns] (T) @b[signals] (...)
@end[Display]


@Section(Cand and Cor)
@label(sect-cand)@label(sect-cor)

Two additional binary operators are provided.
These are the @i(conditional and) operator,
@b[cand],
and the @i(conditional or) operator,
@b[cor].
@index(cand)@index(cor)@index(bool)
@IndexSecondary(Primary="expression", Secondary="conditional")
The result of evaluating:
@begin[SyntaxDisplay]
expression@-{1} @b[cand] expression@-{2}
@end[SyntaxDisplay]
is the boolean @i(and) of expression@-{1} and expression@-{2}.
However,
if expression@-{1} is @b[false],
expression@-{2} is never evaluated.
The result of evaluating:
@begin[SyntaxDisplay]
expression@-{1} @b[cor] expression@-{2}
@end[SyntaxDisplay]
is the boolean @i(or) of expression@-{1} and expression@-{2},
but expression@-{2} is not evaluated unless expression@-{1} is @b[false].
For both @b[cand] and @b[cor],
expression@-{1} and expression@-{2} must have type @b[bool].

Because of the conditional expression evaluation involved,
uses of @b[cand] and @b[cor] are not equivalent to any procedure call.


@Section(Precedence)

When an expression is not fully parenthesized,
the proper nesting of subexpressions might be ambiguous.
The following precedence rules are used to resolve such ambiguity.
The precedence of each infix operator is given in the table below.
Higher precedence operations are performed first.
@index(precedence)@IndexSecondary(Primary="operator", Secondary="precedence")
Prefix operators always have precedence over infix operators.

@begin(Table)
@Caption(Precedence for Infix Operators)
@bar()
@begin[Display]
@TabClear()
@TabSet(1.8 inches)
@BlankSpace(.5 lines)
@u(Precedence @\Operators)
@BlankSpace(.5 lines)
5 @\**
@BlankSpace(.5 lines)
4 @\*     /     //
@BlankSpace(.5 lines)
3 @\+     @MinusSign()     ||
@BlankSpace(.5 lines)
2 @\<     <=    =     >=     >     ~<     ~<=     ~=     ~>=     ~>
@BlankSpace(.5 lines)
1 @\&     @b[cand]
@BlankSpace(.5 lines)
0 @\|     @b[cor]
@end[Display]
@bar()
@end(Table)

The order of evaluation for operators of the same precedence is left to right,
except for **,
which is right to left.
@IndexEntry(Key=" **", Entry="**", Number)


@Section(Up and Down)


There are no implicit type conversions in Argus.
Two forms of expression exist for explicit conversions.
@index(up)@index(down)
These are:
@begin[SyntaxDisplay]
@b[up] ( expression )
@b[down] ( expression )
@end[SyntaxDisplay]

@b(Up) and @b[down] may be used only within the body of a cluster operation
(see Section @ref(sect-cluster)).
@b(Up) changes the type of the expression from the representation type
of the cluster to the abstract type.
@b(Down) converts the type of the expression from the abstract type
to the representation type.
