@Part(statements, Root "manual.mss")

@Chapter(Statements)
@Label(sect-statements)

In this section,
we discuss most of the @i(statements) of Argus,
emphasizing the interaction of actions and the various kinds
of control flow statements.
We postpone discussion of the @b[signal], @b[exit], and @b[except] statements,
which are used for signalling and handling exceptions,
until Section @ref(sect-exception).
See Appendix @ref(apdx-syntax) for the complete syntax of statements.

Atomic actions allow sequences of statements to appear to be indivisible to
other actions.
Sequences of statements that are not within an action
are executed divisibly;
that is, other processes may observe intermediate states between statements.
Statements are executed for their side-effects and do not return any values.
Most statements are @i(control) statements;
@index(statement)@IndexSecondary(Primary="statement", Secondary="control")
these permit the
programmer to
create processes and to dictate how control flows in a process.
The rest are @i(simple) statements:  assignment and calls
(see Section @ref(sect-assign)).

A control statement can control
a group of equates, declarations, and statements rather than
just a single statement.
Such a group is called a @i(body),
@index(body)
and has the form:
@begin[SyntaxDisplay]
@TabClear()
body @BNFequals() @^@BNFlcurly() equate @BNFrcurly()
 @\@BNFlcurly() statement @BNFrcurly()
@end[SyntaxDisplay]
Note that statements include declarations
(see Sections @ref(sect-declaration) and Appendix @ref(apdx-syntax)).
@index(declaration)
@IndexSecondary(Primary="declaration", Secondary="as statement")
No special terminator is needed to signify the end of a body;
reserved words used in the various compound statements serve to delimit
the bodies.
The statements in a body are executed sequentially in textual order.


@Section(Calls)

A call statement may be used to call a procedure, handler, or creator.
@index(call)@IndexSecondary(Primary="call", Secondary="statement")
For procedures and handlers its form is the same as a call expression:
@begin[SyntaxDisplay]
primary ( @BNFlbkt() expression , @BNFdots() @BNFrbkt() )
@end[SyntaxDisplay]
The @i(primary) must be a procedure, or handler object.
The type of each actual @i(expression) must be included in
the type of the corresponding formal argument.
The procedure or handler may or may not return results;
if it does return results,
they are discarded.

For creator calls the syntax is similar, but one can optionally specify
the node at which the guardian is to be created:
@begin[SyntaxDisplay]
primary ( @BNFlbkt() expression , @BNFdots() @BNFrbkt() ) @BNFlbkt() @@ primary @BNFrbkt()
@end[SyntaxDisplay]
The @i(primary) following the at-sign (@@) must be of type @b[node].
@IndexEntry(Key=" @@", Entry="@@", Number)
@IndexSecondary(Primary="creator call", Secondary="as statement")

The details of procedure, handler, and creator calls are described in Sections
@ref(sect-proc-call), @ref(sect-handlercall), and @ref(sect-creatorcall).


@Section(Update Statements)

Two special statements are provided for updating components of
record and array-like objects.
In addition they may be used with user-defined types with the appropriate properties.
These statements resemble assignments syntactically,
but are actually call statements.
@index(update statement)
@IndexSecondary(Primary="statement", Secondary="update")

@SubSection(Element Update)

The element update statement has the form:
@IndexEntry(Key=" :=", Entry=":=", Number)
@IndexSecondary(Primary="statement", Secondary="element update")
@index(store operation)
@IndexEntry(Key=" []", Entry="[]", Number)
@begin[SyntaxDisplay]
primary [ expression@-{1} ] := expression@-{2}
@end[SyntaxDisplay]
This form is merely syntactic sugar for a call of a @i(store) operation;
it is equivalent to the call statement:
@begin[SyntaxDisplay]
T$store(primary, expression@-{1}, expression@-{2})
@end[SyntaxDisplay]
where @i(T) is the type of the @i(primary).
@i(T) must provide a procedure named @i(store)
that takes three arguments whose types include those of
@i(primary), @i(expression@-{1}), and @i(expression@-{2}), respectively.


@SubSection(Component Update)

The component update statement has the form:
@IndexEntry(Key=" :=", Entry=":=", Number)
@IndexSecondary(Primary="statement", Secondary="component update")
@IndexEntry(Key=" .", Entry=".", Number)
@index(set@us<> operation)
@begin[SyntaxDisplay]
primary @big(.) name := expression
@end[SyntaxDisplay]
This form is syntactic sugar for a call of a @i(set@us<>) operation
whose name is formed by attaching @i(set@us<>) to the name given.
For example, if the name is @i(f), then the statement above is
equivalent to the call statement:
@begin[Display]
T$set@us<>f(primary, expression)
@end[Display]
where @i(T) is the type of the @i(primary).
@i(T) must provide a procedure operation named @i(set@us<>f),
where @i(f) is the name given in the component update statement.
This procedure must take two arguments whose types include the types of
@i(primary) and @i(expression), respectively.


@Section(Block Statement)

The block statement permits a sequence of statements to be grouped together into
a single statement.
@index(block)@IndexSecondary(Primary="statement", Secondary="block")
Its form is:
@begin[SyntaxDisplay]
@b[begin] body @b[end]
@end[SyntaxDisplay]
Since the syntax already permits bodies inside control statements,
the main use of the block statement is to group statements together for
use with the @b[except] statement (see Section @ref(sect-exception)).


@Section(Fork Statement)
@Label(sect-fork)

A @b[fork] statement creates an autonomous process.
@index(fork)@IndexSecondary(Primary="statement", Secondary="fork")
The @b[fork] statement has the form:
@begin[SyntaxDisplay]
@b[fork] primary ( @BNFlbkt() expression, @BNFdots() @BNFrbkt() )
@end[SyntaxDisplay]
where the @i(primary) is a procedure object whose type has no
results or signals (see Section @ref(sect-procedure)).
The type of each actual @i(expression) must be included in the type
of the corresponding formal.

Execution of the @b[fork] statement starts by evaluating the primary and actual
argument expressions from left to right.
Any exceptions raised by the evaluation of the primary or the expressions
are raised by the @b[fork] statement.
If no exceptions are raised,
then a new process is created
and execution resumes after the @b[fork] statement in the old process.
The new process starts by calling the given procedure with the
argument objects.
This new process terminates if and when the procedure call does.
However, if the guardian crashes the process goes away
(like any other process).

Note that the new process does not run in an action,
although the procedure called can start a topaction if desired.
There is no mechanism for waiting for the termination of the new process.
The procedure called in a @b[fork] statement cannot return any results or
signal any exceptions.


@Section(Enter Statement)
@Label(sect-enter)

Sequential actions are created by means of the @b[enter] statement,
which has two forms:
@index(enter)@IndexSecondary(Primary="statement", Secondary="enter")
@index(action)@IndexSecondary(Primary="action", Secondary="enter statement")
@index(topaction)@index(subaction)
@begin[SyntaxDisplay]
@b[enter] @b[topaction]  body @b[end]
@end[SyntaxDisplay]
and
@begin[SyntaxDisplay]
@b[enter] @b[action]  body @b[end]
@end[SyntaxDisplay]
The @b[topaction] qualifier causes the @i(body) to execute as a new top-level action.
The @b[action] qualifier causes the @i(body) to execute as a subaction of the
current action;
@IndexSecondary(Primary="qualifier", Secondary="action, topaction")
an attempt to execute an @b[enter action] statement in a process
that is not executing an action is a programming error
and causes the guardian to crash.
When the body terminates, it does so either by committing or aborting.
Normal completion of the body results in the action committing.
@index(commit)
Statements that transfer control out of the @b[enter] statment
(@b[exit], @b[leave], @b[break], @b[continue], @b[return], @b[signal],
and @b[resignal])
normally commit the action unless are prefixed with @b[abort]
(e.g., @b[abort exit]).
@IndexSecondary(Primary="statement", Secondary="abort prefix")
@IndexSecondary(Primary="abort", Secondary="qualifier")
@IndexSecondary(Primary="qualifier", Secondary="abort")
Two-phase commit of a topaction may fail,
in which case the @b[enter topaction] statement raises
an @i(unavailable) exception.
@index(two-phase commit)@index(unavailable)


@Section(Coenter Statement)
@Label(sect-coenter)

Concurrent actions and processes are created
by means of the @b[coenter] statement:
@index(coenter)@IndexSecondary(Primary="statement", Secondary="coenter")
@index(action)@IndexSecondary(Primary="action", Secondary="coenter statement")
@index(topaction)@index(concurrency)@index(process)@index(coarm)
@index(subaction)
@begin[SyntaxDisplay]
@b[coenter] coarm @BNFlcurly() coarm @BNFrcurly() @b[end]
@end[SyntaxDisplay]
where
@begin[SyntaxDisplay]
@TabClear()

coarm @^@BNFequals() @^armtag @BNFlbkt() @b[foreach] decl , @BNFdots() @b[in] call @BNFrbkt()
 @\      body

armtag @^@BNFequals() @b[action]
 @\@BNFor() @b[topaction]
 @\@BNFor() @b[process]
@end[SyntaxDisplay]

Execution of the @b[coenter] starts by creating all of the coarm processes,
sequentially, in textual order.
A @b[foreach] clause indicates that multiple instances of the coarm
will be created.
@index(foreach)@IndexSecondary(Primary="coenter", Secondary="foreach clause")
The call in a @b[foreach] clause must be an iterator call.
At each yield of the iterator,
a new coarm process is created and the objects yielded
are assigned to newly declared variables in that process.
(This implicit assignment must be legal, see Section @ref(sect-type-incl).)
Each coarm process has separate, local instances of the variables declared
in the @b[foreach] clause.

The process executing the @b[coenter] is suspended
until after the @b[coenter] is finished.
Once all coarm processes are created,
they are started simultaneously as concurrent siblings.
Each coarm instance runs in a separate process,
and each coarm with an @i(armtag) of @b[topaction] or @b[action]
executes within a new top-level action or subaction, respectively.
An attempt to execute a @b[coenter] with a @b[process] coarm
when in an action, or to execute a @b[coenter] with an @b[action] coarm
when not in an action is an error and will cause the guardian to crash
(see Table @ref(table-coenter-legality)).
@begin(Table)
@Caption(Legality of @b[coenter] statements.)
@tag(table-coenter-legality)
@bar()
@begin(Display)
@TabClear()
@TabDivide(3)
		@\         process executing the @b[coenter] is:
armtag		@\not in an action	@\running an action

@b[action]	@\not legal		@\legal
@b[topaction]	@\legal			@\legal
@b(process)	@\legal			@\not legal
@end(Display)
@bar()
@end(Table)

A simple example making use of @b[foreach] is:
@begin[Display]
@TabClear()
@b[coenter] @b[action] @b[foreach] i: @b[int] @b[in] @b[int]$from@us<>to (1, 5)
      p (i)
      @b[end]
@end[Display]
which creates five processes, each with a local variable @i(i),
having the value 1 in the first process, 2 in the second process, and so on.
Each process runs in a newly created subaction.
This statement is legal only if the process executing it is running an action.

A coarm may terminate without terminating the entire @b[coenter]
(and sibling coarms)
either by normal completion of its body,
or by executing a @b[leave] statement (see Section @ref(sect-leave)).
The commit of a coarm declared as a topaction
may terminate in an @i(unavailable) exception if two-phase commit fails.
@index(commit)@index(unavailable)
@index(two-phase commit)
@IndexSecondary(Primary="commit", Secondary="two phase commit protocol")
Such an exception can only be handled outside the @b[coenter] statement,
and thus will force termination of the entire @b[coenter] (as explained below).

A coarm may also terminate by transferring control
outside the @b[coenter] statement.
When such a transfer of control occurs,
the following steps take place.
@begin(enumerate)
Any containing statements are terminated divisibly,
to the outermost level of the coarm,
at which point the coarm becomes the @i(controlling) coarm.

Once there is a controlling coarm,
every other active coarm will be terminated
(and abort if declared as an action)
as soon as it leaves all @b[seize] statements;
@index(abort)@IndexSecondary(Primary="action", Secondary="termination")
@IndexSecondary(Primary="action", Secondary="abortion versus seize statements")
@index(controlling coarm)@IndexSecondary(Primary="coarm", Secondary="controlling")
the controlling coarm is suspended until all other coarms terminate.

The controlling coarm then commits or aborts if declared as an action;
if declared as a topaction and the two-phase commit fails,
an @i(unavailable) exception is raised by the @b[coenter] statement.

Finally, the entire @b[coenter] terminates,
and control flow continues outside the @b[coenter] statement.
@end(enumerate)
Divisible termination implies, for instance, that
a nested topaction may commit while its parent action aborts.
@index(nested topaction)
@IndexSecondary(Primary="action", Secondary="nested topaction")
@IndexSecondary(Primary="divisible", Secondary="termination")
@index(divisible termination)
@IndexSecondary(Primary="action", Secondary="divisible termination of")

A simple example of early termination is
reading from a replicated database,
@index(replicated database example)
@IndexSecondary(Primary="example", Secondary="replicated data base")
where any copy can supply the necessary information:
@begin[Display]
@TabClear()
@b[coenter] @b[action] @b[foreach] db: database @b[in] all@us<>replicas (...)
      @b[return]( database$read (db))
      @b[end]
@end[Display]
When one of these coarms completes first,
it tries to commit itself and abort the others.
The aborts take place immediately (since there are no @b[seize] statements);
it is not necessary for the handler calls to finish.
It is possible that some descendants of an aborted coarm may
be running at remote sites when the coarm aborts;
the Argus system ensures that such orphans will be aborted before
they can make their presence known or detect that they are in fact orphans
(see Section @ref(sect-orphan)).
@index(orphan)@IndexSecondary(Primary="action", Secondary="orphan")


@Section(Leave Statement)
@Label(sect-leave)

The @b[leave] statement has the form:
@index(leave)@IndexSecondary(Primary="statement", Secondary="leave")
@begin[SyntaxDisplay]
@BNFlbkt() @b[abort] @BNFrbkt() @b[leave]
@end[SyntaxDisplay]
Executing a @b[leave] statement
terminates the innermost @b[enter] statement or @b[coenter] coarm
in which it appears.
If the process terminated is an action,
then it commits unless the @b[abort] qualifier is present,
@IndexSecondary(Primary="abort", Secondary="qualifier")
@IndexSecondary(Primary="qualifier", Secondary="abort")
@IndexSecondary(Primary="statement", Secondary="abort leave")
in which case the action aborts.
The @b[abort] qualifier can only be used textually within
an @b[enter] statement or
within an @b[action] or @b[topaction] coarm of a @b[coenter]
statement.

Note that unlike the other control flow statements,
@b[leave] does not affect concurrent siblings in a @b[coenter]
(see Section @ref(sect-coenter)).


@Section(Return Statement)

The form of the @b[return] statement is:
@index(return)@IndexSecondary(Primary="statement", Secondary="return")
@begin[SyntaxDisplay]
@BNFlbkt() @b[abort] @BNFrbkt() @b[return] @BNFlbkt() ( expression , @BNFdots() ) @BNFrbkt()
@end[SyntaxDisplay]
The @b[return] statement terminates execution of the containing routine.
If the @b[return] statement occurs in an iterator no results can be returned.
If the @b[return] statement is in a procedure, handler, or creator
the type of each @i(expression) must be included in the corresponding return type
of the routine.
The @i(expressions) (if any) are evaluated from left to right,
and the objects obtained become the results of the routine.

If no @b[abort] qualifier is present, then all containing actions
@index(abort)
@IndexSecondary(Primary="statement", Secondary="abort return")
(if any) terminated by this statement are committed.
If the @b[abort] qualifier is present, then all terminated actions are aborted.
Note that unlike the @b[leave] statement,
@b[return] will abort concurrent siblings
if executed within a coarm of a @b[coenter] statement
(see Section @ref(sect-coenter)).
The @b[abort] qualifier can only be used textually within
an @b[enter] statement, an @b[action] or @b[topaction] coarm of a @b[coenter]
statement, or the body of a handler or creator.

Within a handler or creator,
the result objects are encoded just before the activation action
terminates, but after all control flow and nested action termination.
@index(encode)
If encoding of any result object terminates in a @i(failure) exception,
then the activation action aborts and the handler or creator terminates
with the same exception.


@Section(Yield Statement)

The form of a @b[yield] statement is:
@index(yield)@IndexSecondary(Primary="statement", Secondary="yield")
@index(iterator)
@begin[SyntaxDisplay]
@b[yield] @BNFlbkt() ( expression , @BNFdots() ) @BNFrbkt()
@end[SyntaxDisplay]
The @b[yield] statement may occur only in the body of an iterator.
The effect of a @b[yield] statement is to suspend execution of the
iterator invocation,
and return control to the calling @b[for] statement or @b[foreach] clause.
The values obtained by evaluating the @i(expressions) (left to right) are passed
back to the caller.
The type of each @i(expression) must be included in the corresponding yield type of
the iterator.
Upon resumption,
execution of the
iterator continues at the statement following the @b[yield] statement.

A @b[yield] statement cannot appear textually inside an @b[enter], @b[coenter], or
@b[seize] statement.


@Section(Conditional Statement)

The form of the conditional statement is:
@index(if)@IndexSecondary(Primary="statement", Secondary="if")
@IndexSecondary(Primary="statement", Secondary="conditional")
@index(else)@index(elseif)@index(then)
@begin[SyntaxDisplay]
@TabClear()
@b[if] expression @b[then] body
    @BNFlcurly() @b[elseif] expression @b[then] body @BNFrcurly()
    @BNFlbkt() @b[else] body @BNFrbkt()
    @b[end]
@end[SyntaxDisplay]
The @i(expressions) must be of type @b[bool].
They are evaluated successively until one is found to be @b[true].
The @i(body) corresponding to the first true expression is executed,
and the execution of the @b[if] statement then terminates.
If there is an @b[else] clause and if 
none of the @i(expressions) is @b[true],
then the @i(body) in the @b[else] clause is executed.



@Section(While Statement)

The @b[while] statement has the form:
@index(while)@IndexSecondary(Primary="statement", Secondary="while")
@index(loop)
@begin[SyntaxDisplay]
@b[while] expression @b[do] body @b[end]
@end[SyntaxDisplay]
Its effect is to repeatedly execute the @i(body) as long as the @i(expression)
remains true.
The @i(expression) must be of type @b[bool].
If the value of the expression is true,
the body is executed,
and then the entire @b[while] statement is executed again.
When the expression evaluates to false,
execution of the @b[while] statement terminates.


@Section(For Statement)
@Label(sect-for-stmt)

An iterator (see Section @ref(sect-iterators))
can be called by a @b[for] statement.
@index(for)@IndexSecondary(Primary="statement", Secondary="for")
@IndexSecondary(Primary="statement", Secondary="iteration")@index(iterator)
The iterator produces a sequence of @i(items)
(where an item is a group of zero or more objects)
one item at a time;
the @i(body) of the @b[for] statement is executed for each item in the sequence.

The @b[for] statement has the form:
@begin[SyntaxDisplay]
@TabClear()
@b[for] @BNFlbkt() decl , @BNFdots() @BNFrbkt() @b[in] call @b[do] body @b[end]
@end[SyntaxDisplay]
or
@begin[SyntaxDisplay]
@TabClear()
@b[for] @BNFlbkt()  idn , @BNFdots() @BNFrbkt() @b[in] call @b[do] body @b[end]
@end[SyntaxDisplay]
The call must be an iterator call.
The second form (with an @i(idn) list) uses distinct,
previously declared variables to serve as the loop variables,
while the first form (with a @i(decl) list) form introduces new variables,
local to the @b[for] statement,
for this purpose.
In either case,
the type of each variable must include the corresponding yield type of
the called iterator (see Section @ref(sect-iterators))
and the number of variables must also match the yield type.

Execution of the @b[for] statement begins by calling
the iterator,
which either yields an item or terminates.
If it yields an item (by executing a @b[yield] statement),
its execution is temporarily suspended,
the objects in the item are assigned to the loop variables,
and the body of the @b[for] statement is executed.
The next cycle of the loop is begun by resuming
execution of the iterator after the @b[yield] statement which suspended it.
Whenever the iterator terminates,
the entire @b[for] statement terminates.


@Section(Break and Continue Statements)

The @b[break] statement has the form:
@index(break)@IndexSecondary(Primary="statement", Secondary="break")
@IndexSecondary(Primary="statement", Secondary="abort break")
@begin[SyntaxDisplay]
@BNFlbkt() @b[abort] @BNFrbkt() @b[break]
@end[SyntaxDisplay]
Its effect is to terminate execution of the
smallest @b[for] or @b[while] loop statement in which it appears.
Execution continues with the statement following that loop.

The @b[continue] statement has the form:
@index(continue)@IndexSecondary(Primary="statement", Secondary="continue")
@IndexSecondary(Primary="statement", Secondary="abort continue")
@begin[SyntaxDisplay]
@BNFlbkt() @b[abort] @BNFrbkt() @b[continue]
@end[SyntaxDisplay]
Its effect is to start the next cycle (if any)
of the smallest @b[for] or @b[while] loop statement
in which it appears.

Terminating a cycle of a loop may also terminate one or more containing
actions.
If no @b[abort] qualifier is present,
then all these terminated actions (if any) are committed.
If the @b[abort] qualifier is present,
then all of the terminated actions are aborted.
Unlike @b[leave], @b[break] and @b[continue] will abort concurrent sibling
actions when control flow leaves a containing @b[coenter]
(see Section @ref(sect-coenter)).

The @b[abort] qualifier can only be used textually within
an @b[enter] statement or an @b[action] or @b[topaction] coarm
of a @b[coenter] statement.


@Section(Tagcase Statement)
@Label(sect-tagcase)

The @b[tagcase] statement can be used to decompose @b[oneof] and @b[variant]
objects;
@index(tagcase)@IndexSecondary(Primary="statement", Secondary="tagcase")
@index(oneof)@index(variant)
@b[atomic@us<>variant] objects can be decomposed with the @b[tagtest]
or @b[tagwait] statements.
The decomposition is indivisible for @b[variant] objects;
thus,
use of the @b[tagcase] statement for variants
is not equivalent to using a conditional statement
in combination with @i(is@us<>) and @i(value@us<>) operations
(see Section @ref(variant-type-def)).

The form of the @b[tagcase] statement is:
@begin[SyntaxDisplay]
@TabClear()
@b[tagcase] expression
    tag@us<>arm @BNFlcurly() tag@us<>arm @BNFrcurly()
    @BNFlbkt() @b[others] : body @BNFrbkt()
    @b[end]
@end[SyntaxDisplay]
where
@begin[SyntaxDisplay]
@TabClear()
tag@us<>arm @^@BNFequals() @b[tag] name , @BNFdots() @BNFlbkt() ( idn: type@us<>spec ) @BNFrbkt() : body
@end[SyntaxDisplay]
The @i(expression) must evaluate to a @b[oneof] or @b[variant] object.
The tag of this object is then matched against the names on the @i(tag@us<>arms).
When a match is found,
if a declaration (@i(idn):@ @i(type@us<>spec)) exists,
the value component of the object is assigned to the
new local variable @i(idn).
The matching @i(body) is then executed;
@i(idn) is defined only in that body.
If no match is found,
the @i(body) in the @b[others] arm is executed.

In a syntactically correct @b[tagcase] statement,
the following three constraints are satisfied.
@begin(enumerate)
The type of the @i(expression) must be some @b[oneof] or @b[variant] type, @i(T).

The tags named in the @i(tag@us<>arms) must be a subset of the tags of @i(T),
and no tag may occur more than once.

If all tags of @i(T) are present,
there is no @b[others] arm;
otherwise an @b[others] arm must be present.
@end(enumerate)
On any tag@us<>arm containing a declaration (@i(idn: type@us<>spec)),
@i(type@us<>spec) must include the type(s) of @i(T)
corresponding to the tag or tags named in that @i(tag@us<>arm).


@Section(Tagtest and Tagwait Statements)
@label(sect-tagtest)

The @b[tagtest] and @b[tagwait] statements are provided for decomposing
@b[atomic@us<>variant] objects,
@IndexEntry(Key="atomic_variant", Entry="atomic@us<>variant", Number)
permitting the selection of a body based on the tag of the object
to be made indivisibly with the testing or acquisition of specified locks.


@SubSection(Tagtest Statement)

The form of the @b[tagtest] statement is:
@index(tagtest)
@IndexSecondary(Primary="statement", Secondary="tagtest")
@begin[SyntaxDisplay]
@TabClear()
@b[tagtest] expression
    atag@us<>arm @BNFlcurly() atag@us<>arm @BNFrcurly()
    @BNFlbkt() @b[others] : body @BNFrbkt()
    @b[end]
@end[SyntaxDisplay]
where
@begin[SyntaxDisplay]
@TabClear()
atag@us<>arm @^@BNFequals() tag@us<>kind name , @BNFdots() @BNFlbkt() ( idn: type@us<>spec ) @BNFrbkt() : body
tag@us<>kind @^@BNFequals() @b[tag]
 @\@BNFor() @b[wtag]
@end[SyntaxDisplay]
The @i(expression) must evaluate to an @b[atomic@us<>variant] object.
If a read lock could be obtained on the @b[atomic@us<>variant] object
by the current action,
then the tag of the object is matched against the names on the @i(atag@us<>arms);
otherwise the @b[others] arm, if present, is executed.
If a matching name is found, then the @i(tag@us<>kind) is considered.
@begin(itemize)
If the @i(tag@us<>kind) is @b[tag],
a read lock is obtained on the object and the match is complete.

If the @i(tag@us<>kind) is @b[wtag]
and the current action can obtain a write lock on the object,
then a write lock is obtained and the match is complete.
@end(itemize)
When a complete match is found,
if a declaration (@i(idn):@ @i(type@us<>spec)) exists,
the value component of the object is assigned to the
new local variable @i(idn).
The matching @i(body) is then executed;
@i(idn) is defined only in that body.
The entire matching process, including testing and acquisition of locks,
is indivisible.

If a complete match is not found,
or the object was not readable by the action,
then the @b[others] arm (if any) is executed;
if there is no @b[others] arm, the @b[tagtest] statement terminates.
If no complete match is found, then no locks are acquired.

The @b[tagtest] statement will only obtain a lock if it is possible to do so
without "waiting".
For example, suppose that the internal state of the @b[atomic@us<>variant]
indicates that some previous action acquired a conflicting lock.  This action
may have since aborted, or may have committed up to an ancestor of the action
executing the @b[tagtest], but determining such facts may require system-level
communication to other guardians.
In this case the @b[tagtest] statement may give misleading information,
because it may not indicate a match.
Apparent anomalies in testing locks may occur even if the action executing
the @b[tagtest] "knows" that the lock can be acquired,
so that the use of
@b[tagtest] to avoid deadlocks or long delays may result in excessive aborts.


@SubSection(Tagwait Statement)

The form of the @b[tagwait] statement is:
@index(tagwait)
@IndexSecondary(Primary="statement", Secondary="tagwait")
@begin[SyntaxDisplay]
@TabClear()
@b[tagwait] expression
    atag@us<>arm @BNFlcurly() atag@us<>arm @BNFrcurly()
    @b[end]
@end[SyntaxDisplay]
Execution of the @b[tagwait] statement proceeds as for the @b[tagtest] statement,
but if no complete match is found,
or if the object is not readable by the current action,
then the entire matching process is repeated (after a system-controlled delay),
until a complete match is found.
Although there is no @b[others] arm in a @b[tagwait] statement,
all tag names do not have to be listed.


@SubSection(Common Constraints)

@b[Tagtest] and @b[tagwait] statements may be executed only within an action.
An attempt to execute a @b[tagtest] or @b[tagwait] statement in a process
that is not executing an action is an error
and will cause the guardian to crash
after evaluating the @i(expression).

In a syntactically correct @b[tagtest] or @b[tagwait] statement,
the following three constraints are satisfied.
@begin(enumerate)
The type of the @i(expression) must be some @b[atomic@us<>variant] type, @i(T).

The tags named in the @i(atag@us<>arms) must be a subset of the tags of @i(T),
and no tag may occur more than once.

Finally,
on any @i(atag@us<>arm) containing a declaration (@i(idn):@ @i(type@us<>spec)),
@i(type@us<>spec) must include the type(s) specified as corresponding
in @i(T) to the tag or tags named in the @i(atag@us<>arm).
@end(enumerate)

A simple example of a @b[tagtest] statement is garbage collecting the elements
of an array that are in the @i(dequeued) state:
@begin[Display]
@TabClear()
item = @b[atomic@us<>variant][enqueued: @b[int], dequeued: @b[null]]
@b[for] i: item @b[in] @b[array][item]$elements(l) @b[do]
      @b[tagtest] i
          @b[tag] dequeued: @b[array][item]$reml(l)
          @b[others]: @b[break]
          @b[end]
      @b[end]
@end[Display]


@Section(Seize Statement)
@Label(sect-seize)

The @b[seize] statement has the form:
@index(seize)@IndexSecondary(Primary="statement", Secondary="seize")
@begin[SyntaxDisplay]
@b[seize] expression @b[do] body @b[end]
@end[SyntaxDisplay]
The @i(expression) must evaluate to a @b[mutex] object.
The executing process then attempts to gain @i(possession)
of that @b[mutex] object,
and waits to do so if necessary.
Only one process,
whether user or system defined,
may possess a given @b[mutex] object at one time.
Once the process gains possession,
the @i(body) of the @b[seize] statement is executed.
When the body terminates, possession of the @b[mutex] is released.
This includes termination of the body by statements that transfer control
out of the body.

The body of a @b[seize] statement is considered to be a critical section;
@index(critical section)
a process executing in the body of a @b[seize] statement
can only be forcibly terminated by crashing the guardian at which the
process is running.
See Section @ref(sect-atomic) for the reasons for this
and for more discussion of the use of @b[mutex].

Multiple, nested seizes of the same @b[mutex] object are allowed,
and nest properly.
A process seizing a @b[mutex] that it has already seized
will not deadlock with itself,
and possession is not really released until the outermost
seize terminates.


@Section(Pause Statement)
@Label(sect-pause)

The @b[pause] statement has the form:
@index(pause)@IndexSecondary(Primary="statement", Secondary="pause")
@begin[SyntaxDisplay]
@b[pause]
@end[SyntaxDisplay]
The @b[pause] statement must occur within an enclosing @b[seize] statement.
Its effect is to release the @b[mutex] object associated with
the smallest enclosing @b[seize] statement,
suspend execution of the process for a system-controlled period of time,
and then regain possession and continue execution.

If multiple, nested seizes on the mutex object have been performed,
@b[pause] will not actually release possession.
For example, possession is not released in the following:
@begin[Display]
@TabClear()
@b[seize] m @b[do]
      @b[seize] m @b[do]
            @b[pause]         % does not really release possession
            @b[end]
      @b[end]
@end[Display]
In general, nested seizes should be avoided when @b[pause] must be used,
and @b[pause] should be avoided when nested seizes must be used.


@Section(Terminate Statement)
@Label(sect-terminate)

The @b[terminate] statement may occur only within a guardian definition
(see Sect @ref(sect-guardian)).
The form of a @b[terminate] statement is:
@index(terminate)@IndexSecondary(Primary="statement", Secondary="terminate")
@IndexSecondary(Primary="guardian", Secondary="termination")
@IndexSecondary(Primary="termination", Secondary="of a guardian")
@begin[SyntaxDisplay]
@b[terminate]
@end[SyntaxDisplay]
When executed within an action,
its effect is to cause the eventual destruction of the guardian
after the
enclosing action commits to the top.
If a process attempts to execute
@b[terminate] while not running an action,
a topaction is created to execute the @b[terminate]
and immediately commit.

Let @i(A) be the action that is executing the @b[terminate].
The effect of this statement is the following:
@begin(enumerate)
Action @i(A) must wait until the action that created the
guardian is committed relative to @i(A).
In the case of a permanent guardian whose creation has committed to the
top there will be no wait,
but for a recently created guardian there may be a delay.

If multiple processes are attempting to execute @b[terminate] statements, at
most one at at time may proceed to the next step.

If @i(A) commits to the top,
the guardian will be destroyed at some time after topaction commit.
If some ancestor of @i(A) aborts, however,
the guardian will be unaffected.
The guardian is also unaffected
during the time
between @i(A) executing @b[terminate] and @i(A) committing to the top.
@end(enumerate)

In order to avoid serialization
@index(serializable)
problems, creation or destruction of a guardian must be synchronized with use
of that guardian via atomic objects such as the catalog
(see Section @ref(sect-catalog)).
