@Part(exceptions, Root "manual.mss")

@Chapter(Exception Handling and Exits)
@Label(sect-exception)@Label(sect-excepth)

A routine is designed to perform a certain task.
However, in some cases that task may be impossible to perform.
In such a case, instead of returning normally (which would
imply successful performance of the intended task), the
routine should notify its caller by signalling an @i(exception),
consisting of a descriptive name and zero or more result objects.
@index(exception)@index(signal)@SeeAlso(Primary="signal", Other="exception")

The exception handling mechanism consists of two parts:
signalling exceptions and handling exceptions.
Signalling is the way a routine notifies its caller of an exceptional condition;
handling is the way the caller responds to such notification.
A signalled exception always goes to the immediate caller,
and the exception must be handled in that caller.
When a routine signals an exception,
the current activation of that routine terminates and the corresponding call
(in the caller) is said to @i(raise) the exception.
@IndexSecondary(Primary="termination", Secondary="exceptional")
When a call raises an exception,
control immediately transfers to the closest applicable exception handler.
Exception handlers are attached to statements;
when execution of the exception handler completes,
control passes to the statement following the one to which the exception
handler is attached.
For brevity, exception handlers will be called "handlers" in this chapter;
these should not be confused with the remote call handlers
of guardians (see Section @ref(sect-guardian)).


@Section(Signal Statement)
@Label(sect-signal)

An exception is signalled with a @b[signal] statement, which has the form:
@index(signal)@IndexSecondary(Primary="statement", Secondary="signal")
@IndexSecondary(Primary="statement", Secondary="abort signal")
@begin[SyntaxDisplay]
@TabClear()
@BNFlbkt() @b[abort] @BNFrbkt() @b[signal] name @BNFlbkt() ( expression , @BNFdots() ) @BNFrbkt()
@end[SyntaxDisplay]
A @b[signal] statement may appear anywhere in the body of a routine.
The execution of a @b[signal] statement begins with evaluation of the expressions (if any),
from left to right,
to produce a list of @i(exception) @i(results).
@IndexSecondary(Primary="exception", Secondary="result")
The activation of the routine is then terminated.
Execution continues in the caller as described in Section @ref(sect-handle)
below.

The exception name must be one of the exception names listed in
the routine heading.
@IndexSecondary(Primary="exception", Secondary="name")
If the corresponding exception specification in the heading has the form:
@begin[Display]
name(T@-{1}, @BNFdots(), T@-{n})
@end[Display]
then there must be exactly @i(n) expressions in the @b[signal] statement,
and the type of the @i(ith) expression must be included in @i(T)@-{i}.

If no @b[abort] qualifier is present, then all containing actions
(if any) terminated by this statement are committed.
@index(commit)@index(abort)
@IndexSecondary(Primary="qualifier", Secondary="abort")
@IndexSecondary(Primary="abort", Secondary="qualifier")
@IndexSecondary(Primary="action", Secondary="termination")
If the @b[abort] qualifier is present, then all terminated actions are aborted.
Unlike the @b[leave] statement,
@b[signal] will terminate (abort) concurrent siblings
if executed within a @b[coenter] statement (see Section @ref(sect-coenter)).
The @b[abort] qualifier can only be used textually within
an @b[enter] statement, an @b[action] or @b[topaction] coarm of a @b[coenter]
statement, or the body of a handler or creator.

Within a handler or creator,
the result objects are encoded just before the activation action
terminates, but after termination of all control flow and nested actions.
If encoding of any result object terminates in a @i(failure) exception,
then the activation action aborts and the handler or creator terminates
with the @i(failure) exception.


@Section(Except Statement)
@Label(sect-handle)

When a routine activation terminates by signalling an exception,
the called routine is said to @i(raise) that exception.
@index(raise)@IndexSecondary(Primary="exception", Secondary="raise")
By attaching exception handlers to statements,
the caller can specify the action to be taken when an exception is raised
by a call within a statement or by the statement itself.
@IndexSecondary(Primary="exception", Secondary="handler")
@IndexSecondary(Primary="exception", Secondary="handling")
@SeeAlso(Primary="handler", Other="exception")

A @i(statement) with handlers attached is called an @b[except] statement,
@index(except)@IndexSecondary(Primary="statement", Secondary="except")
and has the form:
@begin[SyntaxDisplay]
@TabClear()
statement @b[except] @^@BNFlcurly() when@us<>handler @BNFrcurly()
 @\@BNFlbkt() others@us<>handler @BNFrbkt()
 @\@b[end]
@end[SyntaxDisplay]
where
@begin[SyntaxDisplay]
@TabClear()
when@us<>handler @^@BNFequals() @b[when] name , @BNFdots() @BNFlbkt() ( decl , @BNFdots() ) @BNFrbkt() : body
 @\@BNFor() @b[when] name , @BNFdots() ( * ) : body

others@us<>handler @^@BNFequals() @b[others] @BNFlbkt() ( idn : @b[string] ) @BNFrbkt() : body
@end[SyntaxDisplay]
Let @i(S) be the @i(statement) to which the handlers are attached,
and let @i(X) be the entire @b[except] statement.
Each @i(when@us<>handler) specifies one or more exception names and a @i(body).
The @i(body) is executed if an exception with one of those names is raised
by a call in @i(S).
Each of the names listed in the @i(when@us<>handlers) must be distinct.
The optional @i(others@us<>handler) is used to handle all exceptions
not explicitly named in the @i(when@us<>handlers).
The statement @i(S) can be any form of statement,
and can even be another @b[except] statement.
As an example, consider the following @b[except] statement:
@begin[Display]
@TabClear()
m.send@us<>mail(user, my@us<>message)
	@b[except] @^@b[when] no@us<>such@us<>user:  ...  % body 1
@\@b[when] unavailable, failure (s: @b[string]):  ...  % body 2
@\@b[others] (ename: @b[string]):  ...  % body 3
@\@b[end]
@end[Display]
This statement handles exceptions arising from a remote call.
If the call raises a @i(no@us<>such@us<>user) exception,
then "body 1" will be executed.
If the call raises a @i(failure) or @i(unavailable) exception,
then "body 2" will be executed.
Any other exception will be handled by "body 3."

If, during the execution of @i(S),
some call in @i(S) raises an exception @i(E),
control transfers to
the textually closest handler for @i(E) that is attached to a statement
containing the call.
When execution of the handler completes,
control passes to the statement following the one to which the handler is
attached.
Thus if the closest handler is attached to @i(S),
the statement following @i(X) is executed next.
If execution of @i(S) completes without raising an exception,
the attached handlers are not executed.

An exception raised inside a handler is treated the same as any other exception:
control passes to the closest handler for that exception.
Note that an exception raised in some handler attached to @i(S) cannot be handled
by any handler attached to @i(S);
the exception can be handled within the handler,
or it can be handled by some handler attached to a statement containing @i(X).
For example, in the following @b[except] statement:
@begin[Display]
@TabClear()
times3@us<>plus1(a)
	@b[except] @^@b[when] @^limits:
@\ @\a := a + a
@\@b[when] overflow:  ...  % body 2
@\@b[end]
@end[Display]
any @i(overflow) signal raised by the expression @i(a + a) will not be handled
in "body 2," because this @i(overflow) handler is not in an @b[except]
statement attached to the assignment statement @i(a := a + a).

We now consider the forms of exception handlers in more detail.
The form:
@begin[SyntaxDisplay]
@b[when] name , @BNFdots() @BNFlbkt() ( decl , @BNFdots() ) @BNFrbkt() : body
@end[SyntaxDisplay]
is used to handle exceptions with the given names
when the exception results are of interest.
The optional declared variables,
which are local to the handler,
are assigned the exception results before the body is executed.
Every exception potentially handled by this form must have the same number
of results as there are declared variables,
and the types of the variables must include the types of the results.
The form:
@begin[SyntaxDisplay]
@b[when] name , @BNFdots() ( * ) : body
@end[SyntaxDisplay]
handles all exceptions with the given names,
@IndexEntry(Key=" (*)", Entry="(*)", Number)
regardless of whether or not there are exception results;
any actual results are discarded.
Using this form,
exceptions with differing numbers and types of results can be handled together.

The form:
@begin[SyntaxDisplay]
@b[others] @BNFlbkt() ( idn : @b[string] ) @BNFrbkt() : body
@end[SyntaxDisplay]
is optional,
and must appear last in a handler list.
This form handles any exception not handled by other handlers in the list.
If a variable is declared,
it must be of type @b[string].
The variable,
which is local to the handler,
is assigned a lower case string representing the actual exception name;
any results are discarded.

Note that number and type of exception results
are ignored when matching exceptions to handlers;
only the names of exceptions are used.
Thus the following is illegal,
in that @b[int]$@i(div) signals @i(zero@us<>divide) without any results
(see Section @ref(int-type-def)),
but the closest handler has a declared variable:
@begin[Display]
@TabClear()
@b[begin]@^
@\y: @b[int] := 0
@\x: @^@b[int] := 3 / y
@\@\@b[except] @b[when] zero@us<>divide (z: @b[int]): @b[return] @b[end]
@\@b[end]
@\@\@b[except] @b[when] zero@us<>divide: @b[return] @b[end]
@end[Display]

A call need not be surrounded by @b[except] statements
that handle all potential exceptions.
In many cases the programmer can prove that
a particular exception will not arise;
for example, the call @b[int]$@i(div)(@i(x),@ 7)
will never signal zero@us<>divide.
However, if some call raises an exception for which there is no handler,
then the guardian crashes due to this error@Foot{
The implementation of the Argus should
log unhandled exceptions in some fashion, to aid later debugging.
During debugging, an unhandled exception would be trapped by the debugger
before the crash.}.


@Section(Resignal Statement)

A @b[resignal] statement is a syntactically abbreviated form of
exception handling:
@index(resignal)@IndexSecondary(Primary="statement", Secondary="resignal")
@IndexSecondary(Primary="abort", Secondary="qualifier")
@IndexSecondary(Primary="statement", Secondary="abort resignal")
@begin[SyntaxDisplay]
statement @BNFlbkt() @b[abort] @BNFrbkt() @b[resignal] name , @BNFdots()
@end[SyntaxDisplay]
Each name listed must be distinct,
and each must be one of the condition names listed
in the routine heading.
The @b[resignal] statement acts like an @b[except] statement containing a
handler for each condition named,
where each handler simply signals that exception with
exactly the same results.
Thus,
if the @b[resignal] clause names an exception with a specification
in the routine heading of the form:
@begin[Display]
name(T@-{1}, @BNFdots(), T@-{n})
@end[Display]
then effectively there is a handler of the form:
@begin[Display]
@b[when] name (x@-{1}: T@-{1}, @BNFdots(), x@-{n}: T@-{n}): @BNFlbkt() @b[abort] @BNFrbkt() @b[signal] name(x@-{1}, @BNFdots(), x@-{n})
@end[Display]
which has an @b[abort] qualifier if and only if the @b[resignal] statement did.
As for an explicit handler of this form,
every exception potentially handled by this implicit handler must have the
same number of results as declared in the exception specification,
and the types of the results must be included in the types listed in
the exception specification.

If no @b[abort] qualifier is present, then all containing actions
(if any) terminated by this statement are committed.
If the @b[abort] qualifier is present, then all terminated actions are aborted.
Unlike the @b[leave] statement, @b[resignal] will abort concurrent siblings
if executed within a @b[coenter] statement (see Section @ref(sect-coenter)).
The @b[abort] qualifier can only be used textually within
an @b[enter] statement, an @b[action] or @b[topaction] coarm of a @b[coenter]
statement, or the body of a handler or creator.


@Section(Exit Statement)

An @b[exit] statement has the form:
@index(exit)@IndexSecondary(Primary="statement", Secondary="exit")@index(abort)
@begin[SyntaxDisplay]
@BNFlbkt() @b[abort] @BNFrbkt() @b[exit] name @BNFlbkt() ( expression , @BNFdots() ) @BNFrbkt()
@end[SyntaxDisplay]
An @b[exit] statement is similar to a @b[signal] statement except
that where the @b[signal] statement @i(signals) an exception
to the @i(calling) routine, the @b[exit] statement @i(raises) the
exception directly in the @i(current) routine.
Thus an @b[exit] causes a transfer of control within a routine but does not
terminate the routine.
An exception raised by an @b[exit] statement
must be handled explicitly by a containing @b[except] statement
with a handler of the form:
@begin[Display]
@b[when] name , @BNFdots() @BNFlbkt() ( decl , @BNFdots() ) @BNFrbkt() : body
@end[Display]
As usual,
the types of the expressions in the @b[exit] statement must be included in
the types of the variables declared in the handler.
The handler must be an explicit one,
i.e.,
exits to the implicit handlers of @b[resignal] statements are illegal.

If no @b[abort] qualifier is present,
then all containing actions (if any) terminated by the @b[exit] statement
are committed.
If the @b[abort] qualifier is present,
then all terminated actions are aborted.
Unlike the @b[leave] statement,
@b[exit] will abort concurrent siblings
when control flow leaves a containing @b[coenter] statement
(see Section @ref(sect-coenter)).
The @b[abort] qualifier can only be used textually within
an @b[enter] statement or an @b[action] or @b[topaction] coarm
of a @b[coenter] statement.

The @b[exit] statement and the @b[signal] statement mesh nicely to form
a uniform mechanism.
The @b[signal] statement can be viewed simply as terminating a routine activation;
an exit is then performed at the point of invocation in the caller.
(Because this exit is implicit,
it is not subject to the restrictions on exits listed above.)



@Section(Exceptions and Actions)
@Label(sect-action-except)

A new action is created by a handler call, creator call, @b[enter] statement,
or @b[action] or @b[topaction] arm of a @b[coenter] statement.
@IndexSecondary(Primary="action", Secondary="and exception handling")
@IndexSecondary(Primary="abort", Secondary="and exception handling")
@IndexSecondary(Primary="commit", Secondary="and exception handling")
@IndexSecondary(Primary="exception", Secondary="action termination")
In addition,
the @b[recover] code of a guardian runs as an action.
When control flows out of an action, that action is committed unless
action is taken to prevent its committing.
To abort an action, it is necessary to qualify control flow statements
such as @b[exit], @b[signal], @b[resignal], and @b[leave] with the keyword
@b[abort]
(see Section @ref(sect-statements)).

However, there is an additional complication.
Not only will explicit termination of actions by @b[exit], @b[signal], and
@b[resignal] statements commit actions, but also @i(implicit) termination
by flow of control out of an action body when an exception raised within that
body is handled outside the action's body.
Thus, if an exception which is raised by a call within an action
is not to commit the action, then it is necessary to catch the exception
within the action.
This is particularly important when dealing with topactions.
A common desire is to catch all "unexpected" exceptions, but still have the
topaction abort.  In this case, the catch-all exception handler must be
placed inside the topaction.  However, an @i(unavailable) handler must
still be placed outside the topaction, since the two-phase commit may fail.
@index(two-phase commit)

An @b[action] or @b[topaction] coarm of a @b[coenter] statement will not
abort its concurrent siblings when it ends in
either normal completion of its body or by a @b[leave] statement.
However, if control flows otherwise out of the @b[coenter] statement from within
one of the coarms, the entire @b[coenter] is terminated as described in
Section @ref(sect-coenter).
Thus, a @b[coenter] statement should must be used carefully to ensure
the proper behavior in case of exceptions.
There may be circumstances where a separate exception
handler will have to be used for each coarm to ensure the proper behavior,
even when the exception handling is identical for each coarm.


@Section(Failure Exceptions)
@Label(sect-failure-exceptions)

Argus responds to unhandled exceptions differently than CLU.
@index(CLU)@IndexSecondary(Primary="exception", Secondary="versus CLU")
@index(failure)@IndexSecondary(Primary="failure", Secondary="versus CLU")
@index(unhandled exception)
@IndexSecondary(Primary="exception", Secondary="unhandled")
@IndexSecondary(Primary="guardian", Secondary="crash")
In CLU, an unhandled exception in some routine
causes that routine to terminate with the @i(failure) exception.
In Argus, however, an unhandled exception causes the guardian
that is running the routine to crash.
Our motivation for this change is that an unhandled exception
is typically a symptom of a programming error that cannot be
handled by the calling routine.
Furthermore, crashing the guardian limits the damage that the programming error
can cause.

Procedures and iterators in Argus no longer have an implicit @i(failure)
exception associated with them.
Instead, such a routine may list
@i(failure) explicitly in its signals clause and
@i(failure) may have
any number (and type) of exception results.
@i(Failure) should be used to indicate an unexpected
(and possibly catastrophic) failure of a lower-level abstraction, for
example,
when there is a failure in a type parameter's routines
(for instance in @i(similar) or @i(copy) operations).
Another example is when there is an unwanted side effect,
such as a bounds exception in @b[array][t]$@i(elements) caused by a mutation of
the array argument.
Various operations of the built-in types signal @i(failure)
under such circumstances.

For handlers and creators, @i(failure) is used to indicate that a remote
call has failed; thus the exception @i(failure)(@b[string])
is implicit in the type of every handler and creator
(see Section @ref(sect-handler-defs)).
When a remote call terminates with the @i(failure) exception, this means
that not only has this call failed, but that the call is unlikely to succeed
if repeated.
