@Part(transmit, Root "manual.mss")

@Chapter(Transmissibility)
@Label(sect-transmit)

A type is said to be @i(transmissible) if it defines a @b[transmit] operation
that allows the values of its objects to be
sent in messages or stored in @b[image] objects.
@index(transmissible)@index(transmit)@index(image)
@IndexSecondary(Primary="object", Secondary="transmissible")
@IndexSecondary(Primary="type", Secondary="transmissible")
Only objects of transmissible type may be used as arguments to handler calls
or creator calls.
This section describes how transmission is defined for the Argus built-in types
and for user-defined types.


@Section(The Transmit Operation)


Transmissibility is a property of a data abstraction and must
be stated in the specification of that abstraction.
A transmissible data type @i(T) can be thought of as having
an additional operation,
@begin[Display]
@b[transmit] = @b[proctype] (T) @b[returns] (T) @b[signals] (failure(@b[string]))
@end[Display]
which is called implicitly during message transmission.
Given an object,
@b[transmit] produces a different object,
which may even reside at a different guardian from the original.
The relation between the original object and the transmitted object
is defined by the specification for @b[transmit].
Although the exact specification of @b[transmit] is type dependent,
the values of the two objects will typically be equal.
(Value equality is also part of a type's specification;
see the discussion in Section 13.3 of the
@i(CLU Reference Manual)@Foot{
Liskov, B. @i(et al.), @i(CLU Reference Manual),
Lecture Notes in Computer Science, volume 114,
(Springer-Verlag, New York, 1981).}).
The @b[transmit] operation for a type thus defines a call-by-value semantics
for its objects.
@IndexSecondary(Primary="call", Secondary="by value")


@Section(Transmission for Built-in Types)

The unstructured built-in types (@b[int], @b[char], @b[bool], ...)
are transmissible,
with the exception of @b[proctype], @b[itertype], and @b[any].
The @b[transmit] operations of the unstructured types preserve value equality,
with the exception of the @b[real] type,
which, because of possible round-off errors,
guarantees only that the two values differ by very little.

The structured types (instances of @b[array], @b[struct], @b[atomic@us<>variant], ...)
are transmissible if and only if all their type parameters are transmissible.
The @b[transmit] operation for a structured type is defined in terms of the
@b[transmit] operations of the component types.
For example,
if an object @i(x) is an array containing elements of type @i(T), then
the @b[transmit] operation for @i(x) creates a new array @i(y) with the
same bounds as the original, and with elements:
@begin[Display]
y[i] = T$@b[transmit](x[i])
@end[Display]
Thus transmission of the built-in structured types will preserve
value equality only if transmission of the component types does.

The @b[transmit] operation for @b[mutex][T] acquires and holds the lock during
the transmission (actually, during the encoding, see below) of the contained
object.


@Section(Transmit for Abstract Types)
@Label(sect-transmit-abstract)


The type implemented by a cluster is transmissible if the reserved word
@b[transmit] appears in the @b[is]-list at the head of the cluster.
Unlike the other operations provided by a type,
the @b[transmit] operation cannot be called directly by users, and in fact
is not implemented directly in the cluster.
Instead,
@b[transmit] is implemented indirectly in the following way.
Each transmissible type is given a canonical representation,
called its @i(external representation type).
@index(external representation type)
@IndexSecondary(Primary="representation", Secondary="external")
@IndexSecondary(Primary="type", Secondary="external representation")
The external representation type of an abstract type @i(T) is any convenient
transmissible type @i(XT).
This type can be another abstract type if desired;
there is no requirement that @i(XT) be a built-in type.
Intuitively,
the meaning of the external representation is that values of type @i(XT) will
be used in messages to represent values of type @i(T).
The choice of external representation type is made for the abstract type
as a whole and must be used in every implementation of that type.
(There are currently no provisions for changing the external representation of
a type once it has been established in the library.)

Each implementation of the abstract type @i(T)
must provide two operations to map between values of the abstract type
and values of the external representation type.
There is an operation
@begin[Display]
encode = @b[proc] (a: T) @b[returns] (XT) @BNFlbkt() @b[signals] (failure(@b[string])) @BNFrbkt()
@end[Display]
@index(encode)
to map from @i(T) values to @i(XT) values (for sending messages) and an
operation
@begin[Display]
decode = @b[proc] (x: XT) @b[returns] (T) @BNFlbkt() @b[signals] (failure(@b[string])) @BNFrbkt()
@end[Display]
@index(decode)
to map from @i(XT) values to @i(T) values (for receiving messages).
The @b[transmit] operation for @i(T) is defined by the following identity:
@begin[Display]
T$@b[transmit] (x)  =  T$decode (XT$@b[transmit] (T$encode(x)))
@end[Display]
Intuitively, the correctness requirement for @i(encode) and @i(decode)
is that they preserve the abstract @i(T) values:  @i(encode) maps
a value of type @i(T) into the @i(XT) value that represents it,
while @i(decode) performs the reverse mapping@Foot{
Herlihy, M. and Liskov, B.,
"A Value Transmission Method for Abstract Data Types",
@i(ACM Transactions on Programming Languages and Systems),
volume 4, number 4, (Oct. 1982), pages 527-551.
}.


@i(Encode) and @i(decode) are called implicitly by the Argus system
during handler and creator calls.
If @i(encode) and @i(decode) do not appear in the cluster's @b[is]-list,
then they will be accessible to the Argus system,
but may not be named directly by users of the type.
A @i(failure) exception raised by one of these operations will be caught by
the Argus system and resignalled to the caller (see Section @ref(sect-hcall)).

An abstract type's @i(encode) and @i(decode) operations
should not cause any side effects.
This is because the number of calls to @i(encode) or @i(decode)
is unpredictable, since
arguments or results may be encoded and decoded 
several times as the system tries to
establish communication.
In addition, verifying the correctness
of transmission is easier if @i(encode) and
@i(decode) are simply transformations to and from the external
representation.

When defining a parameterized module (see Section @ref(sect-param)),
it may be necessary to require a type parameter to be transmissible.
@IndexSecondary(Primary="transmit", Secondary="for parameterized modules")
A special type restriction:
@begin[SyntaxDisplay]
@b[has] @b[transmit]
@end[SyntaxDisplay]
is provided for this purpose.
To permit instantiation only with transmissible
type parameters,
this restriction should appear in the @b[where] clause of the cluster.
Alternatively,
by placing identical @b[where] clauses in the headings of @i(encode) and
@i(decode) procedures,
one can ensure that an instantiation of the cluster is transmissible
only if the type parameters are transmissible
(see Section @ref(sect-param)).

As an example,
Figure @ref(fig-key-item-table) shows part of a cluster defining a @i(key-item table) that
stores pairs of values,
where one value (the @i(key)) is used to retrieve the other (the @i(item)).
@index(table example, transmission of)
@IndexSecondary(Primary="example", Secondary="key-item table")
The key-item table type has operations for creating empty tables,
inserting pairs, retrieving the item paired with a given key,
deleting pairs, and iterating through all key-item pairs.
The table is represented by a sorted binary tree,
and its external representation is an array of key-item pairs.
The table type is transmissible only if both type parameters are transmissible.

@begin[Figure]
@Caption[Partial implementation of table.]
@tag(fig-key-item-table)
@bar()
@begin[Display]
table@^ = @b[cluster] [key, item: @b[type]] @b[is] create, insert, lookup, allpairs, delete, @b[transmit], ...
        @b[where] key @b[has] @^lt: @b[proctype] (key, key) @b[returns] (@b[bool]),
                 @\equal: @b[proctype] (key, key) @b[returns] (@b[bool])

@\pair = @b[record][k: key,  i: item]
@\nod = @b[record][k: key,  i: item,  left, right: table[key, item]]
@\@b[rep] = @b[variant][empty: @b[null],  some: nod]
@\xrep = @b[array][pair]        %  the external representation type

@\% The internal representation is a sorted binary tree.  All pairs in the table
@\% to the left (right) of a node have keys less than (greater than) the key in
@\% that node.

@\%  ...  other operations omitted

@\encode@^ = @b[proc] @^(t: table[key, item]) @b[returns] (xrep)
@\      @\@\ @b[where] key @b[has] @b[transmit], item @b[has transmit]
@\@\xr: xrep := xrep$new()     % create an empty array
@\@\% use allpairs to extract the pairs from the tree
@\@\@b[for] @^p: pair @b[in] allpairs(t) @b[do]
@\@\@\% Add the pair to the high end of the array.
@\@\@\xrep$addh(xr, p)
@\@\@\@b[end]
@\@\@b[return](xr)
@\@\@b[end] encode

@\decode = @b[proc] (xtbl: xrep) @b[returns] (table[key, item])
@\      @\@\ @b[where] key @b[has] @b[transmit], item @b[has transmit]
@\@\t: table[key, item] := create()     % create empty table
@\@\@b[for] p: pair @b[in] xrep$elements(xr) @b[do]
@\@\@\% xrep$elements yields all elements of array xr
@\@\@\insert(t, p.key, p.item)     % enter pair in table
@\@\@\@b[end]
@\@\@b[return](t)
@\@\@b[end] decode
@\@b[end] table
@end[Display]
@bar()
@end[Figure]

@Section(Sharing)
@Label(sect-transmit-sharing)

When an object of structured built-in type is encoded and decoded,
sharing among the object's components is preserved.
@IndexSecondary(Primary="sharing", Secondary="and transmission")
@IndexSecondary(Primary="object", Secondary="sharing")
For example,
let @i(a) be an @b[array][@i(T)] object such that @i(a[i]) and @i(a[j]) refer to a
single object of type @i(T).
If @i(a2) is an @b[array][@i(T)] object created by transmitting @i(a),
then @i(a2[i]) and @i(a2[j]) also name a single object of type @i(T).

All sharing is preserved among all components
of multiple objects of built-in type
when those objects are encoded together.
Thus, sharing is preserved for objects that are arguments
of the same remote call or are results of the same remote call,
unless the arguments are encoded at different times
(see the discussion of the @b[bind] expression in Section @ref(sect-bind)).
For example,
let @i(a) and @i(b) be @b[array][@i(T)] objects such that
@i(a[i]) and @i(b[j]) refer to a single object of type @i(T).
If @i(a2) and @i(b2) are arrays created by sending @i(a) and @i(b) as
arguments in a single handler call,
then @i(a2[i]) and @i(b2[j]) also refer to a single object.


Whether an abstract type's @b[transmit] operation preserves sharing is part
of that type's specification,
but sharing should usually be preserved for abstract types.
In the key-item table implementation of Figure @ref(fig-key-item-table),
there are two types of sharing that should be preserved:
sharing of keys and items among multiple tables
sent in a single message,
and sharing of items bound to the same key in a single table.
The key-item table example shows how to implement an abstract type
whose transmission preserves sharing by choosing an external representation
type whose @b[transmit] operation preserves sharing.


Care must be taken when the references among objects to be transmitted
are cyclic, as in a circular list.
@IndexSecondary(Primary="object", Secondary="transmission of cyclic objects")
Decoding such objects can result in a @i(failure) exception unless
@i(encode) and @i(decode) are implemented in one of two ways:
@begin(enumerate)
the internal and external representation types are identical
and @i(encode) and @i(decode) return
their argument object without modifying it or accessing its components, or

the external representation object must be free of cycles.
@end(enumerate)
