@Part(atomics, Root "manual.mss")

@Chapter(Atomic Types)
@Label(sect-atomic)@Label(sect-utypes)

In Argus, atomicity is enforced by the objects shared among actions,
rather than by the individual actions themselves.
@index(atomic)@index(action)@index(commit)@index(abort)@index(synchronization)
@index(recovery)
@IndexSecondary(Primary="atomic", Secondary="type")
@IndexSecondary(Primary="type", Secondary="atomic")
@IndexSecondary(Primary="object", Secondary="atomic")
Types whose objects ensure atomicity of the actions sharing them
are called @i(atomic types);
objects of atomic types are called @i(atomic objects).
In this chapter we define what it means for a type to be atomic
and describe the mechanisms provided by Argus
to support the implementation of atomic types.

Atomicity consists of two properties:  serializability and recoverability.
@index(serializable)@index(recoverable)
An atomic type's objects must synchronize actions
to ensure that the actions are serializable.
An atomic type's objects must also recover from actions
that abort to ensure that actions appear
to execute either completely or not at all.

In addition, an atomic type must be @i(resilient):
@index(resilience)
@IndexSecondary(Primary="stable", Secondary="storage")
@SeeAlso(Primary="stable", Other="resilience")
@IndexSecondary(Primary="variable", Secondary="stable")
the type must be
implemented so that its objects can be saved on stable storage.
This ensures that the effects of an action that commits to the
top (that is, an action that commits, as do all of its ancestors)
will survive crashes.

This chapter provides definitions of the mechanisms used for 
user-defined types in Argus.  For example implementations, see 
Weihl, W. and Liskov, B.,
"Implementation of Resilient, Atomic Data Types,"
@i(ACM Transactions on Programming Languages and Systems),
volume 7, number 2 (April 1985), pages 244-269.

The remainder of this chapter is organized as follows.
In Section @ref(sect-synch) and Section @ref(sect-resilience), we present the details 
of the mechanisms.
Section @ref(sect-synch) focuses on synchronization and recovery of actions,
while Section @ref(sect-resilience) deals primarily with resilience.
In Section @ref(sect-mutex-guide), we discuss some guidelines to keep in mind
when using the mechanisms described in Section @ref(sect-synch) and 
Section @ref(sect-resilience).
In Sections @ref(sect-prescript) and @ref(sect-commute), 
we define more precisely what it means for a type
to be atomic.  Finally, in @ref(sect-multimutex), we discuss some details that
are important for user-defined atomic types that are implemented using
multiple mutexes.

@Section(Action Synchronization and Recovery)
@Label(sect-synch)

In this section we describe the mechanisms provided by Argus to
support synchronization and recovery of actions.
These mechanisms are designed specifically to support implementations
of atomic types that allow highly concurrent access to objects.

Like a non-atomic type,
an atomic type is implemented by a
cluster that defines a
representation for the objects of the type,
and an implementation for each operation
of the type in terms of that representation.
However, the implementation of an atomic type must solve
some problems that do not occur for ordinary types,
namely:  synchronizing concurrent actions,
making visible to other actions the effects of committed actions,
hiding the effects of aborted actions,
and providing resilience against crashes.

An implementation of
a user-defined atomic type must be able to find out about the
commits and aborts of actions.
In Argus, implementations use objects of built-in atomic types
for this purpose.
The representation of a user-defined atomic type
is typically a combination of atomic and non-atomic objects;
the non-atomic objects are used to hold
information that can be accessed by concurrent actions,
while the atomic objects
contain information that allows the non-atomic data
to be interpreted properly.
The built-in atomic objects can be used
to answer the following question:
did the action that caused
a particular change to the representation: 
@Begin(Itemize)
commit
(so the new information is now available to other actions),

abort
(so the change should be forgotten),
or 

is it still active
(so the information cannot be released yet)?
@End(Itemize)
The operations available on built-in atomic objects
have been extended to support this type of use;
in particular, the @i(can@us<>read) and @i(can@us<>write) operations
on atomic arrays, records and variants,
and the @b[tagtest] and @b[tagwait] statements,
are intended to be used for this purpose.
(We do not expect user-defined atomic types to support such operations,
however.)

The use of atomic objects in the representation
permits operation implementations
to discover what happened to previous actions
and to synchronize concurrent actions.
However, since part of the representation of a user-defined atomic object
may be non-atomic, the implementation also needs a way to synchronize
concurrent operation executions on that non-atomic data.

Synchronization for non-atomic data is provided by
the @b[mutex] type generator.
@index(mutex)
As discussed in Section @ref(sect-mutex),
a @b[mutex][T] object is essentially a container for an object of some
type, @i(T), that can be used to provide mutual exclusion for the
contained object.
The @b[seize] statement, described in Section @ref(sect-seize) , is used to gain
possession of a mutex object.
@index(seize)
The @b[seize] statement ensures that a process
has exclusive access to a mutex object while it executes
the body of the @b[seize].
Sometimes a process discovers after examining an object that it
needs to wait (for example, until some action completes) before
it can continue.
The @b[pause] statement, described in Section @ref(sect-pause), can be used in
the body of a @b[seize] statement to release possession of the seized
object, pause for an indeterminate amount of time, and then regain
possession.

@Section(Resilience)
@Label(sect-resilience)@index(recoverable)@index(resilience)
@SeeAlso(Primary="resilience", Other="recoverable")

If a user-defined atomic object is accessible
from the stable variables of some guardian,
it should be written to stable storage
whenever an action that modified it commits to the top.
In this section, we discuss how user-defined
atomic types can be implemented to ensure that their
objects are written to stable storage properly.
Such an implementation will make use of
some additional properties of mutex objects.

In addition to its use for synchronizing user processes, @b[mutex] 
is used
for three other functions:  notifying the
system when information needs to be written to stable storage,
defining what information is written to stable storage,
and ensuring that information is written to stable storage
in a consistent state.

To minimize the amount of information that must be
written to stable storage when actions commit,
the Argus system only copies new information to stable storage.
For built-in atomic objects, it copies accessible objects
modified or made newly accessible from a stable variable
by an action that commits to the top.
For mutex objects, it also copies newly accessible objects to stable storage.
In addition, the @b[mutex] operation
@begin[Display]
@TabClear()
changed = @b[proc] (m: @b[mutex][T])
@end[Display]
is provided for notifying the system
that an existing mutex object should be written to stable storage.
@IndexSecondary(Primary="mutex", Secondary="changed operation")
Calling this operation will cause the object to be written to stable
storage
(assuming it is accessible) by the time
the action that executed the @i(changed) operation commits to the top.
Sometime
after the action calls @i(changed),
and before its top-level ancestor commits, the system will
copy the mutex object to stable storage.
@i(Changed) must be called from a process running an action.

Mutex objects also define how much information
must be written to stable storage.
Copying a mutex object involves copying the contained object.
By choosing the proper granularity of mutex objects
the user can control how much data must be written to stable storage at a time.
For example, a large data base can be broken into partitions
that are written to stable storage independently by dividing
it among
several mutex objects.
Such a division 
can be used to
limit the amount of data written to stable storage by calling @i(changed)
only for those
partitions actually modified by a committing action.

In copying a mutex object, the system will copy all objects
reachable from it, excluding other mutex or built-in atomic objects.
A contained mutex or built-in atomic object
will be copied only if necessary; that is,
only if it is: 
@Begin(Itemize)
a mutex object for which (a descendant of) the completing
action called the @i(changed) operation,

a built-in atomic object that was modified by the action,
or 

a newly accessible object for which no stable copy exists.
@End(Itemize)
Furthermore, the component is copied independently of the containing
mutex object; they may be copied in either order (or simultaneously),
subject to the constraint that the system cannot copy a mutex object
without first gaining possession of it.

Finally, mutex objects can be used to ensure that information
is in a consistent state when it is written to stable storage.
The system will gain possession of a mutex object before writing it
to stable storage.
By making all modifications to mutex objects
inside @b[seize] statements, the user's code can prevent the system
from copying a mutex object when it is in an inconsistent state.

Some details of the effect of @i(changed) are important for atomic types
that are implemented as multiple mutexes.  These details are presented in
Section @ref(sect-multimutex).

@Section(Guidelines)
@Label(sect-mutex-guide)

This section discusses some guidelines to be followed when
implementing atomic types.  There are additional guidelines to follow
when multiple mutexes are used to implement an atomic type;
those guidelines are discussed in Section @ref(sect-multimutex).
@IndexSecondary(Primary="mutex", Secondary="guidelines")

An important concept for describing the resilience of user-defined atomic
types is synchrony.
An object is @i(synchronous)
@index(synchronous)
if it is not possible to observe that
any portion of the object is copied to stable storage
at a different time from any other portion.
For example, an object of type @b[array][@b[mutex][@b[int]]]
would not be synchronous,
because elements of the array can be copied at different times.
A type is synchronous if all of its objects are synchronous.
Whether a type is synchronous or not is an important property of its
behavior and should be stated in its specification.
The built-in atomic types are synchronous;
user-defined types must also be synchronous if they are to be atomic.

To ensure the resilience and serializability of a user-defined atomic type
independently of how it is used,
the form of the @b[rep] for an atomic type should be one of the following
possibilities.
@begin(enumerate)
The @b[rep] is itself atomic.  Note that @b[mutex] is @i(not) an atomic type.

The @b[rep] is @b[mutex][@i(t)] where @i(t) is a synchronous type.
For example, @i(t) could be atomic, or it could be the representation of
an atomic type,
if the operations on the this fictitious atomic type are coded in-line
so that the entire type behaves atomically.

The @b[rep] is an atomic collection of mutex types
containing synchronous types.

The @b[rep] is a mutable collection of synchronous types,
and objects of the representation type are never modified
after they are initialized.
That is, mutation may be used to create the initial state of such an object,
but once this has been done the object must never be modified.
@end(enumerate)

When using mutex objects, there are a few rules to remember.
First, @i(changed) must be called after the last modification
(on behalf of some action) to the contained object.
This is true because the Argus system is free to copy the mutex to
stable storage as soon as @i(changed) has been called.

In addition, @i(changed) should be called
even if the object is not accessible from the
stable variables of a guardian.
In part this rule is just an example of separation of concerns:
the implementation of the atomic type should be done independently
of any assumptions about how the object will be used.
Therefore the type should be implemented as if its objects were accessible
from the stable variables of some guardian.
However, in addition, if this rule is not followed, it is possible
that stable storage will not be updated properly.
This situation can occur if an object was accessible, then becomes
inaccessible, and later becomes accessible again.
The system guarantees that no problems arise if @i(changed) is always
called after the last modification to the object.

Mutex objects should not share data with one another,
unless the shared data is atomic or @b[mutex].
@IndexSecondary(Primary="mutex", Secondary="sharing")
@IndexSecondary(Primary="object", Secondary="sharing")
One reason for this rule is that in copying mutex objects to stable
storage Argus does not preserve this kind of sharing.

A final point about mutex objects is that it is unwise to
do any activity that is likely to take a long time inside
a @b[seize] statement.
For example, a handler call should not be done from inside
a @b[seize] statement if possible.
Also, it is unwise to wait for a lock inside a @b[seize] unless the programmer
can be certain that the lock is available or will be soon.
Otherwise, a deadlock may occur.
An example of where waiting for a lock in a nested
@b[seize] statement is safe is where all processes seize the two
mutex objects in the same order.

@Section(A Prescription for Atomicity)
@Label(sect-prescript)@Label(sect-prescription)

In this section, we discuss how to decide how much
concurrency is possible in implementing an atomic type.
In writing specifications for atomic types, we have found it helpful
to pin down the behavior of the operations, initially assuming no concurrency
and no failures, and to deal with concurrency and failures later.
In other words, we imagine that the objects will exist in an environment
in which all actions are executed sequentially, and in which actions
never abort.

Although a sequential specification of this sort does
not say anything explicit about permissible concurrency, it
does impose limits on how much concurrency can be provided.
Implementations can differ in how much concurrency is provided,
but no implementation can exceed these limits.
Therefore, it is important to understand what the limits are.

This section and the following section together provide 
a precise definition of permissible concurrency
for an atomic type.
This definition is based on two facts about Argus and the
way it supports implementations of atomic type.
First, in implementing an atomic type, it is only necessary to
be concerned about active actions.
Once an action has committed to the top, it is not possible
for it to be aborted later, and its changes to atomic objects
become visible to other actions.
So, for example,
an implementation of an atomic type needs to prevent one action
from observing the modifications of other actions that are still
active, but it does not have to prevent an action from observing
modifications by actions that have already committed.
Second, the only method available to an atomic type for controlling
the activities of actions is to delay actions while they are executing
operations of the type.
An atomic type cannot prevent an action from calling an operation,
although it can prevent that call from proceeding.
Also, an atomic type cannot prevent an action that previously finished
a call of an operation from completing either by committing or by aborting.

Given the sequential specification of the operations of a type,
these facts lead to two constraints on the concurrency permitted
among actions using the type.
While an implementation can allow no more concurrency than permitted by these
constraints,
some implementations, like that for the built-in type generator
@b[atomic@us<>array]
(see Section @ref(atomic_array-type-def)),
may allow less concurrency than permitted by their sequential specifications
and our concurrency constraints.

The first constraint is that
@begin(Itemize)
an action can observe the effects
of other actions only if those actions committed relative to the first action.
@end(Itemize)
This constraint implies that the results returned
by operations executed by one action can reflect changes made by
operations executed by other actions only if those actions committed
relative to the first action.
For example, in an atomic array @i(a), if one action performs a
@i(store)(a, 3, 7), a second (unrelated) action can receive the answer "7"
from a call of @i(fetch)(a, 3) only if the first action committed to the top.
If the first action is still active, the second action must be delayed
until the first action completes.
This first constraint supports recoverability since it ensures that effects
of aborted actions cannot be observed by other actions.
It also supports serializability, since it prevents concurrent actions
from observing one another's changes.

However, more is needed for serializability.
Thus, we have our second constraint:
@begin(Itemize)
operations executed by
one action cannot invalidate the results of operations executed by
a concurrent action.
@end(Itemize)
For example, suppose an action @i(A) executes the @i(size) operation
on an atomic array object, receiving @i(n) as the result.
Now suppose another action @i(B) is permitted to execute @i(addh).
The @i(addh) operation will increase the size of the array to
@i(n) + 1, invalidating the results of the @i(size) operation
executed by @i(A).
Since @i(A) observed the state of the array before @i(B) executed @i(addh),
@i(A) must precede @i(B) in any sequential execution of the actions
(since sequential executions must be consistent with the sequential
specifications of the objects).
Now suppose that @i(B) commits.
By assumption, @i(A) cannot be prevented from seeing the effects of @i(B).
If @i(A) observes any effect of @i(B), it will have to follow @i(B) in any
sequential execution.
Since @i(A) cannot both precede and follow @i(B) in a sequential execution,
serializability would be violated.
Thus, once @i(A) executes @i(size), an action that calls @i(addh)
must be delayed until @i(A) completes.

@Section(Commuting Operations)
@Label(sect-commute)

To state our requirements more precisely,
consider a simple situation involving two concurrent actions
each executing a single operation on a shared atomic object @i(X).
(The actions may be executing operations on other
shared objects also, but in Argus each object must individually ensure the
atomicity of the actions using it,
so we focus on the operations involving a single object.)
A fairly simple condition that guarantees serializability is the following.
Suppose @i(X) is an object of type @i(T).
@i(X) has a current state determined by the operations performed by
previously committed actions.
Suppose @i(O@-{1}) and @i(O@-{2}) are two executions of operations on @i(X)
in its current state.
(@i(O@-{1}) and @i(O@-{2}) might be executions of the same operation
or different operations.)
If @i(O@-{1}) has been executed by an action @i(A) and @i(A)
has not yet committed or aborted,
@i(O@-{2}) can be performed by a concurrent action @i(B) only if @i(O@-{1}) and
@i(O@-{2}) @i(commute):  given the current state of @i(X), the effect
(as described by the sequential specification of @i(T)) of performing
@i(O@-{1}) on @i(X) followed by @i(O@-{2}) is the same as performing
@i(O@-{2}) on @i(X) followed by @i(O@-{1}).
It is important to realize that when we say "effect" we include
both the results returned and any modifications to the state of @i(X).

The intuitive explanation of why the above condition works is as follows.
Suppose @i(O@-{1}) and @i(O@-{2}) are performed by concurrent actions
@i(A) and @i(B) at @i(X).
If @i(O@-{1}) and @i(O@-{2}) commute, then the order in which @i(A) and @i(B)
are serialized globally does not matter at @i(X).
If @i(A) is serialized before @i(B), then the local effect at @i(X) is as if
@i(O@-{1}) were performed before @i(O@-{2}), while if @i(B)
is serialized before @i(A),
the local effect is as if @i(O@-{2}) were performed before @i(O@-{1}).
But these two effects are the same since @i(O@-{1}) and @i(O@-{2}) commute.

The common
method of dividing operations into readers and writers and using
read/write locking works because it allows operations to be executed
by concurrent actions only when the operations commute.
More concurrency is possible with our commutativity condition
than with readers/writers
because the meaning of the individual operations and the arguments of
the calls can be considered.
For example, calls of the atomic array operation @i(addh) always
commute with calls of @i(addl), yet both these operations are writers.
As another example, @i(store)(@i(X, i, e@-{1})) and @i(store)(@i(X, j, e@-{2}))
commute if @i(i) @NotEquals() @i(j).

We require only that @i(O@-{1}) and @i(O@-{2}) commute when they are executed
starting in the current state.
Consider a bank account object, with operations to deposit
a sum of money, to withdraw a sum of money (with the possible result that
it signals @i(insufficient funds) if the current balance is less
than the sum requested), and to examine the current balance.
Two withdraw operations, say for amounts @i(m) and @i(n), do not
commute when the current balance is the maximum of
@i(m) and @i(n):  either operation when executed in this state
will succeed in withdrawing the requested sum, but the other operation
must signal @i(insufficient funds) if executed in the resulting state.
They do commute whenever the current balance is at least the sum
of @i(m) and @i(n).
Thus if one action has executed a withdraw operation, our condition allows
a second action to execute another withdraw operation while the first
action is still active as long as there are sufficient funds to satisfy
both withdrawal requests.

Our condition must be extended to cover two additional cases.
First, there may be more than two concurrent actions at a time.
Suppose @i(A@-{1}),...,@i(A@-{n}) are concurrent actions,
each performing a single
operation execution @i(O@-{1}),...,@i(O@-{n}), respectively, on @i(X).
(As before, the concurrent actions may be sharing other objects as well.)
@Label(sect-sharing)@IndexSecondary(Primary="sharing", Secondary="and mutex")
Since @i(A@-{1}),...,@i(A@-{n}) are permitted to be concurrent at @i(X),
there is no local control over the order in which they may appear to occur.
Therefore, all possible orders must have the same effect at @i(X).
This is true provided that all permutations of @i(O@-{1}),...,@i(O@-{n})
have the same effect when executed in the current state, where effect includes
both results obtained and modifications to @i(X).

The second extension acknowledges that actions can perform
sequences of operation executions.
Consider concurrent actions @i(A@-{1}),...,@i(A@-{n})
each performing a sequence @i(S@-{1}),...,@i(S@-{n}),
respectively, of operation executions.
This is permissible if all sequences @i(S@-{i1}),...,@i(S@-{in}),
obtained by concatenating the sequences @i(S@-{1}),...,@i(S@-{n}),
in some order, produce the same effect.
For example, suppose action @i(A) executed @i(addh) followed by @i(remh)
on an array.
This sequence of operations has no net effect on the array.
It is then permissible to allow a concurrent action @i(B) to execute
@i(size) on the same array, provided the answer returned is the size
of the array before @i(A) executed @i(addh) or after it
executed @i(remh).

Note that in requiring certain sequences of operations to have the same effect,
we are considering the effect of the operations as described by the
specification of the type.
Thus we are concerned with the abstract state of @i(X), and not with the
concrete state of its storage representation.
Therefore, we may allow two operations (or sequences of operations)
that do commute in terms of their effect on the abstract state of @i(X)
to be performed by concurrent actions,
even though they do not commute in terms
of their effect on the representation of @i(X).
This distinction between an abstraction and its implementation is crucial
in achieving reasonable performance.

It is important to realize that the constraints that are imposed by atomicity
based on the sequential specification of a type are only an upper bound
on the concurrency that an implementation may provide.
A specification may contain additional constraints that further
constrain implementations;
these constraints may be essential for showing that actions using the type
do not deadlock, or for showing other kinds of termination properties.
For example,
the specification of the built-in atomic types explicitly describes
the locking rules used by their implementations;
users of these types are guaranteed that the built-in
atomic types will not permit more concurrency than allowed by these
rules (for instance, actions writing different components of an array,
or different fields of a record, cannot do so concurrently).

@Section(Multiple Mutexes)
@Label(sect-multimutex)

Section @ref(sect-resilience) presented
a discussion of copying mutex objects 
to stable storage.  That discussion is adequate for simple implementations
that use just one mutex object.
@IndexSecondary(Primary="mutex", Secondary="multiple")
Sometimes, however, it is desirable to use more than
one mutex object in representing a user-defined
atomic object;
for example, a partitioned data base would be
implemented this way.
Such implementations require an understanding
of some details that could be
ignored when just one mutex object was used in the representation.
In particular, the implementor must understand
the effects of crashes on recovery of mutex objects
and some problems that can arise because copying
to stable storage is incremental.

The writing of mutex objects to stable storage
is recoverable for each topaction at each guardian:  either all
mutexes modified by an action at a guardian are installed
on stable storage, or none of them are.
That is,
if an action modified more than one mutex object at a guardian,
then after a crash either all those objects will be recovered,
or none of them will be.
This property
makes it easier to preserve consistency among multiple mutex objects.
However, even if an action aborts,
the new versions of mutexes it modified
may still be recovered from stable storage after a crash;
Argus only guarantees that if any new versions are recovered,
all of them will be.
Note also that when an action aborts,
it is possible that new versions will be
installed at some of the guardians but not at others.

Although mutex objects modified by a single action are
installed on stable storage as a group,
the copies are made one at a time.
Incremental copying has the following impact on programs.
The true state of an object usually includes the states of
all contained objects,
and a predicate
(the @i(representation invariant))
expressing a consistency condition on an
object state would normally constrain the states of contained objects.
For example, suppose we had an atomic type
@i(semiqueue) that allowed concurrent actions to enqueue and 
dequeue different items (that is, for a semiqueue the @i(enq) and @i(deq) 
operations commute so long as they involve different objects in the
@i(semiqueue)@Foot{
See Weihl, W. and Liskov, B.,
"Implementation of Resilient, Atomic Data Types,"
@i(ACM Transactions on Programming Languages and Systems),
volume 7, number 2 (April 1985), pages 244-269.}).
Then consider an implementation of a
@i(double-queue) that (for some reason) kept two
copies of the semiqueue and was represented by:
@begin[Display]
@b[rep] = @b[struct] [first, second:  semiqueue]
@end[Display]
where the representation invariant required that the states of the
two semiqueues be the same.
Now suppose the system is handling the top-level commit of some action @i(A)
that modified both semiqueues
contained in the double-queue, and while this is happening a
second action @i(B) is modifying those semiqueues.
Then it is possible that when the @i(first) semiqueue is written
to stable storage it contains @i(B)'s changes,
but when the @i(second) semiqueue is written to stable storage
it does not contain @i(B)'s changes.
Therefore, the information in stable storage
appears not to satisfy the representation invariant of the double-queue.

However, the representation invariant
of the double-queue really is satisfied,
for the following reason.
First note that the information in stable storage is
only of interest after a crash.
So suppose there is a crash.
Now there are two possibilities:
@begin(enumerate)
Before that crash, @i(B) also committed to the top.
In this case the data read back from stable storage is,
in fact, consistent,
since it must reflect @i(B)'s changes to both the
@i(first) and @i(second) semiqueues.

@i(B) aborted or had not yet committed before the crash.
In either case,
@i(B) aborts.
Therefore, the changes made to the @i(first) semiqueue by @i(B)
will be hidden by the semiqueue implementation:
at the abstract level, the two semiqueues @i(do) have the same state.
@end(enumerate)
The point of the above example is that if the
objects being written to stable storage are atomic,
then the fact that they are written incrementally
causes no problems.

On the other hand,
when an atomic type is implemented with a representation
consisting of several mutex objects,
the programmer must be aware that these objects are
written to stable storage incrementally,
and care must be taken to ensure that the
representation invariant is still preserved and that information
is not lost in spite of incremental writing.
If the implementation of a type requires that one mutex object (call it @i(M1))
be written to
stable storage before another (call it @i(M2)), then the write of @i(M1) must 
be contained in an action that commits to the top before the action that
writes @i(M2) is run.
