@Part(guidelines, Root "manual.mss")

@Appendix(Rules and Guidelines for Using Argus)
@Label(apdx-rules)

This appendix collects the rules and guidelines that
@index(guidelines)@index(pragmatics)@index(rules)
should be followed when programming in Argus.
Following these rules makes @b[seize] statements meaningful, actions atomic,
and so on.
In some rare cases there may be valid reasons for violating these guidelines,
but doing so greatly increases the difficulty
of building, debugging, and running the resulting system.

All of the rules listed in this appendix are based on information
appearing elsewhere in the manual.
Each rule is followed by a brief rationale,
including a reference to the section of the manual
from which it is drawn.

@Section(Serializability and Actions)

@Bullet()
Actions should share only atomic objects.

@i(Rationale:)
Actions that share non-atomic data are not necessarily serializable.
[Section @ref(sect-atomic-objects)]


@Bullet()
A subaction that aborts should not
return any information obtained from data shared with other concurrent actions.

@i(Rationale:) Returning such data may violate serializability.
[Section @ref(sect-nested-actions)]


@Bullet()
A nested topaction should be serializable before its parent.
This is true if either
@begin(enumerate)
the nested topaction performs a benevolent side effect (a change to
the state of the representation that does not affect the abstract state), or

all communication between the nested topaction and its parent is through
atomic objects.
@end(enumerate)

@i(Rationale:) Other uses may violate serializability.
[Section @ref(sect-nested-topaction)]


@Bullet()
The creation or destruction of a guardian must be synchronized with the use
of that guardian via atomic objects such as the catalog.

@i(Rationale:) Otherwise serializability may be violated.
[Section @ref(sect-terminate)]


@Section(Actions and Exceptions)

@Bullet()
If an exception raised by a call should not commit an action, the
exception must be handled within that action.

@i(Rationale:)
If an exception raised within an action body is handled outside the action,
the implicit flow of control outside of the action will commit the action.
[Section @ref(sect-action-except)]


@Section(Stable Variables)

@Bullet()
Stable variables should denote resilient data objects.

@i(Rationale:)
Only data objects 
that are (reachable from the stable variables and)
resilient are written to stable storage when a topaction commits.
(This can be ensured by having stable variables only
denote objects of an atomic type or objects protected by @b(mutex).)
Non-resilient objects stored in stable variables are only
written to stable storage when the guardian is created.
[Section @ref(sect-guardian-state)]


@Bullet()
If a bound procedure or iterator will be accessible from a stable variable,
@begin(enumerate)
the procedure or iterator being bound must be atomic and

only atomic objects should be bound as arguments.
@end(enumerate)

@i(Rationale:)
The bound procedure or iterator may be stored in stable
storage, and non-atomic data is only written to stable storage once.
[Section @ref(sect-bind)]


@Section(Transmission and Transmissibility)

@Bullet()
An abstract type's @i(encode) and @i(decode) operations
should not cause side effects.

@i(Rationale:)
The number of calls to an
@i(encode) or @i(decode) operation is unpredictable, since
arguments or results may be encoded and decoded
several times as the system tries to
establish communication.
In addition, verifying the correctness of transmission is easier
if @i(encode) and @i(decode) are simply transformations to and from the
external representation.
[Section @ref(sect-transmit-abstract)]


@Bullet()
If the naming relation among objects to be transmitted is cyclic (e.g., a
circular list) then @i(encode) and @i(decode) must be implemented in one of
two ways:
@begin(enumerate)
The internal and external representation types must
be identical, and @i(encode) and @i(decode) return their argument without
modifying or accessing it, or

The external representation object must be acyclic.
@end(enumerate)

@i(Rationale:) A circular external representation may cause decode to
fail.
[Section @ref(sect-transmit-sharing)]


@Bullet()
Objects that share other objects should be bound into a handler or
creator in the same
@b[bind] expression.

@i(Rationale:)
Sharing is only preserved among objects bound at the same time.
[Section @ref(sect-bind)]


@Section(Mutex)

@Bullet()
Mutual exclusion or atomic data should be used to
synchronize access to all shared objects.

@i(Rationale:)
In the presence of concurrency, any interleaving of indivisible events is
possible.  Without synchronization mechanisms, this concurrency will be visible
to programs, significantly complicating coding and testing.
[Section @ref(sect-assign)]


@Bullet()
All modifications to mutex objects should be made inside @b[seize]
statements.

@i(Rationale:)
The system will gain possession of a mutex object before writing it to stable
storage; thus,
seizing a mutex in order to modify it
will prevent the system from copying a mutex object
when it is in an inconsistent state.
This also prevents other processes from seeing inconsistent data
[Section @ref(sect-resilience) and Section @ref(sect-synch)]


@Bullet()
Nested seizes should be avoided when @b[pause] is used, and @b[pause]
must be avoided when nested seizes are used.

@i(Rationale:)
A @b(pause) in a nested seize does not actually release possession of the mutex
object.  [Section  @ref(sect-pause)]


@Bullet()
If an object is referred to by a mutex object, it should not be
referred to by any other object, nor should it be denoted by a variable
except when in possession of the containing mutex.

@i(Rationale:)
If an object contained in a mutex can be reached by a method other than
seizing the mutex, the mutual exclusion property of the mutex is undermined.
[Section @ref(sect-mutex)]


@Bullet()
No activity that is likely to take a long time should be performed while in
a @b[seize] statement.  In particular, programs should not make handler calls
or wait for locks on atomic objects while in possession of a mutex.

@i(Rationale:)
Waiting for a lock while in a mutex is likely to cause a deadlock with other
actions or between the action holding the mutex and the Argus system.
[Section @ref(sect-mutex-guide)]


@Bullet()
Mutex objects should not share data with one another, unless the shared data
is atomic or @b[mutex].

@i(Rationale:) Sharing of non-atomic objects between mutex objects is not
preserved when the mutexes are written to stable storage.
[Section @ref(sect-mutex-guide)]


@Bullet()
@b[Mutex][@i(t)]$@i(changed) must be called after the last modification (on
behalf of some action) to the contained object of a mutex.

@i(Rationale:)
The Argus
system is free to copy the mutex to stable storage as soon as
@b[mutex][@i(t)]$@i(changed) has been called.
Changes after the last call to
@b[mutex][@i(t)]$@i(changed) but before topaction commit may not be written to
stable storage.
[Section @ref(sect-mutex-guide)]


@Bullet()
@b[Mutex][@i(t)]$@i(changed) should be called even if the mutex object changed
is not accessible from the stable variables.

@i(Rationale:)
In a scenario where the object was accessible,
becomes inaccessible, then becomes accessible again,
it is possible that
stable storage would not be updated properly if this rule were not followed.
The system guarantees that no
problems with updating stable storage will arise if
@b[mutex][@i(t)]$@i(changed) is always
called after the last modification to the object.
[Section @ref(sect-mutex-guide)]


@Bullet()
An atomic type implemented with a representation consisting of several
mutex objects should use separate topactions to ensure that the mutexes are
written to stable storage in an order that preserves the
correctness of the representation.

@i(Rationale:)
Mutexes are written to
stable storage incrementally.
Sometimes, subtle timing problems can be
caused by incremental writing if this rule is not followed.
[Section @ref(sect-mutex-guide)]


@Section(User-Defined Atomic Objects)

@Bullet()
If an atomic object @i(X) of type @i(T)
provides operations @i(O)@-{1} and @i(O)@-{2}, and action @i(A) has executed
@i(O)@-{1} but not yet committed, then operation @i(O)@-{2} can be performed
by a concurrent
action @i(B) only if @i(O)@-{1} and @i(O)@-{2} @i(commute):
given the current state of @i(X), the
effect (as described by the sequential specification of @i(T))
of performing @i(O)@-{1},
then @i(O)@-{2} is the same as performing @i(O)@-{2}, then @i(O)@-{1}.
"Effect" includes both results
returned and the (abstract) state modified.

@i(Rationale:)
There are two concurrency constraints for user-defined atomic objects:
@begin(enumerate)
An action can observe the effects of other actions only if those actions
committed relative to the first action.

Operations executed by one action cannot invalidate the results of operations
executed by a concurrent action.
@end(enumerate)
Two operations (or sequences of operations)
that commute in their effect on the abstract state of @i(X) may be permitted to
run concurrently, even if they do not commute in their effect on the
representation of @i(X).  This distinction between an abstraction and its
implementation is crucial in achieving reasonable performance.
[Section @ref(sect-prescription)]


@Bullet()
If a user-defined atomic object is accessible from the stable variables of
some guardian, it should be written to stable storage whenever an action that
modifies it commits to the top.

@i(Rationale:)
A user-defined atomic type that is not written to stable storage on topaction
commit will not be resilient.
[Section @ref(sect-resilience)]


@Bullet()
The form of the @b[rep] for a user-defined atomic type should be
one of the following possibilities.
@begin(enumerate)
The @b[rep] is itself atomic.  Note that @b[mutex] is @i(not) an atomic type.

The @b[rep] is @b[mutex][@i(t)] where @i(t) is a synchronous type.
For example, @i(t) could be atomic, or it could be the representation of
an atomic type,
if the operations on the this fictitious atomic type are coded in-line
so that the entire type behaves atomically.

The @b[rep] is an atomic collection of mutex types
containing synchronous types.

The @b[rep] is a mutable collection of synchronous types,
and objects of the representation type are never modified
after they are initialized.
That is, mutation may be used to create the initial state of such an object,
but once this has been done the object must never be modified.
@end(enumerate)

@i(Rationale:)
In any other case it will be impossible to guarantee the resilience
or serializability of the type's objects independently of how they are used.
[Section @ref(sect-mutex-guide)]


@Section(Subordinate Where Clauses)

@Bullet()
A @i(where) clause requirement on a cluster as a whole
should be used whenever the actual parameters make
some difference in the abstraction.
For example, in a @i(set) cluster, the type parameter's @i(equal) operation
must be required by the cluster as a whole,
in order to preserve type safety and the representation invariant.

@i(Rationale:)
Argus assumes that requirements that are not placed on the cluster as a whole
do not affect the semantics of the abstraction or the representation.
[Section @ref(sect-instantiations)]
